
      PROGRAM POLFIT
C-----------------------------------------------------------------
C     Reads a number of XFOIL/MSES Mach drag-rise "polar" files, 
C     and generates a database file to be used  as a lookup table 
c     for the 2D  cdf,cdp,cm(Ma,cl,t/c)   functions.
C     The is defined on a logical box with an arbitrary number
C     of points in each direction (up to the array limits),
C     and with arbitrary spacing.  This data can be interpolated
C     with tri-linear or tri-cubic spline interpolation.
C
C     Usage:
C      % polfit polar_list_file table_file  [ Mmax ]
C
C     The input polar_list_file has the following format
C     (blank lines are ignored):
C
C     N1  N2
C
C     Par1(1 )
C     Par1(2 )
C     Par1(. )
C     Par1(. )
C     Par1(N1)
C
C     Par2(1 )
C     Par2(2 )
C     Par2(. )
C     Par2(. )
C     Par2(N2)
C
C     polarfile(1 ,1 )
C     polarfile(2 ,1 )
C     polarfile(. ,1 )
C     polarfile(. ,1 )
C     polarfile(N1,1 )
C
C     polarfile(1 ,2 )
C     polarfile(2 ,2 )
C     polarfile(. ,2 )
C     polarfile(. ,2 )
C     polarfile(N1,2 )
C
C     polarfile(1 ,. )
C     polarfile(2 ,. )
C     polarfile(. ,. )
C     polarfile(. ,. )
C     polarfile(N1,. )
C
C     polarfile(1 ,N2)
C     polarfile(2 ,N2)
C     polarfile(. ,N2)
C     polarfile(. ,N2)
C     polarfile(N1,N2)
C
C
C     See bottom of program for the output table_file format
C
C-----------------------------------------------------------------
      implicit real (a-h,m,o-z)

      include 'PINDEX.INC'
c
      parameter (nax=100,  ! number of Mach points in each Mach drag-rise polar
     &           nparx=10, ! number of cl and  t/c values
     &           isx=2 )   ! number of airfoil sides 
      parameter (npx = nparx*nparx)
c
      character*80 fnpol
      character*32 name, title
      character*16 code
      common/all_c/
     &  fnpol(npx), name(npx), title, code
c
      common/all_i/
     &  npar1, npar2,
     &  na(npx), npol, icol(npx), ilin(npx),
     &  nbl(npx),
     &  iretyp(npx),imatyp(npx)
c
      common/all_r/
     &  par1(nparx), par2(nparx),
     &  mach(npx),reyn(npx),ptrat(npx),etap(npx),
     &  acrit(isx,npx),
     &  xtrip(isx,npx),
     &  cpol(nax,iptot,npx), 
     &  cpolsd(nax,isx,jptot,npx),
     &  xtr(nax,isx,npx),
     &  version,
     &  cpolm(nax,5)
c
      parameter (imarrx=5000)
      common/spl_r/
     &  marr(imarrx),
     &  mtmp(imarrx),
     &  f(imarrx),
     &  fm(imarrx)
c
      common/spl_i/
     &  indx(imarrx)

      include 'airf.inc'


      character*80 argp1, argp2, argp3, fname
      character*128 line
      logical error
      logical lrevar

 1000 format(a)
      call getarg(1,argp1)
      call getarg(2,argp2)
      call getarg(3,argp3)
c
      lu = 1
      fname = argp1
      open(lu,file=fname,status='old')
      iline = 0
c
      call rdline(lu,line,nline,iline)
      read(line,*) nacl, natc
c
      if(nacl .gt. jadim) then
       write(*,*) 'Array overflow.  Increase jadim to', nacl
       stop
      endif

      if(natc .gt. kadim) then
       write(*,*) 'Array overflow.  Increase kadim to', natc
       stop
      endif

      do j = 1, nacl
        call rdline(lu,line,nline,iline)
        read(line,*) acl(j)
      enddo
c
      do k = 1, natc
        call rdline(lu,line,nline,iline)
        read(line,*) atc(k)
      enddo
c
      npol = nacl*natc

      do ip = 1, npol
        call rdline(lu,line,nline,iline)

        if(line .eq. 'EOF') then
         write(*,*) 'Premature end-of-file for', fname
         write(*,*) 'Expected number of polar filenames:', npol
         stop
        endif

        if(line .eq. 'ERR') then
         write(*,*) 'Read error on line number', 
     &               iline,'  of file ', fname
         stop
        endif

        fnpol(ip) = line

        lup = 9
        call polread(lup,fnpol(ip),error,
     &    nax,na(ip),cpol(1,1,ip), 
     &    reyn(ip),mach(ip),acrit(1,ip),xtrip(1,ip),ptrat(ip),etap(ip),
     &    name(ip),iretyp(ip),imatyp(ip),
     &    isx,nbl(ip),cpolsd(1,1,1,ip),
     &    code,version )
        if(error) then
         write(*,*) 'polar file read error:  ', fnpol(ip)
         stop
        endif

        do i = 2, na(ip)
          if(cpol(i,ima,ip) .eq. cpol(i-1,ima,ip)) then
           write(*,*) '* zero m interval in file', fnpol(ip)
          endif
        enddo

      enddo
c
      close(lu)
c

      if(argp3 .ne. ' ') then
       read(argp3,*) mmax
      else
       mmax = 0.
      endif


c      ip = 1
c      do ia = 1, na(ip)
c         write(*,'(1x,i3,f12.6)') ia, cpol(ia,ima,ip)
c      enddo
c      pause

c
      reyn1 = reyn(1)
      lrevar = .false.
      do ip = 2, npol
        if(reyn(ip) .ne. reyn1) then
         lrevar = .true.
        endif
      enddo

      if(lrevar) then
       write(*,*)
       write(*,*) '* warning: re different between polar files...'
       do ip = 1, npol
         write(*,*) ip, reyn(ip), '  ', fnpol(ip)(1:32)
       enddo
      endif

c---- extrapolate polars to mmax if necessary
      do ip = 1, npol
        iap = na(ip) + 1
        iao = na(ip)
        iam = na(ip) - 1
        macho = cpol(iao,ima,ip)
        machm = cpol(iam,ima,ip)
        if(macho .lt. mmax) then
         if(na(ip) .ge. nax) stop 'array overflow.  increase nax'
         do k = 1, iptot
            write(*,*) k, cpol(iam,k,ip), cpol(iao,k,ip), cpol(iap,k,ip)
           cpol(iap,k,ip) = cpol(iao,k,ip)
     &    + (cpol(iao,k,ip)-cpol(iam,k,ip))
     &     *(mmax-macho)/(macho-machm)
         enddo
         na(ip) = iap
         write(*,*) 'extrapolated polar', ip
        endif
      enddo

c-------------------------------------------------------------

c---- fill mach values array from all the polars
      imarr = 0
      do ip = 1, npol
        do ia = 1, na(ip)
          machp = cpol(ia,ima,ip)
          if(mmax .eq. 0.0 .or. machp .le. mmax+0.001) then
           imarr = imarr + 1
           nmarr = min( imarr , imarrx)
           marr(nmarr) = machp
          endif
        enddo
      enddo
      if(imarr .gt. imarrx) then 
       write(*,*) 'array overflow.  increase imarrx to', imarr
       stop
      endif

c---- set indx array for sorted mach array
      call hsort(nmarr,marr,indx)

c---- set sorted mach array
      call asort(nmarr,marr,indx,mtmp)

c---- set "stripped" mach array which has duplicates removed
      tol = 0.004
      call remd(nmarr,marr,tol)
      call remd(nmarr,marr,tol)

      do i = 1, nmarr
        write(*,'(1x,i4,f10.5)') i, marr(i)
      enddo

c---- set final mach array
      nama = nmarr
      do i = 1, nama
        ama(i) = marr(i)
      enddo

c---- set dependent variables  cdf,cdp,cm
      nafun = 3

c---- go over cl values
      do k = 1, natc

c------ go over t/c values
        do j = 1, nacl
          ip = (k-1)*nacl + j

c-------- spline cd,cdp,cm functions vs mach for this cl,t/c pair
          call spline(cpol(1,icd,ip),cpolm(1,1),cpol(1,ima,ip),na(ip))
          call spline(cpol(1,icp,ip),cpolm(1,2),cpol(1,ima,ip),na(ip))
          call spline(cpol(1,icm,ip),cpolm(1,3),cpol(1,ima,ip),na(ip))

c-------- set cdf,cpd,cm and their m derivatives
c-         at new mach values from sorted, stripped array
          do i = 1, nama
            call sdeval(ama(i),cpol(1,icd,ip),
     &              cpolm(1,1),cpol(1,ima,ip),na(ip), cd , cd_m )
            call sdeval(ama(i),cpol(1,icp,ip),
     &              cpolm(1,2),cpol(1,ima,ip),na(ip), cdp, cdp_m)
            call sdeval(ama(i),cpol(1,icm,ip),
     &              cpolm(1,3),cpol(1,ima,ip),na(ip), cm , cm_m )
            cdf   = cd   - cdp
            cdf_m = cd_m - cdp_m
             
            afun(i,j,k,1) = cdf
            afun(i,j,k,2) = cdp
            afun(i,j,k,3) = cm

            afunm(i,j,k,1) = cdf_m
            afunm(i,j,k,2) = cdp_m
            afunm(i,j,k,3) = cm_m
          enddo
        enddo
      enddo


cc---- for each mach point, perform 2d spline over cl,tc
c      do n = 1, nafun
c        do k = 1, nama
c          call spln2d(acl,atc,afun(1,1,k,n),
c     &                     afun_cl(1,1,k,n),
c     &                     afun_tc(1,1,k,n),
c     &                  afun_cl_tc(1,1,k,n),
c     &                nacl,natc,iadim,jadim )
c        enddo
c      enddo
c
cc---- now 1d-spline everything versus ma
c      do n = 1, nafun
c        do k = 1, natc
c          do j = 1, nacl
c
c            do i = 1, nama
c              f(i) = afun(i,j,k,n)
c            enddo
c            call spline(f,fm,ama,nama)
c            do i = 1, nama
c              afunm(i,j,k,n) = fm(i)
c            enddo
c
c            do i = 1, nama
c              f(i) = afun_cl(i,j,k,n)
c            enddo
c            call spline(f,fm,ama,nama)
c            do i = 1, nama
c              afunm_cl(i,j,k,n) = fm(i)
c            enddo
c
c            do i = 1, nama
c              f(i) = afun_tc(i,j,k,n)
c            enddo
c            call spline(f,fm,ama,nama)
c            do i = 1, nama
c              afunm_tc(i,j,k,n) = fm(i)
c            enddo
c
c            do i = 1, nama
c              f(i) = afun_cl_tc(i,j,k,n)
c            enddo
c            call spline(f,fm,ama,nama)
c            do i = 1, nama
c              afunm_cl_tc(i,j,k,n) = fm(i)
c            enddo
c
c          enddo
c        enddo
c      enddo


c-------------------------------------------------------------
c---- write output file (customize as needed)
c
      lu = 2
      fname = argp2
      open(lu,file=fname,status='unknown')
      rewind(lu)
c
      write(lu,2100) nama, nacl, natc, nafun

 2100 format(1x,4i5)
c
      do i = 1, nama
        write(lu,2200) ama(i)
      enddo

      do j = 1, nacl
        write(lu,2200) acl(j)
      enddo

      do k = 1, natc
        write(lu,2200) atc(k)
      enddo

 2200 format(1x,8g14.6)
c
      write(lu,2200) reyn1
c
      do k = 1, natc
        do j = 1, nacl
          do i = 1, nama
            write(lu,2200) 
     &        (afun(i,j,k,n), n = 1, nafun)
           enddo
        enddo
      enddo
c
cc---- also write out the derivatives w.r.t. Mach
c      do k = 1, natc
c        do j = 1, nacl
c          do i = 1, nama
c            write(lu,2200) 
c     &        (afunm(i,j,k,n), n = 1, nafun)
c           enddo
c        enddo
c      enddo
c
      close(lu)


      stop


c---- dump data into fort.* files for plotting
      do k = 1, natc
      lu  = 40 + k
      lup = 30 + k
      write(*,*) k, 'tc =', atc(k), lu

      do j = 1, nacl
        do i = 1, nama
          write(lu,2200) acl(j), ama(i),
     &        (afun(i,j,k,n), n = 1, nafun),
     &        (afunm(i,j,k,n), n = 1, nafun)
        enddo
        write(lu,*)

c        ip = (j-1)*natc + i
c        do ia = 1, na(ip)
c          write(lup,2200) cpol(ia,icl,ip), cpol(ia,ima,ip),
c     &        cpol(ia,icd,ip)-cpol(ia,icp,ip),
c     &        cpol(ia,icp,ip),
c     &        cpol(ia,icm,ip)
c        enddo
c        write(lup,*)

      enddo
      enddo


      do j = 1, nacl
      lu  = 50 + j
      lup = 60 + j
      write(*,*) j, 'cl =', Acl(j), lu

      do k = 1, nAtc
        do i = 1, nAMa
          write(lu,2200) Atc(k), AMa(i),
     &        (Afun(i,j,k,n), n = 1, nAfun),
     &        (AfunM(i,j,k,n), n = 1, nAfun)
        enddo
        write(lu,*)


c       ip = (j-1)*nAtc + i
c       do ia = 1, na(ip)
c         write(lup,2200) cpol(ia,icl,ip), cpol(ia,ima,ip),
c    &        cpol(ia,icd,ip)-cpol(ia,icp,ip),
c    &        cpol(ia,icp,ip),
c    &        cpol(ia,icm,ip)
c       enddo
c       write(lup,*)

      enddo
      enddo

      i = 18
      j = 4
      k = 5
      do k = 1, natc
        write(*,*) atc(k), afun(i,j,k,1)+afun(i,j,k,2)
      enddo

      stop
      end ! polfit


