
      PROGRAM POLFIT
C----------------------------------------------------------------------------
C     NOTE:  
C     This is an older version of POLFIT, used with TASOPT 2.15 and earlier.
C     It allows a variable number of Mach points for each cl,t/c pair.  
C     Gives more flexibility for point clustering to resolve the sharp
C     drag rise, but is considerably slower to interrogate.
C
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C
C     Reads a number of XFOIL/MSES polar files, 
C     and generates a database file to be used as 
C     a lookup table for a cd(p0,p1,p2) function
C     of three parameters p0, p1, p2.
C     The p1,p2 parameters form a N1xN2 logically
C     rectangular grid, while the number of p0
C     parameters at each such grid point is arbitrary.
C
C     Usage:
C      % polfit polar_list_file table_file
C
C     The input polar_list_file has the following format
C     (blank lines are ignored):
C
C     N1  N2
C
C     Par1(1 )
C     Par1(2 )
C     Par1(. )
C     Par1(. )
C     Par1(N1)
C
C     Par2(1 )
C     Par2(2 )
C     Par2(. )
C     Par2(. )
C     Par2(N2)
C
C     polarfile(1 ,1 )
C     polarfile(2 ,1 )
C     polarfile(. ,1 )
C     polarfile(. ,1 )
C     polarfile(N1,1 )
C
C     polarfile(1 ,2 )
C     polarfile(2 ,2 )
C     polarfile(. ,2 )
C     polarfile(. ,2 )
C     polarfile(N1,2 )
C
C     polarfile(1 ,. )
C     polarfile(2 ,. )
C     polarfile(. ,. )
C     polarfile(. ,. )
C     polarfile(N1,. )
C
C     polarfile(1 ,N2)
C     polarfile(2 ,N2)
C     polarfile(. ,N2)
C     polarfile(. ,N2)
C     polarfile(N1,N2)
C
C
C     See bottom of program for the format of the output table_file
C----------------------------------------------------------------------------
C
      INCLUDE 'PINDEX.INC'
C
      PARAMETER (NAX=100, NPARX=10, NFX=128, ISX=2)
      PARAMETER (NPX = NPARX*NPARX)
C
      CHARACTER*80 FNPOL
      CHARACTER*32 NAME, TITLE
      CHARACTER*16 CODE
      COMMON/ALL_C/
     &  FNPOL(NPX), NAME(NPX), TITLE, CODE
C
      COMMON/ALL_I/
     &  NPAR1, NPAR2,
     &  NA(NPX), NPOL, ICOL(NPX), ILIN(NPX),
     &  NBL(NPX),
     &  IRETYP(NPX),IMATYP(NPX)
C
      COMMON/ALL_R/
     &  PAR1(NPARX), PAR2(NPARX),
     &  MACH(NPX),REYN(NPX),PTRAT(NPX),ETAP(NPX),
     &  ACRIT(ISX,NPX),
     &  XTRIP(ISX,NPX),
     &  CPOL(NAX,IPTOT,NPX), 
     &  CPOLSD(NAX,ISX,JPTOT,NPX),
     &  XTR(NAX,ISX,NPX),
     &  VERSION
C
C
C
      CHARACTER*80 ARGP1, ARGP2, FNAME
      CHARACTER*128 LINE
      LOGICAL ERROR
      LOGICAL LBLANK, LREVAR

 1000 FORMAT(A)
      CALL GETARG(1,ARGP1)
      CALL GETARG(2,ARGP2)
C
      LU = 1
      FNAME = ARGP1
      OPEN(LU,FILE=FNAME,STATUS='OLD')
      ILINE = 0
C
      CALL RDLINE(LU,LINE,NLINE,ILINE)
      READ(LINE,*) NPAR1, NPAR2
C
      DO IPAR1 = 1, NPAR1
        CALL RDLINE(LU,LINE,NLINE,ILINE)
        READ(LINE,*) PAR1(IPAR1)
      ENDDO
C
      DO IPAR2 = 1, NPAR2
        CALL RDLINE(LU,LINE,NLINE,ILINE)
        READ(LINE,*) PAR2(IPAR2)
      ENDDO
C
      NPOL = NPAR1*NPAR2
      DO IP = 1, NPOL
        CALL RDLINE(LU,LINE,NLINE,ILINE)
        FNPOL(IP) = LINE
C
        LUP = 9
        CALL POLREAD(LUP,FNPOL(IP),ERROR,
     &    NAX,NA(IP),CPOL(1,1,IP), 
     &    REYN(IP),MACH(IP),ACRIT(1,IP),XTRIP(1,IP),PTRAT(IP),ETAP(IP),
     &    NAME(IP),IRETYP(IP),IMATYP(IP),
     &    ISX,NBL(IP),CPOLSD(1,1,1,IP),
     &    CODE,VERSION )
        IF(ERROR) THEN
         WRITE(*,*) 'Polar file READ error:  ', FNPOL(IP)
         STOP
        ENDIF

        DO I = 2, NA(IP)
          IF(CPOL(I,IMA,IP) .EQ. CPOL(I-1,IMA,IP)) THEN
           WRITE(*,*) '* zero M interval in file', FNPOL(IP)
          ENDIF
        ENDDO

      ENDDO
C
      CLOSE(LU)
C
      NAMAX = 0
      DO IP = 1, NPOL
        NAMAX = MAX(NAMAX,NA(IP))
      ENDDO
C
      REYN1 = REYN(1)
      LREVAR = .FALSE.
      DO IP = 2, NPOL
        IF(REYN(IP) .NE. REYN1) THEN
         LREVAR = .TRUE.
        ENDIF
      ENDDO

      IF(LREVAR) THEN
       WRITE(*,*)
       WRITE(*,*) '* WARNING: Re different between polar files...'
       DO IP = 1, NPOL
         WRITE(*,*) IP, REYN(IP), '  ', FNPOL(IP)(1:32)
       ENDDO
      ENDIF

C-------------------------------------------------------------
C---- Write output file (customize as needed)
C
C---- number of data columns written out
      NCOL = 5
C
      LU = 2
      FNAME = ARGP2
      OPEN(LU,FILE=FNAME,STATUS='UNKNOWN')
      REWIND(LU)
C
      WRITE(LU,2100) NPAR1, NPAR2, NCOL, NAMAX
 2100 FORMAT(1X,4I5)
C
      DO IPAR1 = 1, NPAR1
        WRITE(LU,2200) PAR1(IPAR1)
      ENDDO
      DO IPAR2 = 1, NPAR2
        WRITE(LU,2200) PAR2(IPAR2)
      ENDDO
 2200 FORMAT(1X,8G14.6)
C
      WRITE(LU,2200) REYN1
C
      DO IPAR2 = 1, NPAR2
        DO IPAR1 = 1, NPAR1
          IP = (IPAR2-1)*NPAR1 + IPAR1
          WRITE(LU,2100) NA(IP)
          DO I = 1, NA(IP)
            RM  = CPOL(I,IMA,IP)
            CM  = CPOL(I,ICM,IP)
            CD  = CPOL(I,ICD,IP)
            CDP = CPOL(I,ICP,IP)
            CDW = CPOL(I,ICW,IP)
            CDF = CD - CDP
            CDW = MAX( CDW , 0.0 )
            WRITE(LU,2200) RM, CDF, CDP, CDW, CM
          ENDDO
        ENDDO
      ENDDO
C
      CLOSE(LU)
C
      STOP
      END ! POLFIT



      LOGICAL FUNCTION LBLANK(LINE)
C---------------------------------------------------------------------
C     Returns T if the string LINE contains only blanks and/or tabs.
C     Returns F otherwise.
C---------------------------------------------------------------------
      CHARACTER*(*) LINE
      CHARACTER*1 BLANK, TAB
C
      BLANK = ' '
      TAB = CHAR(9)
C
      LBLANK = .FALSE.
      NLINE = LEN(LINE)
      DO K = 1, NLINE
        IF(LINE(K:K) .NE. BLANK .AND. LINE(K:K) .NE. TAB) RETURN
      ENDDO
      LBLANK = .TRUE.
C
      RETURN
      END ! LBLANK
