c
c    2-D Spline Package.
c    Interpolates a function f(x,y) over a rectangular x,y region.
c
c    Requires 1-D spline package
c                                              Mark Drela
c                                              1989
c
c    Usage:
c
cC---- fill X(i), Y(j), F(i,j) arrays
c      X(i) = ...
c      Y(j) = ...
c      F(i,j) = ...
c
cC---- calculate spline coefficients for f(x,y) from discrete F(i,j) values
c      CALL SPLN2D(X,Y,F,FX,FY,FXY,NI,NJ,ID,JD)
c
cC---- evaluate splined f(x,y) and its derivatives (FF,FFX...)
cC-    at any number of x,y points XX,YY
c      CALL SEVL2D(XX1,YY1,X,Y,F,FX,FY,FXY,NI,NJ,ID,JD,
c     &            FF1,FFX1,FFY1,FFXX1,FFYY1,FFXY1)
c      CALL SEVL2D(XX2,YY2,X,Y,F,FX,FY,FXY,NI,NJ,ID,JD,
c     &            FF2,FFX2,FFY2,FFXX2,FFYY2,FFXY2)
c      CALL SEVL2D(XX3,YY3,X,Y,F,FX,FY,FXY,NI,NJ,ID,JD,
c     &            FF3,FFX3,FFY3,FFXX3,FFYY3,FFXY3)
c



      SUBROUTINE SPLN2D(X,Y,F,FX,FY,FXY,NI,NJ,ID,JD)
      DIMENSION X(ID),Y(JD),F(ID,JD)
      DIMENSION FX(ID,JD), FY(ID,JD), FXY(ID,JD)
C-------------------------------------------------------
C     Calculates spline coefficients for F(X,Y).        |
C     Natural (zero 3rd deriv) end conditions are used. |
C     To evaluate the spline at some values of X,Y,     |
C     use SEVL2D.                                       |
C                                                       |
C     X,Y      independent variable arrays (input)      |
C     F        dependent variable array    (input)      |
C     FX,FY    dF/dX, dF/dY arrays         (calculated) |
C     FXY      d2F/dXdY array              (calculated) |
C     NI,NJ    array size                  (input)      |
C     ID,JD    dimension size              (input)      |
C                                                       |
C-------------------------------------------------------
      PARAMETER (NMAX=1001)
      REAL FTMP(NMAX), FYTMP(NMAX)
C
      IF(NJ.GT.NMAX) 
     &   STOP 'SPLN2D: array overflow, increase NMAX'
C     
C---- set x-derivatives
      DO 1 J=1, NJ
        CALL SEGSPL(F(1,J),FX(1,J),X(1),NI)
    1 CONTINUE
C
C---- set y-derivatives
      DO 2 I=1, NI
        DO 21 J=1, NJ
          FTMP(J) = F(I,J)
 21     CONTINUE
        CALL SEGSPL(FTMP,FYTMP,Y,NJ)
        DO 23 J=1, NJ
          FY(I,J) = FYTMP(J)
 23     CONTINUE
 2    CONTINUE
C
C---- set xy derivatives
      DO 3 J=1, NJ
        CALL SEGSPL(FY(1,J),FXY(1,J),X(1),NI)
 3    CONTINUE
C
      RETURN
      END ! SPLN2D


      SUBROUTINE SEVL2D(XX,YY,X,Y,F,FX,FY,FXY,NI,NJ,ID,JD,
     &                  FF,FFX,FFY,FFXX,FFYY,FFXY)
      DIMENSION X(ID),Y(JD),F(ID,JD)
      DIMENSION FX(ID,JD), FY(ID,JD), FXY(ID,JD)
C-----------------------------------------------------------------
C     Calculates  F, dF/dX,dF/dY, d2F/dX2,d2F/dY2,... at  (XX,YY) |
C     FX,FY,FXY arrays must have been calculated by SPLN2D        |
C-----------------------------------------------------------------
C
      REAL YT(2), FT(2), FXT(2), FXXT(2), FYT(2), FXYT(2), FXXYT(2)
C
C---- locate j-interval containing YY
      JLOW = 1
      J = NJ
C
   10 IF(J-JLOW .LE. 1) GO TO 11
C
      JMID = (J+JLOW)/2
      IF(YY .LT. Y(JMID)) THEN
       J = JMID
      ELSE
       JLOW = JMID
      ENDIF
      GO TO 10
C
 11   CONTINUE
C
      YT(1)    = Y(J-1)
      YT(2)    = Y(J  )
C
C---- evaluate spline over j-interval to get F, Fx, ...
C-    thus defining a 2-point i-interval spline
c      CALL SEVALL(XX,F (1,J-1),FX (1,J-1),X,NI, FT (1),FXT (1),FXXT (1))
c      CALL SEVALL(XX,FY(1,J-1),FXY(1,J-1),X,NI, FYT(1),FXYT(1),FXXYT(1))
C
c      CALL SEVALL(XX,F (1,J  ),FX (1,J  ),X,NI, FT (2),FXT (2),FXXT (2))
c      CALL SEVALL(XX,FY(1,J  ),FXY(1,J  ),X,NI, FYT(2),FXYT(2),FXXYT(2))
C
      FT(1)    = SEVAL(XX, F(1,J-1), FX(1,J-1),X(1),NI)
      FXT(1)   = DEVAL(XX, F(1,J-1), FX(1,J-1),X(1),NI)
      FXXT(1)  = D2VAL(XX, F(1,J-1), FX(1,J-1),X(1),NI)
C
      FT(2)    = SEVAL(XX, F(1,J  ), FX(1,J  ),X(1),NI)
      FXT(2)   = DEVAL(XX, F(1,J  ), FX(1,J  ),X(1),NI)
      FXXT(2)  = D2VAL(XX, F(1,J  ), FX(1,J  ),X(1),NI)
C
      FYT(1)   = SEVAL(XX,FY(1,J-1),FXY(1,J-1),X(1),NI)
      FXYT(1)  = DEVAL(XX,FY(1,J-1),FXY(1,J-1),X(1),NI)
      FXXYT(1) = D2VAL(XX,FY(1,J-1),FXY(1,J-1),X(1),NI)
C
      FYT(2)   = SEVAL(XX,FY(1,J  ),FXY(1,J  ),X(1),NI)
      FXYT(2)  = DEVAL(XX,FY(1,J  ),FXY(1,J  ),X(1),NI)
      FXXYT(2) = D2VAL(XX,FY(1,J  ),FXY(1,J  ),X(1),NI)
C
C---- evaluate 2-point i-interval spline
c      CALL SEVALL(YY,FT,FYT,YT,2, FF,FFY,FFYY)
C
      FF   = SEVAL(YY,FT,FYT,YT,2)
      FFY  = DEVAL(YY,FT,FYT,YT,2)
      FFYY = D2VAL(YY,FT,FYT,YT,2)
C
c      CALL SEVALL(YY,FXT ,FXYT ,YT,2, FFX,FFXY,FFXYY)
C
      FFX  = SEVAL(YY,FXT ,FXYT ,YT,2)
      FFXY = DEVAL(YY,FXT ,FXYT ,YT,2)
C
      FFXX = SEVAL(YY,FXXT,FXXYT,YT,2)
C
      END ! SEVL2D




      SUBROUTINE SEVL2D0(XX,YY,X,Y,F,FX,FY,FXY,NI,NJ,ID,JD,
     &                   FF )
      DIMENSION X(ID),Y(JD),F(ID,JD)
      DIMENSION FX(ID,JD), FY(ID,JD), FXY(ID,JD)
C-----------------------------------------------------------------
C     Same as SEVL2D, but doesn't return derivatives,             |
C     which makes it faster.                                      |
C-----------------------------------------------------------------
C
      REAL YT(2), FT(2), FYT(2)
C
C---- locate j-interval containing YY
      JLOW = 1
      J = NJ
C
   10 IF(J-JLOW .LE. 1) GO TO 11
C
      JMID = (J+JLOW)/2
      IF(YY .LT. Y(JMID)) THEN
       J = JMID
      ELSE
       JLOW = JMID
      ENDIF
      GO TO 10
C
 11   CONTINUE
C
      YT(1)    = Y(J-1)
      YT(2)    = Y(J  )
C
C---- evaluate spline over j-interval to get F, Fx, ...
C-    thus defining a 2-point i-interval spline
      FT(1)    = SEVAL(XX, F(1,J-1), FX(1,J-1),X(1),NI)
      FT(2)    = SEVAL(XX, F(1,J  ), FX(1,J  ),X(1),NI)
      FYT(1)   = SEVAL(XX,FY(1,J-1),FXY(1,J-1),X(1),NI)
      FYT(2)   = SEVAL(XX,FY(1,J  ),FXY(1,J  ),X(1),NI)
C
C---- evaluate 2-point i-interval spline
      FF   = SEVAL(YY,FT,FYT,YT,2)
C
      END ! SEVL2D0

