
     
      subroutine airfun(cl,tau,Mach,
     &                  idim,jdim,kdim,ldim,
     &                  nAMa,nAcl,nAtau,nAfun,
     &                  AMa,Acl,Atau,ARe,
     &                  A,
     &                  A_M,
     &                  A_cl,
     &                  A_tau,
     &                  A_M_cl,
     &                  A_M_tau,
     &                  A_cl_tau,
     &                  A_M_cl_tau,
     &                  cdf, cdp, cdw, cm)
c-----------------------------------------------------------
c     Evaluates tri-cubic spline functions 
c
c      cdf(Mach,cl,tau) = A(...1)
c      cdp(Mach,cl,tau) = A(...2)
c       cm(Mach,cl,tau) = A(...3)
c
c     at specified paramater values Mach,cl,tau.
c-----------------------------------------------------------
      implicit none

      real cl,tau,Mach
      integer idim,jdim,kdim,ldim
      integer nAMa, nAcl, nAtau,nAfun
      real AMa(idim), Acl(jdim), Atau(kdim), ARe
      real       A(idim,jdim,kdim,ldim),
     &         A_M(idim,jdim,kdim,ldim),
     &        A_cl(idim,jdim,kdim,ldim),
     &       A_tau(idim,jdim,kdim,ldim),
     &      A_M_cl(idim,jdim,kdim,ldim),
     &     A_M_tau(idim,jdim,kdim,ldim),
     &    A_cl_tau(idim,jdim,kdim,ldim),
     &  A_M_cl_tau(idim,jdim,kdim,ldim)
      real cdf, cdp, cdw, cm

      real Mamin, Mamax, dMa, tMa,
     &     clmin, clmax, dcl, tcl,
     &     taumin, taumax, dtau, ttau
c
      integer ilow,imid,
     &        jlow,jmid,
     &        klow,kmid

      integer i,j,k,l


      integer io, im, id,
     &        jo, jm, jd,
     &        ko, km, kd
      real fxm, fxo
      real    Ai(2,2),
     &     Ai_cl(2,2),
     &    Ai_tau(2,2),
     & Ai_cl_tau(2,2)
      real    Aij(2),
     &    Aij_tau(2)

      integer jx,kx,lx
      parameter (lx=3)
      real Aijk(lx)

      if(nAfun.gt.lx) then
       write(*,*) 'AIRFUN: Local array overflow. Increase lx to', nAfun
       stop
      endif

c---- find Ma interval i-1..i
      ilow = 1
      io = nAMa
c
   10 if(io-ilow .le. 1) go to 11
c
      imid = (io+ilow)/2
      if(Mach .lt. AMa(imid)) then
       io = imid
      else
       ilow = imid
      endif
      go to 10
c
   11 continue
      im = io-1
      dMa = AMa(io) - AMa(im)
      tMa = (Mach - AMa(im)) / dMa


c---- find cl interval j-1..j
      jlow = 1
      jo = nAcl
c
   20 if(jo-jlow .le. 1) go to 21
c
      jmid = (jo+jlow)/2
      if(cl .lt. Acl(jmid)) then
       jo = jmid
      else
       jlow = jmid
      endif
      go to 20
c
   21 continue
      jm = jo-1
      dcl = Acl(jo) - Acl(jm)
      tcl = (cl - Acl(jm)) / dcl


c---- find tau interval k-1..k
      klow = 1
      ko = nAtau
c
   30 if(ko-klow .le. 1) go to 31
c
      kmid = (ko+klow)/2
      if(tau .lt. Atau(kmid)) then
       ko = kmid
      else
       klow = kmid
      endif
      go to 30
c
   31 continue
      km = ko-1
      dtau = Atau(ko) - Atau(km)
      ttau = (tau - Atau(km)) / dtau


      do l = 1, lx
        Aijk(l) = 0.
      enddo

c---- evaluate tri-cubic spline in i,j,k box
      do l = 1, nAfun

      do jd = 1, 2
      do kd = 1, 2
        j = jo + jd-2
        k = ko + kd-2

        fxm = dMa*A_M(im,j,k,l) - A(io,j,k,l) + A(im,j,k,l)
        fxo = dMa*A_M(io,j,k,l) - A(io,j,k,l) + A(im,j,k,l)
        Ai(jd,kd) = tMa *A(io,j,k,l)
     &       + (1.0-tMa)*A(im,j,k,l) 
     &   + tMa*(1.0-tMa)*((1.0-tMa)*fxm - tMa*fxo)

        fxm = dMa*A_M_cl(im,j,k,l)
     &           - A_cl(io,j,k,l)
     &           + A_cl(im,j,k,l)
        fxo = dMa*A_M_cl(io,j,k,l)
     &           - A_cl(io,j,k,l)
     &           + A_cl(im,j,k,l)
        Ai_cl(jd,kd) = tMa *A_cl(io,j,k,l)
     &          + (1.0-tMa)*A_cl(im,j,k,l) 
     &      + tMa*(1.0-tMa)*((1.0-tMa)*fxm - tMa*fxo)

        fxm = dMa*A_M_tau(im,j,k,l)
     &           - A_tau(io,j,k,l)
     &           + A_tau(im,j,k,l)
        fxo = dMa*A_M_tau(io,j,k,l)
     &           - A_tau(io,j,k,l)
     &           + A_tau(im,j,k,l)
        Ai_tau(jd,kd) = tMa *A_tau(io,j,k,l)
     &           + (1.0-tMa)*A_tau(im,j,k,l) 
     &       + tMa*(1.0-tMa)*((1.0-tMa)*fxm - tMa*fxo)

        fxm = dMa*A_M_cl_tau(im,j,k,l)
     &           - A_cl_tau(io,j,k,l)
     &           + A_cl_tau(im,j,k,l)
        fxo = dMa*A_M_cl_tau(io,j,k,l)
     &           - A_cl_tau(io,j,k,l)
     &           + A_cl_tau(im,j,k,l)
        Ai_cl_tau(jd,kd) = tMa *A_cl_tau(io,j,k,l)
     &              + (1.0-tMa)*A_cl_tau(im,j,k,l) 
     &          + tMa*(1.0-tMa)*((1.0-tMa)*fxm - tMa*fxo)
      enddo
      enddo

      do kd = 1, 2
        fxm = dcl*Ai_cl(1,kd) - Ai(2,kd) + Ai(1,kd)
        fxo = dcl*Ai_cl(2,kd) - Ai(2,kd) + Ai(1,kd)
        Aij(kd) = tcl *Ai(2,kd)
     &     + (1.0-tcl)*Ai(1,kd) 
     & + tcl*(1.0-tcl)*((1.0-tcl)*fxm - tcl*fxo)
    
        fxm = dcl*Ai_cl_tau(1,kd) - Ai_tau(2,kd) + Ai_tau(1,kd)
        fxo = dcl*Ai_cl_tau(2,kd) - Ai_tau(2,kd) + Ai_tau(1,kd)
        Aij_tau(kd) = tcl *Ai_tau(2,kd)
     &         + (1.0-tcl)*Ai_tau(1,kd) 
     &     + tcl*(1.0-tcl)*((1.0-tcl)*fxm - tcl*fxo)
      enddo

      fxm = dtau*Aij_tau(1) - Aij(2) + Aij(1)
      fxo = dtau*Aij_tau(2) - Aij(2) + Aij(1)
      Aijk(l) =    ttau *Aij(2)
     &      + (1.0-ttau)*Aij(1) 
     & + ttau*(1.0-ttau)*((1.0-ttau)*fxm - ttau*fxo)

      enddo

      cdf = Aijk(1)
      cdp = Aijk(2)
      cm  = Aijk(3)
      cdw = 0.
c     cdw = Aijk(4)

c---- add quadratic cd penalty for exceeding database's cl limits
      clmin = Acl(1)
      clmax = Acl(nAcl)
      if    (cl .lt. clmin) then
       cdp = cdp + 1.0*(cl-clmin)**2
      elseif(cl .gt. clmax) then
       cdp = cdp + 1.0*(cl-clmax)**2
      endif

c---- add quadratic cd penalty for exceeding database's tau limits
      taumin = Atau(1)
      taumax = Atau(nAtau)
      if    (tau .lt. taumin) then
       cdp = cdp + 25.0*(tau-taumin)**2
      elseif(tau .gt. taumax) then
       cdp = cdp + 25.0*(tau-taumax)**2
      endif

      return
      end ! airfun
