
      subroutine atmos(h, T,p,rho,a,mu)
C-----------------------------------------------------------
C     Atmospheric functions  T(h), rho(h) ... 
C       valid to h=20km, p(h) valid to h=70km
C-       h   = km ASL
C-       T   = Kelvin
C-       p   = Pa
C-       rho = kg/m^3
C-       a   = m/s
C-       mu  = kg/m-s
C-----------------------------------------------------------
      implicit real (a-z)

      data pSL / 1.0132e5 /   ! Pa
      data TSL    / 288.2  /  ! K
      data Tpause / 216.65 /  ! K
      data Tblend / 2.0    /  ! K (tropopause blending T range)
      data Tlapse / -6.5 /    ! K/km

      data muSL  / 1.78e-5 /  ! kg/m-s
      data Tsuth / 110.0 /    ! K

      data cp  / 1004.0 /  ! J/kg-K
      data gam / 1.4 /

      T = log(1.0+exp((TSL+Tlapse*h-Tpause)/Tblend))*Tblend + Tpause
      p = pSL*exp( - 0.11800*h   /(1.0 + 0.0020*h   )
     &             - 0.00198*h**2/(1.0 + 0.0006*h**2) )
      rho = gam*p/((gam-1.0)*cp*T)
      a = sqrt(gam*p/rho)
      mu = muSL * sqrt(T/TSL)**3 * (TSL+Tsuth)/(T+Tsuth)

      return
      end ! atmos
