
      subroutine balance(pari,parg,para,rfuel,rpay,xipay, 
     &                   itrim )
c--------------------------------------------------------------------
c     Makes one of three (or none) changes to achieve pitch trim
c     Calculates resulting CG, CP, NP locations
c
c  Inputs:  pari(.)  integer fla array
c           parg(.)  geometry parameter array
c           para(.)  aero parameter array
c           rfuel    fuel fraction   Wfuel_actual/Wfuel_MTOW
c           rpay     payload fraction Wpay_actual/Wpay_MTOW
c           xipay    partial-payload packing location
c                      = 0.0   all the way in front  of cabin
c                      = 0.5   all the way in middle of cabin
c                      = 1.0   all the way in back   of cabin
c           iengloc  engine location index
c           itrim      = 0  no changes
c                      = 1  adjust CLh   (horizontal tail cl)
c                      = 2  adjust Sh    (horizontal tail area)
c                      = 3  adjust xwbox (wing box location)
c
c  Outputs: para(iaxCG)  center of gravity
c           para(iaxCP)  center of pressure ( = xCG if itrim=1,2,3 )
c           para(iaxNP)  neutral point location
c
c--------------------------------------------------------------------
      implicit real (a-h,l-z)

      include 'index.inc'
      integer pari(iitotal)
      real parg(igtotal), para(iatotal)

      include 'constants.inc'

      include 'time.inc'

      common /com_ip/ iptemp

      real a(3,3), r(3)

c     call tset(time0)

      iengloc = pari(iiengloc)

      Wpay   = parg(igWpay  )
      Wfuel  = parg(igWfuel )
      Wfuse  = parg(igWfuse )
      Wwing  = parg(igWwing )
      Wstrut = parg(igWstrut)
      Whtail = parg(igWhtail)
      Wvtail = parg(igWvtail)
      Weng   = parg(igWeng  )

      Whpesys = parg(igWMTO) * parg(igfhpesys)
      Wlgnose = parg(igWMTO) * parg(igflgnose)
      Wlgmain = parg(igWMTO) * parg(igflgmain)

      xcabin = 0.5*(parg(igxshell1) + parg(igxshell2))
      lcabin =      parg(igxshell2) - parg(igxshell1) 
      xpay   = xcabin + (xipay-0.5)*lcabin*(1.0-rpay)

      xwbox  = parg(igxwbox)

      call cglpay(parg,
     &            rfuelF, rpayF, xcgF, 
     &            rfuelB, rpayB, xcgB )

c---- wing centroid offset from wingbox, assumed fixed in CG calculations
      dxwing = parg(igxwing) - parg(igxwbox)

c---- main LG offset from wingbox, assumed fixed in CG calculations
      dxlg = xcgB + parg(igdxlgmain) - parg(igxwbox)

      S   = parg(igS)
      Sh  = parg(igSh)
      co  = parg(igco)
      coh = parg(igcoh)
      xhbox = parg(igxhbox)

      cma  = parg(igcma)

c---- for better convergence, 
c      will assume that HT weight will scale with its area
      Sh1 = Sh

c---- total weight, weight moment, and derivatives
      W  = rpay *Wpay
     &   + rfuel*Wfuel
     &   + Wfuse
     &   + Wwing
     &   + Wstrut
     &   + Whtail*Sh/Sh1
     &   + Wvtail
     &   + Weng
     &   + Whpesys
     &   + Wlgnose
     &   + Wlgmain

      W_Sh = Whtail  /Sh1

      xW = rpay * Wpay *xpay
     &   + rfuel*(Wfuel*parg(igxwbox) + parg(igdxWfuel))
     &   + parg(igxWfuse)
     &   + Wwing *parg(igxwbox) + parg(igdxWwing )
     &   + Wstrut*parg(igxwbox) + parg(igdxWstrut)
     &  + (Whtail*parg(igxhbox) + parg(igdxWhtail))*Sh/Sh1
     &   + Wvtail*parg(igxvbox) + parg(igdxWvtail)
     &   + Weng   *parg(igxeng)
     &   + Whpesys*parg(igxhpesys)
     &   + Wlgnose*parg(igxlgnose)
     &   + Wlgmain*(parg(igxwbox) + dxlg)

      xW_xwbox = rfuel*Wfuel
     &         + Wwing
     &         + Wstrut
     &         + Wlgmain
      xW_Sh = (Whtail*parg(igxhbox) + parg(igdxWhtail)) /Sh1

c---- total aero moment and derivatives
      CMw0 = para(iaCMw0)
      CMw1 = para(iaCMw1)
      CMh0 = para(iaCMh0)
      CMh1 = para(iaCMh1)
      CL  = para(iaCL )
      CLh = para(iaCLh)
      CMVf1 = parg(igCMVf1)
      CLMf0 = parg(igCLMf0)

      cCM       = co *CMw0 + (co*CMw1 - xwbox)*(CL - CLh*Sh/S)
     &          + coh*CMh0*Sh/S + (coh*CMh1 - xhbox)*CLh*Sh/S
     &          + CMVf1*(CL-CLMf0)/S
      cCM_xwbox =                     -        (CL - CLh*Sh/S)
      cCM_Sh    =            (co*CMw1 - xwbox)*(   - CLh   /S)
     &          + coh*CMh0   /S + (coh*CMh1 - xhbox)*CLh   /S
      cCM_CLh   =            (co*CMw1 - xwbox)*(   -     Sh/S)
     &                          + (coh*CMh1 - xhbox)    *Sh/S

c---- current CP and CG location
c      xCP = -cCM/CL
c      xCG = xW/W

c---- total-moment residual
      Res       = cCM/CL       + xW/W 
      Res_CLh   = cCM_CLh  /CL
      Res_Sh    = cCM_Sh   /CL + xW_Sh   /W - (xW/W**2)*W_Sh
      Res_xwbox = cCM_xwbox/CL + xW_xwbox/W

c      r(1) = Res
c      a(1,1) = Res_CLh
c      a(1,2) = Res_Sh
c      a(1,3) = Res_xwbox

c---- drive to Res=0 (pitch trim) by one of three changes...

      if    (itrim.eq.1) then
c----- ... adjust horizontal tail CLh
       delCLh = -Res/Res_CLh
       CLh = CLh + delCLh
       cCM = cCM + cCM_CLh*delCLh
       para(iaCLh) = CLh

      elseif(itrim.eq.2) then
c----- ... adjust horizontal tail area
       delSh = -Res/Res_Sh
       Sh = Sh + delSh
       cCM = cCM + cCM_Sh*delSh
       xW  = xW  +  xW_Sh*delSh

       parg(igSh) = Sh

      elseif(itrim.eq.3) then
c----- ... adjust wing box location
       delxwbox = -Res/Res_xwbox
       xwbox = xwbox + delxwbox
       cCM = cCM + cCM_xwbox*delxwbox
       xW  = xW  +  xW_xwbox*delxwbox

       parg(igxwbox) = xwbox
       parg(igxwing) = xwbox + dxwing

      endif

c---- calculate neutral point
      dCLhdCL  = parg(igdCLhdCL)

      dCLndCL  = parg(igdCLndCL)
      neng = parg(igneng)
      xeng = parg(igxeng)
      dfan = parg(igdfan)
      Afan = 0.25*pi*dfan**2
      xengcp = xeng - 0.25*dfan

      xNP       = (co *CMw1 - xwbox)*(dCLhdCL*Sh/S - 1.0)
     &          - (coh*CMh1 - xhbox)* dCLhdCL*Sh/S
     &          - CMVf1/S
     &          - neng*xengcp*dCLndCL*Afan/S
      xNP_xwbox =           -        (dCLhdCL*Sh/S - 1.0)
      xNP_Sh    = (co *CMw1 - xwbox)* dCLhdCL   /S
     &          - (coh*CMh1 - xhbox)* dCLhdCL   /S
      xNP_CLh   = 0.

c---- x locations after one of the trim changes
c-     (xCP,xCG will be equal if moments are linear)
      xCP = -cCM/CL
      xCG = xW/W

c---- component pitching moments
c      cCM  = co *CMw0      + (co *CMw1 - xwbox)*(CL - CLh*Sh/S) - xref*CL
c     &     + coh*CMh0*Sh/S + (coh*CMh1 - xhbox)*      CLh*Sh/S
c     &     + CMVf1*(CL-CLMf0)/S
c
c      cCM_CL  = (co*CMw1  - xwbox - xref)*(1.0 - CLh_CL*Sh/S)
c     &        + (coh*CMh1 - xhbox - xref)*CLh_CL*Sh/S

c     CMwing = (co *CMw0      + (co *CMw1-xwbox-xCG)*(CL-CLh*Sh/S))/cma
c     CMtail = (coh*CMh0*Sh/S + (coh*CMh1-xhbox-xCG)*    CLh*Sh/S )/cma
c     CMfuse = CMVf1*(CL-CLMf0)/(S*cma)

c---- store results for returning
c      para(iaCMwing) = CMwing
c      para(iaCMtail) = CMtail
c      para(iaCMfuse) = CMfuse

      para(iaxCG) = xCG
      para(iaxCP) = xCP
      para(iaxNP) = xNP

c     call tadd(time0,t_balance)

      return
      end ! balance



      subroutine htsize(pari,parg,paraF,paraB,paraC)
c--------------------------------------------------------------------
c     Sets horizontal tail area and wing position to simultaneously:
c      1) Meet pitch trim requirement with forward CG
c      2) Meet stability requirement with aft CG
c
c     Calculates resulting CG, CP, NP locations
c
c  Inputs:  pari(.)  integer fla array
c           parg(.)  geometry parameter array
c           paraF(.) aero parameter array for fwdCG case
c           paraB(.) aero parameter array for aft CG case
c           paraC(.) aero parameter array for cruise tail CL case
c
c  Outputs: parg(igSh)    HT area
c           parg(igxwbox) wingbox location
c           parg(igxwing) wing centroid location
c
c--------------------------------------------------------------------
      implicit real (a-h,l-z)

      include 'index.inc'
      integer pari(iitotal)
      real parg(igtotal)
      real paraF(iatotal),
     &     paraB(iatotal),
     &     paraC(iatotal)

      include 'constants.inc'

      real a(2,2), r(2)
      real a$(2,2), r$(2)
      character*2 ss1, ss2

      include 'time.inc'

      data itmax / 10 /
      data toler / 1.0e-7 /

c     call tset(time0)

      sweep  = parg(igsweep)
      cosL  = cos(sweep *pi/180.0)

c---- set CG limits with worst-case payload arrangements
      call cglpay(parg,
     &            rfuelF, rpayF, xcgF, 
     &            rfuelB, rpayB, xcgB )
      rpayC = 1.0

      parg(igxCGfwd) = xcgF
      parg(igxCGaft) = xcgB

      CLF  = paraF(iaclpmax)*cosL**2
      CLhF = parg(igCLhCGfwd)

      CLC  = paraF(iaCL)
      CLhC = parg(igCLhspec)

      SM = parg(igSMmin)
      cma = parg(igcma)

      iengloc = pari(iiengloc)
      iHTsize = pari(iiHTsize)
      ixwmove = pari(iixwmove)

      Wpay   = parg(igWpay  )
      Wfuel  = parg(igWfuel )
      Wfuse  = parg(igWfuse )
      Wwing  = parg(igWwing )
      Wstrut = parg(igWstrut)
      Whtail = parg(igWhtail)
      Wvtail = parg(igWvtail)
      Weng   = parg(igWeng  )

      Whpesys = parg(igWMTO) * parg(igfhpesys)
      Wlgnose = parg(igWMTO) * parg(igflgnose)
      Wlgmain = parg(igWMTO) * parg(igflgmain)

      xWfuse = parg(igxWfuse)

      dxWfuel  = parg(igdxWfuel )
      dxWwing  = parg(igdxWwing )
      dxWstrut = parg(igdxWstrut)
      dxWhtail = parg(igdxWhtail)
      dxWvtail = parg(igdxWvtail)
      dxlgmain = parg(igdxlgmain)

      xeng    = parg(igxeng)   
      xhpesys = parg(igxhpesys)
      xlgnose = parg(igxlgnose)

      xcabin = 0.5*(parg(igxshell1) + parg(igxshell2))
      lcabin =      parg(igxshell2) - parg(igxshell1) 

c---- payload CG locations for forward and aft CG locations
      xpayF  = xcabin + (0.0  -0.5)*lcabin*(1.0-rpayF)
      xpayB  = xcabin + (1.0  -0.5)*lcabin*(1.0-rpayB)
      xpayC  = xcabin


      xwbox = parg(igxwbox)

c---- wing centroid offset from wingbox, assumed fixed in CG calculations
      dxwing = parg(igxwing) - parg(igxwbox)

c---- main LG offset from wingbox, assumed fixed in CG calculations
      dxlg = parg(igxCGaft) + parg(igdxlgmain) - parg(igxwbox)

      S   = parg(igS)
      Sh  = parg(igSh)
      co  = parg(igco)
      coh = parg(igcoh)
      xhbox = parg(igxhbox)
      xvbox = parg(igxvbox)

      WfuelC = paraC(iafracW)*parg(igWMTO)
     &       - rpayC*Wpay
     &       - Wfuse
     &       - Wwing
     &       - Wstrut
     &       - Whtail
     &       - Wvtail
     &       - Weng
     &       - Whpesys
     &       - Wlgnose
     &       - Wlgmain
      rfuelC = WfuelC/Wfuel


c---- for better convergence, 
c      will assume that HT weight and moment will scale with its area
      Sh_o = Sh
      Whtail_o = Whtail
      dxWhtail_o = dxWhtail

cc      do 100 iter = 0, -2, -1

      do 100 iter = 1, itmax
         
       if    (iter.eq.-1) then
        eps = 1.0e-5 * Sh
        Sh   = Sh$ + eps
        xwbox= xwbox$
        j = 1
       elseif(iter.eq.-2) then
        eps = 1.0e-5 * lcabin
        Sh   = Sh$
        xwbox= xwbox$ + eps
        j = 2
       endif


      Whtail    = (Whtail_o/Sh_o) * Sh
      Whtail_Sh =  Whtail_o/Sh_o

      dxWhtail    = (dxWhtail_o/Sh_o) * Sh
      dxWhtail_Sh =  dxWhtail_o/Sh_o

c---- empty (no fuel, no payload) weight
      We = !   rpay *Wpay
     &     ! + rfuel*Wfuel
     &      + Wfuse
     &      + Wwing
     &      + Wstrut
     &      + Whtail
     &      + Wvtail
     &      + Weng
     &      + Whpesys
     &      + Wlgnose
     &      + Wlgmain

      We_Sh = Whtail_Sh

c---- empty (no-payload) weight moment
      xWe = !   rpay *Wpay*xpay
     &      ! + rfuel*(Wfuel*xwbox+ dxWfuel)
     &       + xWfuse
     &       + Wwing *xwbox    + dxWwing
     &       + Wstrut*xwbox    + dxWstrut
     &       + Whtail*xhbox    + dxWhtail
     &       + Wvtail*xvbox    + dxWvtail
     &       + Weng   *xeng
     &       + Whpesys*xhpesys
     &       + Wlgnose*xlgnose
     &       + Wlgmain*(xwbox + dxlg)

      xWe_Sh = Whtail_Sh*xhbox + dxWhtail_Sh
      xWe_xw = Wwing
     &       + Wstrut
     &       + Wlgmain


c---- total weight at forward and aft CG limits, and cruise
      WF = We + rfuelF*Wfuel + rpayF*Wpay
      WB = We + rfuelB*Wfuel + rpayB*Wpay
      WC = We + rfuelC*Wfuel + rpayC*Wpay

      WF_Sh = We_Sh
      WB_Sh = We_Sh
      WC_Sh = We_Sh

c---- total weight moment at forward and aft CG limits
      xWF = xWe + rpayF*Wpay*xpayF + rfuelF*(Wfuel*xwbox + dxWfuel)
      xWB = xWe + rpayB*Wpay*xpayB + rfuelB*(Wfuel*xwbox + dxWfuel)
      xWC = xWe + rpayC*Wpay*xpayC + rfuelC*(Wfuel*xwbox + dxWfuel)

      xWF_xw = xWe_xw + rfuelF*Wfuel
      xWB_xw = xWe_xw + rfuelB*Wfuel
      xWC_xw = xWe_xw + rfuelC*Wfuel

      xWF_Sh = xWe_Sh
      xWB_Sh = xWe_Sh
      xWC_Sh = xWe_Sh

      if(iHTsize .eq. 1) then
c----- fix HT volume
       lhtail = parg(igxhtail) - (xwbox + dxwing)
       lhtail_xw = -1.0
       Vh = parg(igVh)
       Sh = Vh*S*cma/lhtail

       r(1)   = Sh*lhtail - Vh*S*cma
       a(1,1) =    lhtail
       a(1,2) = Sh*lhtail_xw

      else
c----- set HT area by pitch trim power at forward CG case
       CMw0  = paraF(iaCMw0)
       CMw1  = paraF(iaCMw1)
       CMh0  = paraF(iaCMh0)
       CMh1  = paraF(iaCMh1)
       CMVf1 = parg(igCMVf1)
       CLMf0 = parg(igCLMf0)
       CL    = CLF
       CLh   = CLhF
       cCM    = co *CMw0 + (co*CMw1 - xwbox)*(CL - CLh*Sh/S)
     &        + coh*CMh0*Sh/S + (coh*CMh1 - xhbox)*CLh*Sh/S
     &        + CMVf1*(CL-CLMf0)/S
       cCM_xw =                     -        (CL - CLh*Sh/S)
       cCM_Sh =            (co*CMw1 - xwbox)*(   - CLh   /S)
     &        + coh*CMh0   /S + (coh*CMh1 - xhbox)*CLh   /S

       r(1)   = cCM   /CL + xWF   /WF 
       a(1,1) = cCM_Sh/CL + xWF_Sh/WF - (xWF/WF**2)*WF_Sh
       a(1,2) = cCM_xw/CL + xWF_xw/WF

c       cma = parg(igcma)
c       write(*,'(1x,3f12.5)') CL,  CLh, CMVf1*(CL-CLMf0)/S/cma

      endif


      if    (ixwmove .eq. 0) then
c----- fix wing location
       r(2)   = 0.
       a(2,1) = 0.
       a(2,2) = 1.0

      elseif(ixwmove .eq. 1) then
c----- set wing location to get CLh=CLhspec in cruise
       CMw0  = paraC(iaCMw0)
       CMw1  = paraC(iaCMw1)
       CMh0  = paraC(iaCMh0)
       CMh1  = paraC(iaCMh1)
       CMVf1 = parg(igCMVf1)
       CLMf0 = parg(igCLMf0)
       CL    = CLC
       CLh   = CLhC
       cCM    = co *CMw0 + (co*CMw1 - xwbox)*(CL - CLh*Sh/S)
     &        + coh*CMh0*Sh/S + (coh*CMh1 - xhbox)*CLh*Sh/S
     &        + CMVf1*(CL-CLMf0)/S
cc   &        - neng*xenginl*CLn*Afan/S
       cCM_xw =                     -        (CL - CLh*Sh/S)
       cCM_Sh =            (co*CMw1 - xwbox)*(   - CLh   /S)
     &        + coh*CMh0   /S + (coh*CMh1 - xhbox)*CLh   /S

       r(2)   = cCM   /CL + xWC   /WC 
       a(2,1) = cCM_Sh/CL + xWC_Sh/WC - (xWC/WC**2)*WC_Sh
       a(2,2) = cCM_xw/CL + xWC_xw/WC

      elseif(ixwmove .eq. 2) then
c----- set wing location by stability margin at aft-CG case
       CMw1  = paraB(iaCMw1)
       CMh1  = paraB(iaCMh1)
       CMVf1 = parg(igCMVf1)
       dCLhdCL  = parg(igdCLhdCL)

       dCLndCL  = parg(igdCLndCL)
       neng = parg(igneng)
       xeng = parg(igxeng)
       dfan = parg(igdfan)
       Afan = 0.25*pi*dfan**2
       xengcp = xeng - 0.25*dfan

       xNP    = (co *CMw1 - xwbox)*(dCLhdCL*Sh/S - 1.0)
     &        - (coh*CMh1 - xhbox)* dCLhdCL*Sh/S
     &        - CMVf1/S
     &        - neng*xengcp*dCLndCL*Afan/S
       xNP_xw =           -        (dCLhdCL*Sh/S - 1.0)
       xNP_Sh = (co *CMw1 - xwbox)* dCLhdCL   /S
     &        - (coh*CMh1 - xhbox)* dCLhdCL   /S

       r(2)   = xWB   /WB - xNP    + SM*cma
       a(2,1) = xWB_Sh/WB - xNP_Sh   - (xWB/WB**2)*WB_Sh
       a(2,2) = xWB_xw/WB - xNP_xw

c       write(*,'(1x,i4,19f12.5)') 
c     &   iter, xNP, xwbox,Sh,S,co,coh,xhbox,CMw1,CMh1,dCLhdCL

      endif


c???????????????????????????????????????????????????
      if(iter .eq. 0) then

       r$(1) = r(1)
       r$(2) = r(2)
       a$(1,1) = a(1,1)
       a$(1,2) = a(1,2)
       a$(2,1) = a(2,1)
       a$(2,2) = a(2,2)

       xwbox$ = xwbox
       Sh$ = Sh

      elseif(iter .lt. 0) then
       aa1 = (a(1,j)+a$(1,j))*0.5
       aa2 = (a(2,j)+a$(2,j))*0.5
       dd1 = (r(1)-r$(1))/eps
       dd2 = (r(2)-r$(2))/eps
       call compare(ss1,aa1,dd1)
       call compare(ss2,aa2,dd2)

       write(*,3100) 1,j,aa1,ss1, 2,j,aa2,ss2,
     &               1,j,dd1,ss1, 2,j,dd2,ss2
 3100  format(2('   R',i1,'_',i1,g16.8,2x,a))

      endif

      if(iter.le.0) go to 100
c???????????????????????????????????????????????????


      det = a(1,1)*a(2,2) - a(1,2)*a(2,1)
      d1 = (r(1)  *a(2,2) - a(1,2)*r(2)  ) / det
      d2 = (a(1,1)*r(2)   - r(1)  *a(2,1)) / det

      dSh = -d1
      dxw = -d2

      Sh    = Sh   + dSh
      xwbox = xwbox+ dxw

      dmax = max( abs(dxw)/cma , abs(dSh)/(0.2*S) )
      if(dmax .lt. toler) go to 110

 100  continue
      write(*,*) 'HTSIZE: Pitch not converged. dxwbox,dSh = ', dxw,dSh
 110  continue


c---- set converged results
      parg(igSh  )   = Sh
cc    parg(igWhtail) = (Whtail_o/Sh_o) * Sh
      parg(igxwbox)  = xwbox
      parg(igxwing)  = xwbox + dxwing

      if(iHTsize .eq. 1) then
c----- for fixed HT area, find minimum required CLh for foward-CG trim
       CMw0  = paraF(iaCMw0)
       CMw1  = paraF(iaCMw1)
       CMh0  = paraF(iaCMh0)
       CMh1  = paraF(iaCMh1)
       CMVf1 = parg(igCMVf1)
       CLMf0 = parg(igCLMf0)
       CL    = CLF
       CLh   = CLhF
       cCM     = co *CMw0 + (co*CMw1 - xwbox)*(CL - CLh*Sh/S)
     &         + coh*CMh0*Sh/S + (coh*CMh1 - xhbox)*CLh*Sh/S
     &         + CMVf1*(CL-CLMf0)/S
       cCM_CLh =            (co*CMw1 - xwbox)*(   -     Sh/S)
     &                         + (coh*CMh1 - xhbox)    *Sh/S
       res     = cCM    /CL + xWF/WF 
       res_CLh = cCM_CLh/CL 
       dCLh = -res/res_CLh
       CLh = CLh + dCLh
       parg(igCLhCGfwd) = CLh
      endif

c     call tadd(time0,t_htsize)

      return
      end ! htsize



      subroutine cglpay(parg,
     &                  rfuelF, rpayF, xcgF, 
     &                  rfuelB, rpayB, xcgB )
c==================================================================
c     Calculates min and max xCG locations from payload extremes,
c     and corresponding payload fractions.
c
c     rfuelF,rpayF   give most-forward  location xcgF
c     rfuelB,rpayB   give most-rearward location xcgB
c
c     This version always returns rfuelF = rfuelB = 0.0
c     which gives an explicit solution for rpayF,rpayB.
c
c     The alternative 2D search for rfuel,rpay is kinda ugly, 
c     and unwarranted in practice.
c
c==================================================================
      implicit real (a-h,l-z)

      include 'index.inc'
      real parg(igtotal)

      include 'constants.inc'

      real xi(2), sgn(2), rpay(2), xcg(2)

      Wpay   = parg(igWpay )
      Wfuel  = parg(igWfuel)
      Wpay   = parg(igWpay  )
      Wfuel  = parg(igWfuel )
      Wfuse  = parg(igWfuse )
      Wwing  = parg(igWwing )
      Wstrut = parg(igWstrut)
      Whtail = parg(igWhtail)
      Wvtail = parg(igWvtail)
      Weng   = parg(igWeng  )

      Whpesys = parg(igWMTO) * parg(igfhpesys)
      Wlgnose = parg(igWMTO) * parg(igflgnose)
      Wlgmain = parg(igWMTO) * parg(igflgmain)

      xcabin = 0.5*(parg(igxshell1) + parg(igxshell2))
      lcabin =      parg(igxshell2) - parg(igxshell1) 

      delxw = parg(igxwing) - parg(igxwbox)

c---- zero fuel is assumed to be worst case
      rfuel  = 0.
      rfuelF = 0.
      rfuelB = 0.

      We = !  rpay *Wpay
     &   + rfuel*Wfuel
     &   + Wfuse
     &   + Wwing
     &   + Wstrut
     &   + Whtail
     &   + Wvtail
     &   + Weng
     &   + Whpesys
     &   + Wlgnose
     &   + Wlgmain

      xWe = !  rpay *Wpay*xpay
     &   + rfuel*(Wfuel*parg(igxwbox) + parg(igdxWfuel))
     &   + parg(igxWfuse)
     &   + Wwing *parg(igxwbox) + parg(igdxWwing )
     &   + Wstrut*parg(igxwbox) + parg(igdxWstrut)
     &   + Whtail*parg(igxhbox) + parg(igdxWhtail)
     &   + Wvtail*parg(igxvbox) + parg(igdxWvtail)
     &   + Weng   *parg(igxeng)
     &   + Whpesys*parg(igxhpesys)
     &   + Wlgnose*parg(igxlgnose)
     &   + Wlgmain*(parg(igxwbox) + delxw + parg(igdxlgmain))


ccc   xpay = xcabin + (xipay-0.5)*lcabin*(1.0-rpay)
ccc    W =      Wpay*rpay +  We
ccc   xW = xpay*Wpay*rpay + xWe

      xi(1) = 0.0
      xi(2) = 1.0
      sgn(1) = -1.0
      sgn(2) =  1.0

      do i = 1, 2
        a0 = xWe
        a1 = (xcabin + (xi(i)-0.5)*lcabin)*Wpay
        a2 =         - (xi(i)-0.5)*lcabin *Wpay
        b0 = We
        b1 = Wpay

        AA = a2*b1
        BB = 2.0*a2*b0
        CC = a1*b0 - a0*b1

        rpay(i) = (-BB - sgn(i)*sqrt(BB**2 - 4.0*AA*CC)) * 0.5/AA
        xpay = xcabin + (xi(i)-0.5)*lcabin*(1.0-rpay(i))
        W  =      Wpay*rpay(i) +  We
        xW = xpay*Wpay*rpay(i) + xWe
        xcg(i) = xW/W
      enddo

      rpayF = rpay(1)
      rpayB = rpay(2)

      xcgF = xcg(1)
      xcgB = xcg(2)
     
      return
      end ! cglpay

