
      subroutine gasprop(ttf,tt3,tt4,
     &                   rfuel,cpfuel,hsfuel,
     &                   nhfuel,ncfuel,nnfuel,nofuel,
     &                   alphai,
     &                   cp3,gam3, cp4,gam4, f)
c---------------------------------------------------------------
c     Computes fuel mass fraction needed to reach 
c     a specified combustion-product temperature,
c     and also computes the gas properties of the
c     combusted gas mixture.
c
c  Input:
c     ttf     total temperature of fuel (has weak effect)
c     tt3     total temperature of air before combustion
c     tt4     total temperature of combusted mixture
c     rfuel   gas constant of fuel vapor
c     cpfuel  specific heat of fuel vapor
c     hsfuel  specific enthalpy of formation of fuel (negative)
c     nhfuel  number of hydrogen atoms in fuel molecule
c     ncfuel  number of carbon   atoms in fuel molecule
c     nnfuel  number of nitrogen atoms in fuel molecule
c     nofuel  number of oxygen   atoms in fuel molecule
c             Note:  Only the relative numbers of atoms matter.
c                    Methane : nh=4, nc=1, nn=0, no=0
c                    Propane : nh=8, nc=3, nn=0, no=0
c                    Heavy HC: nh=2, nc=1, nn=0, no=0
c
c  Output:
c     cp4    specific heat of combustion gas
c     gam4   ratio of specific heats of combustion gas
c     f      fuel/air mass fraction
c---------------------------------------------------------------
c
      real alphai(*)
c
      parameter(n=5)
      real ri(n), cpcoldi(n), cphoti(n), hsi(n),
     &     betai(n), deltai(n), gammai(n)
      real lambdai(n)

ccc                   N2       O2       CO2      H2O    fuel(CH4)
      data ri      / 296.94 , 259.82 , 188.96 , 461.91  , 519.65 /
      data cpcoldi / 1041.0 ,  920.0 ,  851.0 , 1864.0  , 2240.0 /
      data cphoti  / 1219.0 , 1125.0 , 1300.0 , 2524.0  ,    0.0 /
ccc   data hsi     /    0.0 ,   0.0  ,-8.949e6,-13.444e6,-5.681e6/
      data hsi     /    0.0 ,   0.0  ,-8.949e6,-13.444e6,-4.675e6/
      data betai   / 0.0    , 0.0    , 0.0    , 0.0     ,  1.0   /
      data deltai  / 0.0    , -4.0   , 2.75   , 2.25    , -1.0   / ! default

      data ts / 298.0 /


c---- molar weights of H, C, N, O
      data wh, wc, wn, wo / 1.00795 , 12.01078 , 14.00672 , 15.99943 /

c---- molar weight of fuel
      wfuel = wh*float(nhfuel)
     &      + wc*float(ncfuel)
     &      + wn*float(nnfuel)
     &      + wo*float(nofuel)
c
c---- molar weights of combustion gases, to balance atoms in reaction equation
c-     Wfuel + W_O2  =  W_N2 + W_CO2 + W_H2O + W_N2
      wn2  = (     wn*2.0) * float(nnfuel)*0.5
      wco2 = (wc + wo*2.0) * float(ncfuel)
      wh2o = (wo + wh*2.0) * float(nhfuel)*0.5
      wo2  = (     wo*2.0)
     &     * (float(ncfuel) + float(nhfuel)*0.25 - float(nofuel)*0.5)
c
c---- store weight fractions of reaction gases, relative to fuel weight
c-    (left side is negative, right side is positive)
      deltai(1) =  wn2/wfuel
      deltai(2) = -wo2/wfuel
      deltai(3) = wco2/wfuel
      deltai(4) = wh2o/wfuel
      deltai(5) = -1.0
c
c
c---- set fuel gas constant, cp, enthalpy
      ri(5)      = rfuel
      cpcoldi(5) = cpfuel
      hsi(5)     = hsfuel
c
      do i = 1, n
        gammai(i) = betai(i) + deltai(i)
      enddo
c
c
      r3  = dot(alphai,ri     ,n)
      cp3 = dot(alphai,cpcoldi,n)
      gam3 = cp3/(cp3-r3)

      dota = dot(alphai,cpcoldi,n)*(tt4-tt3  )
      dotb = dot( betai,cpcoldi,n)*(ttf-ts) + dot( betai,hsi,n)
      dotc = dot(gammai,cphoti ,n)*(tt4-ts) + dot(gammai,hsi,n)

c      dotb1 = dot( betai,cpi,n)*(ttf-ts)
c      dotb2 = dot( betai,hsi,n)
c      dotc1 = dot(gammai,cpi,n)*(tt4-ts)
c      dotc2 = dot(gammai,hsi,n)
c      write(*,*) 'alpha. cp(T4-T3)       =', dota
c      write(*,*) ' beta.[cp(Tf-Ts) + hs] =', dotb
c      write(*,*) 'gamma.[cp(T4-Ts) + hs] =', dotc

      f = dota / (dotb - dotc)

      do i = 1, n
        lambdai(i) = (alphai(i) + f*gammai(i)) / (1.0+f)
      enddo

      cp4  = dot(lambdai,cphoti,n)
      r4   = dot(lambdai,ri    ,n)
      gam4 = cp4/(cp4-r4)

      do i = 1, 5
        alphai(i) = (alphai(i) + gammai(i)*f)/(1.0+f)
      enddo

      return
      end ! gasprop



      real function dot(a,b,n)
      real a(n), b(n)
      dot = 0.
      do i = 1, n
        dot = dot + a(i)*b(i)
      enddo
      return
      end ! dot
