
      subroutine gas_tset(alpha,n, hspec,tguess, t )
c-------------------------------------------------------------------
c     Calculates temperature for a specified enthalpy.
c     The constant-cp equivalent is
c
c              t = (hspec - hf) /cp  
c
c     where hf is the heat of formation included in h(t)
c
c  Input:
c     alpha(.)  mass fractions for gas constituents i = 1..n
c     n         number of constituents present
c     hspec     specified enthalpy
c     tguess    first guess for temperature
c
c  Output:
c     t        temperature
c-------------------------------------------------------------------
      real alpha(n)

      data itmax / 10 /
c      data ttol  / 0.00001 /
      data ttol  / 0.000001 /
c      data ttol  / 0.0000000001 /
 
      t = tguess

c      write(*,*)
c      write(*,*) 'gas_tset'

      do iter = 1, itmax
        call gassum(alpha,n, t, s,s_t, h,h_t, cp,r)
        res   = h - hspec
        res_t = h_t

        dt = -res/res_t

c        write(*,*) iter, t, dt

        if(abs(dt) .lt. ttol) go to 20

        t = t + dt
      enddo
      write(*,*) 'gas_tset: convergence failed.  dT =', dt

 20   continue
      return
      end ! gas_tset


      subroutine gas_tsetd(alpha,n, hspec,tguess, t,
     &                     t_hspec, t_al )
c-------------------------------------------------------------------
c     Came as gas_tset, but also returns derivative
c-------------------------------------------------------------------
      real alpha(n), t_al(n)

      data itmax / 10 /
c      data ttol  / 0.00001 /
      data ttol  / 0.000001 /
c      data ttol  / 0.0000000001 /
 
      t = tguess

c      write(*,*)
c      write(*,*) 'gas_tset'

      do iter = 1, itmax
        call gassum(alpha,n, t, s,s_t, h,h_t, cp,r)
        res   = h - hspec
        res_t = h_t

        dt = -res/res_t

c        write(*,*) iter, t, dt

        if(abs(dt) .lt. ttol) go to 20

         t = t + dt
      enddo
      write(*,*) 'gas_tset: convergence failed.  dT =', dt

 20   continue

      t_hspec = 1.0/h_t

      do i = 1, n
        call gasfun(i,t, si,s_ti, hi,h_ti, cpi,ri)
        t_al(i) = -hi / h_t
      enddo

      return
      end ! gas_tsetd



      subroutine gas_prat(alpha,n, po,to,ho,so,cpo,ro, pi,epol,
     &                             p ,t ,h ,s ,cp ,r )
c-------------------------------------------------------------------
c     Calculates state change for a specified pressure ratio.
c     The constant-cp equivalent is the usual isentropic relations,
c     but with epol included.
c
c       g = cp/(cp-r)
c       gexp = (g-1)/(g*epol)
c       tau = pi^gexp
c       p = po * pi
c       t = to * tau
c       (h-hf) = (ho-hf) * tau
c
c
c  Input:
c     alpha(.)  mass fractions for gas constituents i = 1..n
c     n         number of constituents present
c
c     po        starting pressure
c     to        starting temperature
c     ho        starting enthalpy
c     so        starting entropy-complement
c     cpo       starting specific heat
c     ro        starting gas constant
c
c     pi        pressure ratio
c     epol      polytropic efficiency of change process  , if compression
c     epol   1/(polytropic efficiency of change process) , if expansion
c
c  Output:
c     p       ending pressure
c     t       ending temperature
c     h       ending enthalpy
c     s       ending entropy-complement
c     cp      ending specific heat
c     r       ending gas constant (this will be the same as starting ro)
c-------------------------------------------------------------------
      real alpha(n)

      data itmax / 10 /
c      data ttol  / 0.00001 /
      data ttol  / 0.000001 /
c      data ttol  / 0.0000000001 /
 
      gexp = ro/(cpo*epol)
      t = to * pi**gexp

      pile = log(pi) / epol

c      write(*,*)
c      write(*,*) 'gas_prat'

      do iter = 1, itmax
        call gassum(alpha,n, t, s,s_t, h,h_t, cp,r)
        res   = (s - so)/r - pile
        res_t =  s_t    /r

        dt = -res/res_t

c        write(*,*) iter, t, dt

        if(abs(dt) .lt. ttol) go to 20

        t = t + dt
      enddo
      write(*,*) 'gas_prat: convergence failed.  dT =', dt
      write(*,*) 'To pi =', To, pi

 20   continue
      p = po * pi

      return
      end ! gas_prat



      subroutine gas_pratd(alpha,n, po  , to  , ho  , so,cpo,ro,pi,epol,
     &                              p   , t   , h   , s ,cp ,r,
     &                              p_po,
     &                              p_so, t_so, h_so, s_so,
     &                              p_pi, t_pi, h_pi, s_pi,
     &                              p_ep, t_ep, h_ep, s_ep,
     &                              p_al, t_al, h_al, s_al, cp_al, r_al)
c-------------------------------------------------------------------
c     Same as gas_prat, but also returns Jacobians w.r.t. po,to,pi,epol
c-------------------------------------------------------------------
      real alpha(n)
      real p_al(n), t_al(n), h_al(n), s_al(n), cp_al(n), r_al(n)

      data itmax / 10 /
c      data ttol  / 0.00001 /
      data ttol / 0.000001 /
c      data ttol  / 0.000000001 /
 
      gexp = ro/(cpo*epol)
      t = to * pi**gexp

      pile = log(pi) / epol

c      write(*,*)
c      write(*,*) 'gas_prat'

      do iter = 1, itmax
        call gassum(alpha,n, t, s,s_t, h,h_t, cp,r)

c------ res( s(t,al), r(al), so, pi )
        res   = (s - so)/r - pile
        res_t =  s_t    /r

        dt = -res/res_t

cc        write(*,*) iter, t, dt

        t = t + dt

        if(abs(dt) .lt. ttol) go to 20

      enddo
      write(*,*) 'gas_prat: convergence failed.  dT =', dt
      write(*,*) 'To pi =', To, pi

 20   continue
      call gassumd(alpha,n, t, s,s_t, h,h_t, cp,cp_t, r)
      res_t =  s_t    /r

c---- res( so, pi, al )
      res_so = -1.0/r
      res_pi = -1.0/(epol*pi)
      res_ep = pile/epol
cc    res_al =  si/r - (s-so)/r**2 * ri

      t_so = -res_so/res_t
      t_pi = -res_pi/res_t
      t_ep = -res_ep/res_t
cc    t_al = -res_al/res_t 

cc          t(so,pi,ep,al)
cc       h( t(so,pi,ep,al),al )
cc       s( t(so,pi,ep,al),al )
cc      cp( t(so,pi,ep,al),al )
cc       r( al )
cc    

c---- p( po, so, pi )
      p    = po*pi
      p_so = 0.
      p_po =    pi
      p_pi = po
      p_ep = 0.

c---- h( t(so,pi,ep,al),al )
      h_so = h_t*t_so
      h_pi = h_t*t_pi
      h_ep = h_t*t_ep
cc    h_al = h_t*t_al + hi

c---- s( t(so,pi,ep,al),al )
      s_so = s_t*t_so
      s_pi = s_t*t_pi
      s_ep = s_t*t_ep
cc    s_al = s_t*t_al + si

      do i = 1, n
        call gasfun(i,t, si,s_ti, hi,h_ti, cpi,ri)
        res_al =  si/r - (s-so)/r**2 * ri
        t_al(i) =      -res_al/res_t 
        h_al(i) = h_t*(-res_al/res_t) + hi
        s_al(i) = s_t*(-res_al/res_t) + si
        p_al(i) = 0.
        cp_al(i) = cp_t*(-res_al/res_t) + cpi
        r_al(i) = ri
      enddo

      return
      end ! gas_pratd


      subroutine gas_delh(alpha,n, po,to,ho,so,cpo,ro, delh,epol,
     &                             p ,t ,h ,s ,cp ,r )
c-------------------------------------------------------------------
c     Calculates state change for a specified enthalpy change.
c     The constant-cp equivalent is the usual isentropic relations, 
c     but with epol included.
c
c       t - to = delh/cp
c       g = cp/(cp-r)
c       gexp = (g-1)/(g*epol)
c       tau = t/to
c       pi = tau^(1/gexp)
c       p = po * pi
c       (h-hf) = (ho-hf) * tau
c
c
c
c  Input:
c     alpha(.)  mass fractions for gas constituents i = 1..n
c     n         number of constituents present
c
c     po        starting pressure
c     to        starting temperature
c     ho        starting enthalpy
c     so        starting entropy-complement
c     cpo       starting specific heat
c     ro        starting gas constant
c
c     delh      enthalpy change
c     epol         polytropic efficiency of change process  , if compression
c               1/(polytropic efficiency of change process) , if expansion
c
c  Output:
c     p       ending pressure
c     t       ending temperature
c     h       ending enthalpy
c     s       ending entropy-complement
c     cp      ending specific heat
c     r       ending gas constant (this will be the same as starting ro)
c-------------------------------------------------------------------
      real alpha(n)

      data itmax / 10 /
c      data ttol  / 0.00001 /
      data ttol  / 0.000001 /
c      data ttol  / 0.0000000001 /
 
      
      t = to + delh/cpo

c      write(*,*)
c      write(*,*) 'gas_delh'

      do iter = 1, itmax
        call gassum(alpha,n, t, s,s_t, h,h_t, cp,r)
        res   = h - ho - delh
        res_t = h_t

        dt = -res/res_t

c        write(*,*) iter, t, dt

        if(abs(dt) .lt. ttol) go to 20

        t = t + dt
      enddo
      write(*,*) 'gas_delh: convergence failed.  dT =', dt

 20   continue
      p = po * exp(epol*(s-so)/r)

      return
      end ! gas_delh


      subroutine gas_delhd(alpha,n,
     &                     po, to, ho,so,cpo, ro, delh, epol,
     &                     p , t , h ,s ,cp , r ,
     &                     p_so, 
     &                     p_po,
     &                     p_ep,
     &                     p_ho, t_ho, h_ho, s_ho,
     &                     p_dh, t_dh, h_dh, s_dh,
     &                     p_al, t_al, h_al, s_al, cp_al, r_al)
c---------------------------------------------------------------------
c     Same as gas_delh, but also returns Jacobians w.r.t. po,to,delh
c---------------------------------------------------------------------
      real alpha(n)
      real p_al(n), t_al(n), h_al(n), s_al(n), cp_al(n), r_al(n)

      data itmax / 15 /
c      data ttol  / 0.00001 /
      data ttol  / 0.000001 /
c      data ttol  / 0.000000001 /
 
      
      t = to + delh/cpo
      t = max( t , 0.2*to )

c      write(*,*)
c      write(*,*) 'gas_delh'

      do iter = 1, itmax
        call gassum(alpha,n, t, s,s_t, h,h_t, cp,r)

c------ res( h(t,al), ho, dh )
        res   = h - ho - delh
        res_t = h_t

        dt = -res/res_t

        rlx = 1.0
        if(rlx*dt .lt. -0.4*t) rlx = -0.4*t/dt
        if(rlx*dt .gt.  0.6*t) rlx =  0.6*t/dt

cc        write(*,'(a,i3,f9.3,e13.4,f8.3)')' iter t dt rlx',iter,t,dt,rlx

        t = t + rlx*dt

        if(abs(dt) .lt. ttol) go to 20

      enddo
      write(*,*) 'gas_delh: convergence failed.  dT =', dt

 20   continue
      call gassumd(alpha,n, t, s,s_t, h,h_t, cp,cp_t, r)
      res_t = h_t

      res_ho = -1.0
      res_dh = -1.0
cc    res_al = hi

      t_ho = -res_ho/res_t
      t_dh = -res_dh/res_t
cc    t_al = -res_al/res_t

cc          t(ho,dh,al)
cc       h( t(ho,dh,al),al )
cc       s( t(ho,dh,al),al )
cc      cp( t(ho,dh,al),al )
cc       r( al )
cc    

c---- p( r(al), s( t(ho,dh,al),al ), so, po )
      p = po * exp(epol*(s-so)/r)
      p_r  = -p*epol*(s-so)/r**2
      p_s  =  p*epol/r
      p_so = -p*epol/r
      p_po = exp(epol*(s-so)/r)
      p_ep =  p*(s-so)/r

c---- p( t(ho,dh,al), so, po, al )
      p_t  = p_s*s_t
cc    p_al = p_s*si + p_r*ri

c---- p( t(ho,dh), so, po, al )
cc    p_al = p_s*si + p_r*ri + p_t*t_al
cc    p_al = p_s*si + p_r*ri + p_t*(-res_al/res_t)

c---- p( so,po,ho,dh, al )
ccc   p_so =            p_so
ccc   p_po =            p_po
      p_ho = p_t*t_ho
      p_dh = p_t*t_dh

c---- h( t(ho,dh,al),al )
      h_ho = h_t*t_ho
      h_dh = h_t*t_dh
cc    h_al = h_t*t_al + hi


c---- s( t(ho,dh,al),al )
      s_ho = s_t*t_ho
      s_dh = s_t*t_dh
cc    s_al = s_t*t_al + si

      do i = 1, n
        call gasfun(i,t, si,s_ti, hi,h_ti, cpi,ri)
        res_al = hi
        t_al(i) =      -res_al/res_t
        p_al(i) = p_t*(-res_al/res_t) + p_s*si + p_r*ri
        h_al(i) = h_t*(-res_al/res_t) + hi
        s_al(i) = s_t*(-res_al/res_t) + si
        cp_al(i) = cp_t*(-res_al/res_t) + cpi
        r_al(i) = ri
      enddo

      return
      end ! gas_delhd



      subroutine gas_burn(alpha,beta,gamma,n,ifuel, to,tf,t , f,lambda)
c------------------------------------------------------------------------
c     Calculates fuel/air mass fraction in combustion 
c     with specified start and end temperatures to,t .
c     Calculates mass fractions of post-combustion constituents
c
c  Input:
c     alpha(.)  mass fractions for air  constituents i = 1..n
c     beta(.)   mass fractions for fuel constituents i = 1..n
c     gamma(.)  mass fraction changes of air constituents due to combustion
c     n         number of constituents present, air is 1..n-1, fuel is n
c     ifuel     index specifying fuel molecule
c     to        starting air temperatur
c     tf        starting fuel temperature
c     t         temperature of combustion products
c
c  Output:
c     f          fuel/air mass fraction
c     lambda(.)  mass fractions for combustion product constituents
c------------------------------------------------------------------------
      real alpha(n), beta(n), gamma(n), lambda(n)

      nm = n-1
      call gassum(alpha,nm, to, so,s_t, ho,h_t, cpo,ro)
      call gassum(alpha,nm, t , sa,s_t, ha,h_t, cpa,ra)
      call gassum(gamma,nm, t , sc,s_t, hc,h_t, cpc,rb)
      call gassum(beta ,nm, tf, sf,s_t, hf,h_t, cpf,rf)

c---- add on fuel contribution to hf, which gassum cannot index via ifuel
      call gasfun(ifuel, tf, si,s_t, hi,h_t, cpi,ri)
      hf  = hf  + hi *beta(n)
      
      f = (ha - ho) / (hf - hc)

      do i = 1, n
        lambda(i) = (alpha(i) + f*gamma(i)) / (1.0 + f)
      enddo

      return
      end ! gas_burn


      subroutine gas_burnd(alpha,beta,gamma,n,ifuel, to,tf,t , f,lambda,
     &                     f_to, f_tf, f_t,
     &                     l_to, l_tf, l_t )
c------------------------------------------------------------------------
c     Same as gas_burn, but also returns derivatives.
c------------------------------------------------------------------------
      real alpha(n), beta(n), gamma(n), lambda(n)
      real l_to(n), l_tf(n), l_t(n)

      real l_f

      nm = n-1
      call gassum(alpha,nm, to, so,s_t, ho,ho_to, cpo,ro)
      call gassum(alpha,nm, t , sa,s_t, ha,ha_t , cpa,ra)
      call gassum(gamma,nm, t , sc,s_t, hc,hc_t , cpc,rb)
      call gassum(beta ,nm, tf, sf,s_t, hf,hf_tf, cpf,rf)

c---- add on fuel contribution to hf, which gassum cannot index via ifuel
      call gasfun(ifuel, tf, si,s_t, hi,hi_tf, cpi,ri)
      hf    = hf    + hi   *beta(n)
      hf_tf = hf_tf + hi_tf*beta(n)

      f = (ha-ho)/(hf-hc)
      f_ho = -1.0/(hf-hc)
      f_ha =  1.0/(hf-hc)
      f_hc =    f/(hf-hc)
      f_hf =   -f/(hf-hc)

      f_to = f_ho*ho_to
      f_tf = f_hf*hf_tf
      f_t  = f_ha*ha_t
     &     + f_hc*hc_t

      do i = 1, n
        lambda(i) = (alpha(i) + f*gamma(i)) / (1.0 + f)
        l_f = (gamma(i) - lambda(i)) / (1.0 + f)
        l_to(i) = l_f*f_to
        l_tf(i) = l_f*f_tf
        l_t(i)  = l_f*f_t
      enddo

      return
      end ! gas_burnd


      subroutine gas_mach(alpha,n, po,to,ho,so,cpo,ro, mo, m, epol,
     &                             p ,t ,h ,s ,cp ,r )
c-------------------------------------------------------------------
c     Calculates state change for a specified Mach number change.
c     The constant-cp equivalent is the usual isentropic relations,
c     but with epol included.
c
c       g = cp/(cp-r)
c       gexp = (g-1)/(g*epol)
c       tau = (1 + 0.5*(g-1)*mo^2) / (1 + 0.5*(g-1)*m^2)
c       pi = tau^(1/gexp)
c       p = po * pi
c       t = to * tau
c       (h-hf) = (ho-hf) * tau
c
c
c  Input:
c     alpha(.)  mass fractions for gas constituents i = 1..n
c     n         number of constituents present
c
c     po        starting pressure
c     to        starting temperature
c     ho        starting enthalpy
c     so        starting entropy-complement
c     cpo       starting specific heat
c     ro        starting gas constant
c     mo        starting Mach number
c     m         ending Mach number
c
c     epol      polytropic efficiency of change process  , if compression
c     epol   1/(polytropic efficiency of change process) , if expansion
c
c  Output:
c     p       ending pressure
c     t       ending temperature
c     h       ending enthalpy
c     s       ending entropy-complement
c     cp      ending specific heat
c     r       ending gas constant (this will be the same as starting ro)
c-------------------------------------------------------------------
      real alpha(n)
      real mo, m

      data itmax / 10 /
c      data ttol  / 0.00001 /
      data ttol  / 0.000001 /
c      data ttol  / 0.0000000001 /
 
      uosq = mo**2 * cpo*ro/(cpo-ro) * to
c
c---- initial guess for temperature, using constant-gamma relation
      t = to * (1.0 + 0.5*ro/(cpo-ro)*mo**2)
     &       / (1.0 + 0.5*ro/(cpo-ro)*m**2 )

c      write(*,*)
c      write(*,*) 'gas_mach'

c---- Newton iteration for actual temperature
      do iter = 1, itmax
        call gassum(alpha,n, t, s,s_t, h,h_t, cp,r)
        cp_t = 0.    ! could evaluate this from the cp(T) splines (later)
c
c------ usq( t, cp(t,al), r(al) , m )
        usq    = m**2 * cp*r/(cp-r) * t
        usq_t  = m**2 * cp*r/(cp-r)
        usq_cp = m**2 *    r/(cp-r) * t - usq/(cp-r)

c------ res( h(t), usq( t, cp(t) ) )
        res   = h   + 0.5*usq - ho - 0.5*uosq
        res_t = h_t + 0.5*(usq_t + usq_cp*cp_t)
c
        dt = -res/res_t

c        write(*,*) iter, t, dt

        if(abs(dt) .lt. ttol) go to 20

        t = t + dt
      enddo
      write(*,*) 'gas_mach: convergence failed.  dT =', dt

 20   continue
      p = po * exp(epol*(s-so)/ro)

      return
      end ! gas_mach



      subroutine gas_machd(alpha,n, po,to,ho,so,cpo,ro, mo, m, epol,
     &                             p ,t ,h ,s ,cp ,r,
     &                             p_so, 
     &                             p_po,
     &                             p_ep,
     &                             p_to, t_to, h_to, s_to,
     &                             p_ho, t_ho, h_ho, s_ho,
     &                             p_m , t_m , h_m , s_m ,
     &                             p_al, t_al, h_al, s_al, cp_al, r_al)
c-------------------------------------------------------------------
c     Same as gas_mach, but also returns derivatives
c-------------------------------------------------------------------
      real alpha(n)
      real p_al(n), t_al(n), h_al(n), s_al(n), cp_al(n), r_al(n)
      real mo, m

      data itmax / 10 /
c      data ttol / 0.00001 /
      data ttol / 0.000001 /
c      data ttol / 0.0000000001 /
 
      uosq     = mo**2 * cpo*ro/(cpo-ro) * to
      uosq_mo  = mo*2.0* cpo*ro/(cpo-ro) * to
      uosq_cpo = mo**2 *     ro/(cpo-ro) * to - uosq/(cpo-ro)
      uosq_ro  = mo**2 * cpo   /(cpo-ro) * to + uosq/(cpo-ro)
      uosq_to  = mo**2 * cpo*ro/(cpo-ro)

c---- initial guess for temperature, using constant-gamma relation
      t = to * (1.0 + 0.5*ro/(cpo-ro)*mo**2)
     &       / (1.0 + 0.5*ro/(cpo-ro)*m**2 )

c---- Newton iteration for actual temperature
      do iter = 1, itmax
cc      call gassum(alpha,n, t, s,s_t, h,h_t, cp,r)
        call gassumd(alpha,n, t, s,s_t, h,h_t, cp,cp_t, r)
cc      cp_t = 0.    ! could evaluate this from the cp(T) splines (later)
c
c------ usq( m, cp(t,al), r(al) , t )
        usq    = m**2 * cp*r/(cp-r) * t
        usq_m  = m*2.0* cp*r/(cp-r) * t
        usq_cp = m**2 *    r/(cp-r) * t - usq/(cp-r)
        usq_r  = m**2 * cp  /(cp-r) * t + usq/(cp-r)
        usq_t  = m**2 * cp*r/(cp-r)

c------ res( h(t,al), usq( m, cp(t,al), r(al), t ), ho, uosq(mo,cpo,ro,to) )
        res   = h   + 0.5*usq - ho - 0.5*uosq
        res_t = h_t + 0.5*(usq_t + usq_cp*cp_t)
c
        dt = -res/res_t

c        write(*,*) iter, t, dt

        if(abs(dt) .lt. ttol) go to 20

        t = t + dt
      enddo
      write(*,*) 'gas_mach: convergence failed.  dT =', dt

 20   continue

c---- res( h(t,al), usq( m, cp(t,al), r(al), t ), ho, uosq(mo,cpo,ro,to) )
      res_m   = 0.5*usq_m
      res_ho  = -1.0
      res_mo  = -0.5*uosq_mo
      res_cpo = -0.5*uosq_cpo
      res_ro  = -0.5*uosq_ro
      res_to  = -0.5*uosq_to
cc    res_al = hi + 0.5*(usq_cp*cpi + usq_r*ri)

cc    t(to,ho,cpo,ro, mo, m, al)
      t_to  = -res_to /res_t
      t_ho  = -res_ho /res_t
      t_cpo = -res_cpo/res_t
      t_ro  = -res_ro /res_t
      t_mo  = -res_mo /res_t
      t_m   = -res_m  /res_t
cc    t_al  = -res_al /res_t

c---- p( r(al), s( t(to,ho,cpo,ro,mo,m,al),al ), so, po )
      p = po * exp(epol*(s-so)/r)
      p_r  = -p*epol*(s-so)/r**2
      p_s  =  p*epol/r
      p_so = -p*epol/r
      p_po = exp(epol*(s-so)/r)
      p_ep =  p*(s-so)/r

c---- p( t(to,ho,cpo,ro,mo,m,al), so, po, al )
      p_t  = p_s*s_t
cc    p_al = p_s*si + p_r*ri

c---- p( t(to,ho,cpo,ro,mo,m), so, po, al )
cc    p_al = p_s*si + p_r*ri + p_t*t_al

c---- p( so, po, to,ho,cpo,ro,mo,m, al )
ccc   p_so =            p_so
ccc   p_po =            p_po
      p_to  = p_t*t_to
      p_ho  = p_t*t_ho
      p_cpo = p_t*t_cpo
      p_ro  = p_t*t_ro
      p_mo  = p_t*t_mo
      p_m   = p_t*t_m
ccc   p_al  =           p_al

c---- h( t(to,ho,so,cpo,ro,mo,m,al),al )
      h_to  = h_t*t_to
      h_ho  = h_t*t_ho
      h_cpo = h_t*t_cpo
      h_ro  = h_t*t_ro
      h_mo  = h_t*t_mo
      h_m   = h_t*t_m
cc    h_al  = h_t*t_al + hi

c---- s( t(to,ho,so,cpo,ro,mo,m,al),al )
      s_to  = s_t*t_to
      s_ho  = s_t*t_ho
      s_cpo = s_t*t_cpo
      s_ro  = s_t*t_ro
      s_mo  = s_t*t_mo
      s_m   = s_t*t_m
cc    s_al  = s_t*t_al + si

      do i = 1, n
        call gasfun(i,t, si,s_ti, hi,h_ti, cpi,ri)
        res_al = hi + 0.5*(usq_cp*cpi + usq_r*ri)
        t_al(i) =      -res_al/res_t
        p_al(i) = p_t*(-res_al/res_t) + p_s*si + p_r*ri
        h_al(i) = h_t*(-res_al/res_t) + hi
        s_al(i) = s_t*(-res_al/res_t) + si
        cp_al(i) = cp_t*(-res_al/res_t) + cpi
        r_al(i) = ri
      enddo

      return
      end ! gas_machd


      subroutine gas_mass(alpha,n, po,to,ho,so,cpo,ro, mflux,Mguess,
     &                             p ,t ,h ,s ,cp ,r )
c-------------------------------------------------------------------
c     Calculates state a specified mass flux.
c     Mguess specifies the initial guess, and also selects
c     either the subsonic or the supersonic branch.
c
c  Input:
c     alpha(.)  mass fractions for gas constituents i = 1..n
c     n         number of constituents present
c
c     po        total pressure
c     to        total temperature
c     ho        total enthalpy
c     so        total entropy-complement
c     cpo       total specific heat
c     ro        total gas constant
c
c     mflux     specified mass flux = rho u = mdot/A
c     Mguess    specifies the initial guess for the static quantities
c
c  Output:
c     p       static pressure
c     t       static temperature
c     h       static enthalpy
c     s       static entropy-complement
c     cp      static specific heat
c     r       static gas constant (this will be the same as total ro)
c-------------------------------------------------------------------
      real alpha(n)
      real mflux, Mguess

      data itmax / 25 /
c      data ttol  / 0.00001 /
      data ttol  / 0.000001 /
c      data ttol  / 0.0000000001 /
 
      
      t = to / (1 + 0.5*ro/(cpo-ro) * Mguess**2)

c      write(*,*)
c      write(*,*) 'gas_mass'

      do iter = 1, itmax
        call gassum(alpha,n, t, s,s_t, h,h_t, cp,r)

        p   = po * exp((s-so)/r)
        p_t = p * s_t/r

        res   = 2.0 * p**2 / (t*r)**2 * (ho-h  ) -    mflux**2
        res_t = 4.0 * p*p_t/ (t*r)**2 * (ho-h  )
     &        - 4.0 * p**2 / (t*r)**2 * (ho-h  ) / t
     &        + 2.0 * p**2 / (t*r)**2 * (  -h_t)

        dt = -res/res_t

        rlx = 1.0
        if(rlx*dt .gt. 0.8*(to-t)) rlx = 0.8*(to-t)/dt


c        usq = max( 2.0*(ho-h) , 1.0e-10*ho )
c        u = sqrt(usq)
c        a = sqrt(t*cp*r/(cp-r))
c        write(*,'(1x,i3,2f10.4,e12.4,g12.4)') iter, u/a, t, dt, res_t

        if(abs(dt) .lt. ttol) go to 20

        t = t + rlx*dt
      enddo
      write(*,*) 'gas_mass: convergence failed.  dT =', dt

 20   continue

      return
      end ! gas_mass


      subroutine gassum(alpha,n, t, s,s_t, h,h_t, cp,r)
c-----------------------------------------------------------------
c     Calculates all gas-mixture properties at specified 
c     temperature T, and mixing fractions alpha(.)
c
c  Input:
c     alpha(.)  mass fractions for gas constituents i = 1..n
c     n         number of gas constituents
c     t         temperature T,  Kelvin
c
c  Output:
c     s      entropy-complement function s(T)
c     s_t    ds/dT
c     h      complete enthalpy function h(T)
c     h_t    dh/dT
c     cp     specific heat cp(T)
c     r      ideal-gas constant R
c-----------------------------------------------------------------
      real alpha(*)
      s  = 0.
      h  = 0.
      cp = 0.
      r  = 0.
      s_t = 0.
      h_t = 0.
      do i = 1, n
        call gasfun(i,t, si,s_ti, hi,h_ti, cpi,ri)
        s   = s   + si  *alpha(i)
        h   = h   + hi  *alpha(i)
        cp  = cp  + cpi *alpha(i)
        r   = r   + ri  *alpha(i)
        s_t = s_t + s_ti*alpha(i)
        h_t = h_t + h_ti*alpha(i)
      enddo
      return
      end ! gassum


      subroutine gassumd(alpha,n, t, s,s_t, h,h_t, cp,cp_t, r)
c-----------------------------------------------------------------
c     Same as gassum, but also returns cp_t
c-----------------------------------------------------------------
      real alpha(*)
      s  = 0.
      h  = 0.
      cp = 0.
      r  = 0.
      s_t = 0.
      h_t = 0.
      do i = 1, n
        call gasfun(i,t, si,s_ti, hi,h_ti, cpi,ri)
        s   = s   + si  *alpha(i)
        h   = h   + hi  *alpha(i)
        cp  = cp  + cpi *alpha(i)
        r   = r   + ri  *alpha(i)
        s_t = s_t + s_ti*alpha(i)
        h_t = h_t + h_ti*alpha(i)
      enddo

c---- set dcp/dt via finite difference
      dt = 0.01
      cp_t = 0.
      do i = 1, n
        call gasfun(i,t-dt, si,s_ti, hi,h_ti, cpi1,ri)
        call gasfun(i,t+dt, si,s_ti, hi,h_ti, cpi2,ri)
        cp_ti = (cpi2-cpi1)*0.5/dt
        cp_t  = cp_t  + cp_ti *alpha(i)
      enddo

      return
      end ! gassumd



      subroutine gasfuel(ifuel,gamma,n)
c---------------------------------------------------------------
c     Returns mass fraction of constituent changes
c     as a result of combustion with atmospheric oxygen
c
c  Input:
c     ifuel   index of fuel  (see subroutine gasfun)
c     n       number of constituents in reaction
c
c  Output:
c     gamma(.)  mass fraction changes due to reaction for i = 1..n
c
c---------------------------------------------------------------
      real gamma(n)
c
      integer nchon(4)
      real wchon(4)
c
c---- molar weights of C, H, O, N atoms
      data wchon /  12.01078 , 1.00795 , 15.99943 , 14.00672 /
c
      data in2, io2, ico2, ih2o / 1, 2, 3, 4 /
c
c---- get number of atoms in fuel molecule
      call gaschem(ifuel,nchon)
c
c---- molar weight of fuel
      wfuel = wchon(1)*float(nchon(1))
     &      + wchon(2)*float(nchon(2))
     &      + wchon(3)*float(nchon(3))
     &      + wchon(4)*float(nchon(4))
c
c---- molar weights of combustion gases, to balance atoms in reaction equation
c-     Wfuel + W_O2  =  W_N2 + W_CO2 + W_H2O + W_N2
      wn2  = (           wchon(4)*2.0) *  float(nchon(4))*0.5
      wco2 = (wchon(1) + wchon(3)*2.0) *  float(nchon(1))
      wh2o = (wchon(3) + wchon(2)*2.0) *  float(nchon(2))*0.5
      wo2  = (           wchon(3)*2.0) *( float(nchon(1))
     &                                  + float(nchon(2))*0.25
     &                                  - float(nchon(3))*0.5 )
c
c---- store weight fractions of reaction gases, relative to fuel weight
c-    (left side is negative, right side is positive)
      do i = 1, n
        gamma(i) = 0.0
      enddo
      gamma(in2 ) =  wn2/wfuel
      gamma(io2 ) = -wo2/wfuel
      gamma(ico2) = wco2/wfuel
      gamma(ih2o) = wh2o/wfuel
c
      return
      end ! gasfuel
