
      program gasdtest
c-------------------------------------------------------------------------
c     Tests the derivatives output by the various gas_*d routines 
c     for air and for the selected gas (via ifuel).
c
c     Compares the returned derivatives which are computed analytically
c     with finite-differenced derivatives.  Tags any values which differ.
c-------------------------------------------------------------------------
      implicit real (a-h,l,m,o-z)

      parameter (n=6)
      real alpha(n), beta(n), gamma(n), lambda(n)

      real lam_t3(n), lam_tf(n), lam_t4(n)

      real t_al(n),
     &     p_al(n),
     &     h_al(n),
     &     s_al(n),
     &     cp_al(n),
     &     r_al(n)


      real alpha$(n), lambda$(n)
      real lam_t3$(n), lam_tf$(n), lam_t4$(n)
      real t_al$(n),
     &     p_al$(n),
     &     h_al$(n),
     &     s_al$(n),
     &     cp_al$(n),
     &     r_al$(n)



      real m0
      real alf(n)

      character*2 ss1, ss2, ss3, ss4, ss5, ss6
c
c---- air fractions  N2      O2      CO2    H2O      Ar       fuel
      data alpha / 0.781 , 0.209 , 0.0004, 0.0   , 0.00965,  0.0 /
c
c---- fuel fractions
      data beta  / 0.0   , 0.0   , 0.0   , 0.0   , 0.0    ,  1.0 /
c
c---- finite-difference delta
      eps = 1.0e-6
      eps = 1.0e-7

      nair = n-1

      do i = 1, n
        alpha$(i) = alpha(i)
      enddo
c
c=====================================================================
c---- set fuel, put in last constituent slot n
c      ifuel = 11  ! methane
c      ifuel = 12  ! ethane
c      ifuel = 13  ! propane
c      ifuel = 14  ! butane
c      ifuel = 18  ! octane  
c      ifuel = 24  ! kerosene  (C14H30)
c
c      hfuel = 55.0e6  ! methane 
c      hfuel = 52.0e6  ! ethane  
c      hfuel = 49.0e6  ! propane 
c      hfuel = 46.0e6  ! butane  
c      hfuel = 44.0e6  ! octane  
c      hfuel = 43.0e6  ! kerosene  (C14H30)
c
      ifuel = 11
      hfuel = 55.0e6
c
      call gasfuel(ifuel,gamma,n)
c
cc---------------------------------------------------------------------
cc---- evaluate properties over temperatures for chosen constituent i
cc-     (see gasfun.f header for list of constituents)
c      i = 1
c      nt = 501
c      do it = 1, nt
c        t = 250.0 + (2000.0-250.0)*float(it-1)/float(nt-1)
c        tl = log(t)
c        call gasfun(i,t, s,dsdt,h,dhdt,cp,r)
c        write(1,4444) t,tl,s,dsdt,h,dhdt,cp,r
c 4444   format(1x,9g14.6)
c      enddo
c      stop
cc---------------------------------------------------------------------

c===============================================================
c---- static freestream
      t0 = 215.0
      p0 = 0.22e5
      m0 = 0.80

      call gassum(alpha,nair, t0, s0,dsdt,h0,dhdt,cp0,r0)
      gam0 = cp0/(cp0-r0)
      a0 = sqrt(gam0*r0*t0)
      u0 = m0*a0

c===============================================================
c---- total freestream
      hspec = h0 + 0.5*u0**2
      tguess = t0*(1.0 + 0.5*(gam0-1.0)*m0**2)
      call gas_tset(alpha,nair, hspec,tguess, t1)
c 
      call gassum(alpha,nair, t1, s1,dsdt,h1,dhdt,cp1,r1)
      p1 = p0 * exp((s1-s0)/r1)
c
c===============================================================
c---- diffuser
      hspec = h0 + 0.5*u0**2
      tguess = t0*(1.0 + 0.5*(gam0-1.0)*m0**2)
      call gas_tset(alpha,nair, hspec,tguess, t2)
c 
      epold = 0.99
      call gassum(alpha,nair, t2, s2,dsdt,h2,dhdt,cp2,r2)
      p2 = p0 * exp(epold*(s2-s0)/r2)
      gam2 = cp2/(cp2-r2)
c

 2010 format(1x,  a,g16.8,a,2x )
 2020 format(1x,2(a,g16.8,a,2x))
 2030 format(1x,3(a,g16.8,a,2x))
 2040 format(1x,4(a,g16.8,a,2x))
 2060 format(1x,6(a,g16.8,a,2x))

c===============================================================
c---- compressor
      pic = 20.0
      epolc = 0.95
      call gassum(alpha,nair, t2, s2,dsdt,h2,dhdt,cp2,r2)

      write(*,*) 
      write(*,*) 'gas_pratd...'
      call gas_pratd(alpha,nair,
     &               p2,t2,h2,s2,cp2,r2, pic,epolc,
     &               p3,t3,h3,s3,cp3,r3,
     &               p3_p2,
     &               p3_s2, t3_s2, h3_s2, s3_s2,
     &               p3_pi, t3_pi, h3_pi, s3_pi,
     &               p3_ep, t3_ep, h3_ep, s3_ep,
     &               p_al , t_al , h_al , s_al , cp_al, r_al)
      p2$ = p2
      t2$ = t2
      h2$ = h2
      s2$ = s2
      cp2$ = cp2
      r2$ = r2
      pic$ = pic
      epolc$ = epolc

      dp2  = eps*p2
      ds2  = eps*cp2
      dpic = eps*pic
      dep  = eps
      dal  = eps

      p2$ = p2 + dp2
      call gas_pratd(alpha$,nair,
     &               p2$,t2$,h2$,s2$,cp2$,r2$, pic$,epolc$,
     &               p3$,t3$,h3$,s3$,cp3$,r3$,
     &               p3_p2$,
     &               p3_s2$, t3_s2$, h3_s2$, s3_s2$,
     &               p3_pi$, t3_pi$, h3_pi$, s3_pi$,
     &               p3_ep$, t3_ep$, h3_ep$, s3_ep$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      p2$ = p2
      dd1 = (p3$-p3)/dp2
      aa1 = (p3_p2$+p3_p2)*0.5
      call compare(ss1,aa1,dd1)
      write(*,*)
      write(*,2010) 'p3_p2',aa1,ss1
      write(*,2010) '     ',dd1,ss1

      s2$ = s2 + ds2
      call gas_pratd(alpha$,nair,
     &               p2$,t2$,h2$,s2$,cp2$,r2$, pic$,epolc$,
     &               p3$,t3$,h3$,s3$,cp3$,r3$,
     &               p3_p2$,
     &               p3_s2$, t3_s2$, h3_s2$, s3_s2$,
     &               p3_pi$, t3_pi$, h3_pi$, s3_pi$,
     &               p3_ep$, t3_ep$, h3_ep$, s3_ep$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      s2$ = s2
      dd1 = (p3$-p3)/ds2
      dd2 = (t3$-t3)/ds2
      dd3 = (h3$-h3)/ds2
      dd4 = (s3$-s3)/ds2
      aa1 = (p3_s2$+p3_s2)*0.5
      aa2 = (t3_s2$+t3_s2)*0.5
      aa3 = (h3_s2$+h3_s2)*0.5
      aa4 = (s3_s2$+s3_s2)*0.5
      call compare(ss1,aa1,dd1)
      call compare(ss2,aa2,dd2)
      call compare(ss3,aa3,dd3)
      call compare(ss4,aa4,dd4)
      write(*,*)
      write(*,2040) 'p3_s2',aa1,ss1, 'T3_s2',aa2,ss2,
     &              'h3_s2',aa3,ss3, 's3_s2',aa4,ss4
      write(*,2040) '     ',dd1,ss1, '     ',dd2,ss2,
     &              '     ',dd3,ss3, '     ',dd4,ss4

      pic$ = pic + dpic
      call gas_pratd(alpha$,nair,
     &               p2$,t2$,h2$,s2$,cp2$,r2$, pic$,epolc$,
     &               p3$,t3$,h3$,s3$,cp3$,r3$,
     &               p3_p2$,
     &               p3_s2$, t3_s2$, h3_s2$, s3_s2$,
     &               p3_pi$, t3_pi$, h3_pi$, s3_pi$,
     &               p3_ep$, t3_ep$, h3_ep$, s3_ep$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      pic$ = pic

      dd1 = (p3$-p3)/dpic
      dd2 = (t3$-t3)/dpic
      dd3 = (h3$-h3)/dpic
      dd4 = (s3$-s3)/dpic
      aa1 = (p3_pi$+p3_pi)*0.5
      aa2 = (t3_pi$+t3_pi)*0.5
      aa3 = (h3_pi$+h3_pi)*0.5
      aa4 = (s3_pi$+s3_pi)*0.5
      call compare(ss1,aa1,dd1)
      call compare(ss2,aa2,dd2)
      call compare(ss3,aa3,dd3)
      call compare(ss4,aa4,dd4)
      write(*,*)
      write(*,2040) 'p3_pi',aa1,ss1, 'T3_pi',aa2,ss2,
     &              'h3_pi',aa3,ss3, 's3_pi',aa4,ss4
      write(*,2040) '     ',dd1,ss1, '     ',dd2,ss2,
     &              '     ',dd3,ss3, '     ',dd4,ss4


      epolc$ = epolc + dep
      call gas_pratd(alpha$,nair,
     &               p2$,t2$,h2$,s2$,cp2$,r2$, pic$,epolc$,
     &               p3$,t3$,h3$,s3$,cp3$,r3$,
     &               p3_p2$,
     &               p3_s2$, t3_s2$, h3_s2$, s3_s2$,
     &               p3_pi$, t3_pi$, h3_pi$, s3_pi$,
     &               p3_ep$, t3_ep$, h3_ep$, s3_ep$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      epolc$ = epolc

      dd1 = (p3$-p3)/dep
      dd2 = (t3$-t3)/dep
      dd3 = (h3$-h3)/dep
      dd4 = (s3$-s3)/dep
      aa1 = (p3_ep$+p3_ep)*0.5
      aa2 = (t3_ep$+t3_ep)*0.5
      aa3 = (h3_ep$+h3_ep)*0.5
      aa4 = (s3_ep$+s3_ep)*0.5
      call compare(ss1,aa1,dd1)
      call compare(ss2,aa2,dd2)
      call compare(ss3,aa3,dd3)
      call compare(ss4,aa4,dd4)
      write(*,*)
      write(*,2040) 'p3_ep',aa1,ss1, 'T3_ep',aa2,ss2,
     &              'h3_ep',aa3,ss3, 's3_ep',aa4,ss4
      write(*,2040) '     ',dd1,ss1, '     ',dd2,ss2,
     &              '     ',dd3,ss3, '     ',dd4,ss4


      i = 1
      alpha$(i) = alpha(i) + dal
      call gas_pratd(alpha$,nair,
     &               p2$,t2$,h2$,s2$,cp2$,r2$, pic$,epolc$,
     &               p3$,t3$,h3$,s3$,cp3$,r3$,
     &               p3_p2$,
     &               p3_s2$, t3_s2$, h3_s2$, s3_s2$,
     &               p3_pi$, t3_pi$, h3_pi$, s3_pi$,
     &               p3_ep$, t3_ep$, h3_ep$, s3_ep$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      alpha$(i) = alpha(i)

      dd1 = (p3$-p3)/dal
      dd2 = (t3$-t3)/dal
      dd3 = (h3$-h3)/dal
      dd4 = (s3$-s3)/dal
      dd5 = (cp3$-cp3)/dal
      dd6 = (r3$-r3)/dal
      aa1 = (p_al$(i)+p_al(i))*0.5
      aa2 = (t_al$(i)+t_al(i))*0.5
      aa3 = (h_al$(i)+h_al(i))*0.5
      aa4 = (s_al$(i)+s_al(i))*0.5
      aa5 = (cp_al$(i)+cp_al(i))*0.5
      aa6 = (r_al$(i)+r_al(i))*0.5
      call compare(ss1,aa1,dd1)
      call compare(ss2,aa2,dd2)
      call compare(ss3,aa3,dd3)
      call compare(ss4,aa4,dd4)
      call compare(ss5,aa5,dd5)
      call compare(ss6,aa6,dd6)
      write(*,*)
      write(*,2060) 'p3_al',aa1,ss1, 'T3_al',aa2,ss2,
     &              'h3_al',aa3,ss3, 's3_al',aa4,ss4,
     &              'c3_al',aa5,ss5, 'r3_al',aa6,ss6
      write(*,2060) '     ',dd1,ss1, '     ',dd2,ss2,
     &              '     ',dd3,ss3, '     ',dd4,ss4,
     &              '     ',dd5,ss5, '     ',dd6,ss6


c===============================================================
c---- combustor
      write(*,*) 
      write(*,*) 'gas_burnd...'
      tf = t3
      t4 = 2000.0
      pib = 0.99
      call gas_burnd(alpha,beta,gamma,n,ifuel, t3,tf,t4, 
     &               f,lambda,
     &               f_t3  ,f_tf  ,f_t4,
     &               lam_t3,lam_tf,lam_t4)
      t3$ = t3
      tf$ = tf
      t4$ = t4

      dt = eps*t3

      t3$ = t3 + dt
      call gas_burnd(alpha,beta,gamma,n,ifuel, t3$,tf$,t4$, 
     &               f$,lambda$,
     &               f_t3$  ,f_tf$  ,f_t4$,
     &               lam_t3$,lam_tf$,lam_t4$ )
      t3$ = t3

      i = 3
      dd1 = (f$-f)/dt
      dd2 = (lambda$(i)-lambda(i))/dt
      aa1 = (f_t3$+f_t3)*0.5
      aa2 = (lam_t3$(i)+lam_t3(i))*0.5
      call compare(ss1,aa1,dd1)
      call compare(ss2,aa2,dd2)
      write(*,*)
      write(*,2020) 'f_t3',aa1,ss1, 'lam_t3',aa2,ss2
      write(*,2020) '    ',dd1,ss1, '      ',dd2,ss2

      tf$ = tf + dt
      call gas_burnd(alpha,beta,gamma,n,ifuel, t3$,tf$,t4$, 
     &               f$,lambda$,
     &               f_t3$  ,f_tf$  ,f_t4$,
     &               lam_t3$,lam_tf$,lam_t4$ )
      tf$ = tf

      i = 3
      dd1 = (f$-f)/dt
      dd2 = (lambda$(i)-lambda(i))/dt
      aa1 = (f_tf$+f_tf)*0.5
      aa2 = (lam_tf$(i)+lam_tf(i))*0.5
      call compare(ss1,aa1,dd1)
      call compare(ss2,aa2,dd2)
      write(*,*)
      write(*,2020) 'f_tf',aa1,ss1, 'lam_tf',aa2,ss2
      write(*,2020) '    ',dd1,ss1, '      ',dd2,ss2

      t4$ = t4 + dt
      call gas_burnd(alpha,beta,gamma,n,ifuel, t3$,tf$,t4$, 
     &               f$,lambda$,
     &               f_t3$  ,f_tf$  ,f_t4$,
     &               lam_t3$,lam_tf$,lam_t4$ )
      t4$ = t4

      i = 3
      dd1 = (f$-f)/dt
      dd2 = (lambda$(i)-lambda(i))/dt
      aa1 = (f_t4$+f_t4)*0.5
      aa2 = (lam_t4$(i)+lam_t4(i))*0.5
      call compare(ss1,aa1,dd1)
      call compare(ss2,aa2,dd2)
      write(*,*)
      write(*,2020) 'f_t4',aa1,ss1, 'lam_t4',aa2,ss2
      write(*,2020) '    ',dd1,ss1, '      ',dd2,ss2



      call gassum(lambda,nair, t4, s4,dsdt,h4,dhdt,cp4,r4)
      p4 = p3

c===============================================================
c---- turbine
      write(*,*) 
      write(*,*) 'gas_delhd...'

      delh = -(h3-h2) / (1.0 + f)
      epolt = 0.94

      epi = 1.0/epolt
      call gas_delhd(lambda,nair,
     &               p4,t4,h4,s4,cp4,r4, delh,epi,
     &               p5,t5,h5,s5,cp5,r5,
     &               p5_s4,
     &               p5_p4,
     &               p5_epi,
     &               p5_h4, t5_h4, h5_h4, s5_h4,
     &               p5_dh, t5_dh, h5_dh, s5_dh,
     &               p_al , t_al , h_al , s_al , cp_al, r_al)
      p5_ep = p5_epi*(-epi/epolt)

      p4$ = p4
      t4$ = t4
      h4$ = h4
      s4$ = s4
      cp4$ = cp4
      r4$ = r4
      delh$ = delh
      epi$ = epi
      do i = 1, n
        lambda$(i) = lambda(i)
      enddo

      ds4 = eps*cp4
      dp4 = eps*p4
      dh4 = eps*cp4*t4
      ddh = eps*cp4*t4
      dep = eps
      dal = eps


      s4$ = s4 + ds4
      call gas_delhd(lambda$,nair,
     &               p4$,t4$,h4$,s4$,cp4$,r4$,delh$,epi$,
     &               p5$,t5$,h5$,s5$,cp5$,r5$,
     &               p5_s4$,
     &               p5_p4$,
     &               p5_epi$,
     &               p5_h4$, t5_h4$, h5_h4$, s5_h4$,
     &               p5_dh$, t5_dh$, h5_dh$, s5_dh$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      s4$ = s4

      dd1 = (p5$-p5)/ds4
      aa1 = (p5_s4$+p5_s4)*0.5
      call compare(ss1,aa1,dd1)
      write(*,*)
      write(*,2010) 'p5_s4',aa1,ss1
      write(*,2010) '     ',dd1,ss1

      p4$ = p4 + dp4
      call gas_delhd(lambda$,nair,
     &               p4$,t4$,h4$,s4$,cp4$,r4$,delh$,epi$,
     &               p5$,t5$,h5$,s5$,cp5$,r5$,
     &               p5_s4$,
     &               p5_p4$,
     &               p5_epi$,
     &               p5_h4$, t5_h4$, h5_h4$, s5_h4$,
     &               p5_dh$, t5_dh$, h5_dh$, s5_dh$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      p4$ = p4

      dd1 = (p5$-p5)/dp4
      aa1 = (p5_p4$+p5_p4)*0.5
      call compare(ss1,aa1,dd1)
      write(*,*)
      write(*,2010) 'p5_p4',aa1,ss1
      write(*,2010) '     ',dd1,ss1

      h4$ = h4 + dh4
      call gas_delhd(lambda$,nair,
     &               p4$,t4$,h4$,s4$,cp4$,r4$,delh$,epi$,
     &               p5$,t5$,h5$,s5$,cp5$,r5$,
     &               p5_s4$,
     &               p5_p4$,
     &               p5_epi$,
     &               p5_h4$, t5_h4$, h5_h4$, s5_h4$,
     &               p5_dh$, t5_dh$, h5_dh$, s5_dh$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      h4$ = h4

      dd1 = (p5$-p5)/dh4
      dd2 = (t5$-t5)/dh4
      dd3 = (h5$-h5)/dh4
      dd4 = (s5$-s5)/dh4
      aa1 = (p5_h4$+p5_h4)*0.5
      aa2 = (t5_h4$+t5_h4)*0.5
      aa3 = (h5_h4$+h5_h4)*0.5
      aa4 = (s5_h4$+s5_h4)*0.5
      call compare(ss1,aa1,dd1)
      call compare(ss2,aa2,dd2)
      call compare(ss3,aa3,dd3)
      call compare(ss4,aa4,dd4)
      write(*,*)
      write(*,2040) 'p5_h4',aa1,ss1, 't5_h4',aa2,ss2,
     &              'h5_h4',aa3,ss3, 's5_h4',aa4,ss4
      write(*,2040) '     ',dd1,ss1, '     ',dd2,ss2,
     &              '     ',dd3,ss3, '     ',dd4,ss4


      delh$ = delh + ddh
      call gas_delhd(lambda$,nair,
     &               p4$,t4$,h4$,s4$,cp4$,r4$,delh$,epi$,
     &               p5$,t5$,h5$,s5$,cp5$,r5$,
     &               p5_s4$,
     &               p5_p4$,
     &               p5_epi$,
     &               p5_h4$, t5_h4$, h5_h4$, s5_h4$,
     &               p5_dh$, t5_dh$, h5_dh$, s5_dh$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      delh$ = delh

      dd1 = (p5$-p5)/ddh
      dd2 = (t5$-t5)/ddh
      dd3 = (h5$-h5)/ddh
      dd4 = (s5$-s5)/ddh
      aa1 = (p5_dh$+p5_dh)*0.5
      aa2 = (t5_dh$+t5_dh)*0.5
      aa3 = (h5_dh$+h5_dh)*0.5
      aa4 = (s5_dh$+s5_dh)*0.5
      call compare(ss1,aa1,dd1)
      call compare(ss2,aa2,dd2)
      call compare(ss3,aa3,dd3)
      call compare(ss4,aa4,dd4)
      write(*,*)
      write(*,2040) 'p5_dh',aa1,ss1, 't5_dh',aa2,ss2,
     &              'h5_dh',aa3,ss3, 's5_dh',aa4,ss4
      write(*,2040) '     ',dd1,ss1, '     ',dd2,ss2,
     &              '     ',dd3,ss3, '     ',dd4,ss4

      epolt$ = epolt + dep
      epi$ = 1.0/epolt$
      call gas_delhd(lambda$,nair,
     &               p4$,t4$,h4$,s4$,cp4$,r4$,delh$,epi$,
     &               p5$,t5$,h5$,s5$,cp5$,r5$,
     &               p5_s4$,
     &               p5_p4$,
     &               p5_epi$,
     &               p5_h4$, t5_h4$, h5_h4$, s5_h4$,
     &               p5_dh$, t5_dh$, h5_dh$, s5_dh$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      epolt$ = epolt
      epi$ = 1.0/epolt$
      p5_ep$ = p5_epi$*(-epi$/epolt$)

      dd1 = (p5$-p5)/dep
      aa1 = (p5_ep$+p5_ep)*0.5
      call compare(ss1,aa1,dd1)
      write(*,*)
      write(*,2010) 'p5_ep',aa1,ss1
      write(*,2010) '     ',dd1,ss1


      i = 1
      lambda$(i) = lambda(i) + dal
      call gas_delhd(lambda$,nair,
     &               p4$,t4$,h4$,s4$,cp4$,r4$,delh$,epi$,
     &               p5$,t5$,h5$,s5$,cp5$,r5$,
     &               p5_s4$,
     &               p5_p4$,
     &               p5_epi$,
     &               p5_h4$, t5_h4$, h5_h4$, s5_h4$,
     &               p5_dh$, t5_dh$, h5_dh$, s5_dh$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      lambda$(i) = lambda(i)
      dd1 = (p5$-p5)/dal
      dd2 = (t5$-t5)/dal
      dd3 = (h5$-h5)/dal
      dd4 = (s5$-s5)/dal
      dd5 = (cp5$-cp5)/dal
      dd6 = (r5$-r5)/dal
      aa1 = (p_al$(i)+p_al(i))*0.5
      aa2 = (t_al$(i)+t_al(i))*0.5
      aa3 = (h_al$(i)+h_al(i))*0.5
      aa4 = (s_al$(i)+s_al(i))*0.5
      aa5 = (cp_al$(i)+cp_al(i))*0.5
      aa6 = (r_al$(i)+r_al(i))*0.5
      call compare(ss1,aa1,dd1)
      call compare(ss2,aa2,dd2)
      call compare(ss3,aa3,dd3)
      call compare(ss4,aa4,dd4)
      call compare(ss5,aa5,dd5)
      call compare(ss6,aa6,dd6)
      write(*,*)
      write(*,2060) 'p5_al',aa1,ss1, 'T5_al',aa2,ss2,
     &              'h5_al',aa3,ss3, 's5_al',aa4,ss4,
     &              'c5_al',aa5,ss5, 'r5_al',aa6,ss6
      write(*,2060) '     ',dd1,ss1, '     ',dd2,ss2,
     &              '     ',dd3,ss3, '     ',dd4,ss4,
     &              '     ',dd5,ss5, '     ',dd6,ss6



c===============================================================
c---- specified Mach
      write(*,*) 
      write(*,*) 'gas_machd...'

      m4 = 0.9
      ma = 1.2

      epolt = 0.94
      epi = 1.0/epolt
      call gas_machd(lambda,nair,
     &               p4,t4,h4,s4,cp4,r4,m4,ma,epi,
     &               p5,t5,h5,s5,cp5,r5,
     &               p5_s4,
     &               p5_p4,
     &               p5_epi,
     &               p5_t4, t5_t4, h5_t4, s5_t4,
     &               p5_h4, t5_h4, h5_h4, s5_h4,
     &               p5_ma, t5_ma, h5_ma, s5_ma,
     &               p_al , t_al , h_al , s_al , cp_al, r_al)
      p5_ep = p5_epi*(-epi/epolt)

      p4$ = p4
      t4$ = t4
      h4$ = h4
      s4$ = s4
      cp4$ = cp4
      r4$ = r4
      m4$ = m4
      ma$ = ma
      epi$ = epi
      do i = 1, n
        lambda$(i) = lambda(i)
      enddo

      ds4 = eps*cp4
      dp4 = eps*p4
      dt4 = eps*t4
      dh4 = eps*cp4*t4
      dma = eps
      dal = eps


      s4$ = s4 + ds4
      call gas_machd(lambda$,nair,
     &               p4$,t4$,h4$,s4$,cp4$,r4$,m4$,ma$,epi$,
     &               p5$,t5$,h5$,s5$,cp5$,r5$,
     &               p5_s4$,
     &               p5_p4$,
     &               p5_epi$,
     &               p5_t4$, t5_t4$, h5_t4$, s5_t4$,
     &               p5_h4$, t5_h4$, h5_h4$, s5_h4$,
     &               p5_ma$, t5_ma$, h5_ma$, s5_ma$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      s4$ = s4

      dd1 = (p5$-p5)/ds4
      aa1 = (p5_s4$+p5_s4)*0.5
      call compare(ss1,aa1,dd1)
      write(*,*)
      write(*,2010) 'p5_s4',aa1,ss1
      write(*,2010) '     ',dd1,ss1

      p4$ = p4 + dp4
      call gas_machd(lambda$,nair,
     &               p4$,t4$,h4$,s4$,cp4$,r4$,m4$,ma$,epi$,
     &               p5$,t5$,h5$,s5$,cp5$,r5$,
     &               p5_s4$,
     &               p5_p4$,
     &               p5_epi$,
     &               p5_t4$, t5_t4$, h5_t4$, s5_t4$,
     &               p5_h4$, t5_h4$, h5_h4$, s5_h4$,
     &               p5_ma$, t5_ma$, h5_ma$, s5_ma$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      p4$ = p4

      dd1 = (p5$-p5)/dp4
      aa1 = (p5_p4$+p5_p4)*0.5
      call compare(ss1,aa1,dd1)
      write(*,*)
      write(*,2010) 'p5_p4',aa1,ss1
      write(*,2010) '     ',dd1,ss1

      t4$ = t4 + dt4
      call gas_machd(lambda$,nair,
     &               p4$,t4$,h4$,s4$,cp4$,r4$,m4$,ma$,epi$,
     &               p5$,t5$,h5$,s5$,cp5$,r5$,
     &               p5_s4$,
     &               p5_p4$,
     &               p5_epi$,
     &               p5_t4$, t5_t4$, h5_t4$, s5_t4$,
     &               p5_h4$, t5_h4$, h5_h4$, s5_h4$,
     &               p5_ma$, t5_ma$, h5_ma$, s5_ma$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      t4$ = t4

      dd1 = (p5$-p5)/dt4
      dd2 = (t5$-t5)/dt4
      dd3 = (h5$-h5)/dt4
      dd4 = (s5$-s5)/dt4
      aa1 = (p5_t4$+p5_t4)*0.5
      aa2 = (t5_t4$+t5_t4)*0.5
      aa3 = (h5_t4$+h5_t4)*0.5
      aa4 = (s5_t4$+s5_t4)*0.5
      call compare(ss1,aa1,dd1)
      call compare(ss2,aa2,dd2)
      call compare(ss3,aa3,dd3)
      call compare(ss4,aa4,dd4)
      write(*,*)
      write(*,2040) 'p5_t4',aa1,ss1, 't5_t4',aa2,ss2,
     &              'h5_t4',aa3,ss3, 's5_t4',aa4,ss4
      write(*,2040) '     ',dd1,ss1, '     ',dd2,ss2,
     &              '     ',dd3,ss3, '     ',dd4,ss4


      h4$ = h4 + dh4
      call gas_machd(lambda$,nair,
     &               p4$,t4$,h4$,s4$,cp4$,r4$,m4$,ma$,epi$,
     &               p5$,t5$,h5$,s5$,cp5$,r5$,
     &               p5_s4$,
     &               p5_p4$,
     &               p5_epi$,
     &               p5_t4$, t5_t4$, h5_t4$, s5_t4$,
     &               p5_h4$, t5_h4$, h5_h4$, s5_h4$,
     &               p5_ma$, t5_ma$, h5_ma$, s5_ma$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      h4$ = h4

      dd1 = (p5$-p5)/dh4
      dd2 = (t5$-t5)/dh4
      dd3 = (h5$-h5)/dh4
      dd4 = (s5$-s5)/dh4
      aa1 = (p5_h4$+p5_h4)*0.5
      aa2 = (t5_h4$+t5_h4)*0.5
      aa3 = (h5_h4$+h5_h4)*0.5
      aa4 = (s5_h4$+s5_h4)*0.5
      call compare(ss1,aa1,dd1)
      call compare(ss2,aa2,dd2)
      call compare(ss3,aa3,dd3)
      call compare(ss4,aa4,dd4)
      write(*,*)
      write(*,2040) 'p5_h4',aa1,ss1, 't5_h4',aa2,ss2,
     &              'h5_h4',aa3,ss3, 's5_h4',aa4,ss4
      write(*,2040) '     ',dd1,ss1, '     ',dd2,ss2,
     &              '     ',dd3,ss3, '     ',dd4,ss4

      ma$ = ma + dma
      call gas_machd(lambda$,nair,
     &               p4$,t4$,h4$,s4$,cp4$,r4$,m4$,ma$,epi$,
     &               p5$,t5$,h5$,s5$,cp5$,r5$,
     &               p5_s4$,
     &               p5_p4$,
     &               p5_epi$,
     &               p5_t4$, t5_t4$, h5_t4$, s5_t4$,
     &               p5_h4$, t5_h4$, h5_h4$, s5_h4$,
     &               p5_ma$, t5_ma$, h5_ma$, s5_ma$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      ma$ = ma

      dd1 = (p5$-p5)/dma
      dd2 = (t5$-t5)/dma
      dd3 = (h5$-h5)/dma
      dd4 = (s5$-s5)/dma
      aa1 = (p5_ma$+p5_ma)*0.5
      aa2 = (t5_ma$+t5_ma)*0.5
      aa3 = (h5_ma$+h5_ma)*0.5
      aa4 = (s5_ma$+s5_ma)*0.5
      call compare(ss1,aa1,dd1)
      call compare(ss2,aa2,dd2)
      call compare(ss3,aa3,dd3)
      call compare(ss4,aa4,dd4)
      write(*,*)
      write(*,2040) 'p5_ma',aa1,ss1, 't5_ma',aa2,ss2,
     &              'h5_ma',aa3,ss3, 's5_ma',aa4,ss4
      write(*,2040) '     ',dd1,ss1, '     ',dd2,ss2,
     &              '     ',dd3,ss3, '     ',dd4,ss4


      epolt$ = epolt + dep
      epi$ = 1.0/epolt$
      call gas_machd(lambda$,nair,
     &               p4$,t4$,h4$,s4$,cp4$,r4$,m4$,ma$,epi$,
     &               p5$,t5$,h5$,s5$,cp5$,r5$,
     &               p5_s4$,
     &               p5_p4$,
     &               p5_epi$,
     &               p5_t4$, t5_t4$, h5_t4$, s5_t4$,
     &               p5_h4$, t5_h4$, h5_h4$, s5_h4$,
     &               p5_ma$, t5_ma$, h5_ma$, s5_ma$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      epolt$ = epolt
      epi$ = 1.0/epolt$
      p5_ep$ = p5_epi$*(-epi$/epolt$)

      ma$ = ma
      dd1 = (p5$-p5)/dep
      aa1 = (p5_ep$+p5_ep)*0.5
      call compare(ss1,aa1,dd1)
      write(*,*)
      write(*,2040) 'p5_ep',aa1,ss1
      write(*,2040) '     ',dd1,ss1



      i = 1
      lambda$(i) = lambda(i) + dal
      call gas_machd(lambda$,nair,
     &               p4$,t4$,h4$,s4$,cp4$,r4$,m4$,ma$,epi$,
     &               p5$,t5$,h5$,s5$,cp5$,r5$,
     &               p5_s4$,
     &               p5_p4$,
     &               p5_epi$,
     &               p5_t4$, t5_t4$, h5_t4$, s5_t4$,
     &               p5_h4$, t5_h4$, h5_h4$, s5_h4$,
     &               p5_ma$, t5_ma$, h5_ma$, s5_ma$,
     &               p_al$ , t_al$ , h_al$ , s_al$ , cp_al$, r_al$)
      lambda$(i) = lambda(i)
      dd1 = (p5$-p5)/dal
      dd2 = (t5$-t5)/dal
      dd3 = (h5$-h5)/dal
      dd4 = (s5$-s5)/dal
      dd5 = (cp5$-cp5)/dal
      dd6 = (r5$-r5)/dal
      aa1 = (p_al$(i)+p_al(i))*0.5
      aa2 = (t_al$(i)+t_al(i))*0.5
      aa3 = (h_al$(i)+h_al(i))*0.5
      aa4 = (s_al$(i)+s_al(i))*0.5
      aa5 = (cp_al$(i)+cp_al(i))*0.5
      aa6 = (r_al$(i)+r_al(i))*0.5
      call compare(ss1,aa1,dd1)
      call compare(ss2,aa2,dd2)
      call compare(ss3,aa3,dd3)
      call compare(ss4,aa4,dd4)
      call compare(ss5,aa5,dd5)
      call compare(ss6,aa6,dd6)
      write(*,*)
      write(*,2060) 'p5_al',aa1,ss1, 'T5_al',aa2,ss2,
     &              'h5_al',aa3,ss3, 's5_al',aa4,ss4,
     &              'c5_al',aa5,ss5, 'r5_al',aa6,ss6
      write(*,2060) '     ',dd1,ss1, '     ',dd2,ss2,
     &              '     ',dd3,ss3, '     ',dd4,ss4,
     &              '     ',dd5,ss5, '     ',dd6,ss6



      stop
      end

