
      subroutine gasfun(igas,t, s,s_t, h,h_t, cp,r)
c--------------------------------------------------------------------
c     Computes properties of a thermally-perfect gas
c     with some variable specific heat cp(T).
c
c  Input:
c     igas   index specifying the gas (see if blocks below for list)
c     t      temperature T in Kelvin
c
c  Output:
c     s      entropy-complement function s(T)
c     s_t    ds/dT
c     h      complete enthalpy function h(T)
c     h_t    dh/dT
c     cp     specific heat cp(T)
c     r      ideal-gas constant R
c
c
c     The adiabatic pressure change over a process 1..2 
c     with some polytropic efficiency epol is 
c       p2  =  p1  exp [   epol   (s2-s1)/R ]    compression
c       p2  =  p1  exp [ (1/epol) (s2-s1)/R ]    expansion
c
c--------------------------------------------------------------------
      if    (igas.eq. 1) then
        call     gas_n2(t,s,h,cp,r)
      elseif(igas.eq. 2) then
        call     gas_o2(t,s,h,cp,r)
      elseif(igas.eq. 3) then
        call    gas_co2(t,s,h,cp,r)
      elseif(igas.eq. 4) then
        call    gas_h2o(t,s,h,cp,r)
      elseif(igas.eq. 5) then
        call     gas_ar(t,s,h,cp,r)
      elseif(igas.eq.11) then
        call    gas_ch4(t,s,h,cp,r)
      elseif(igas.eq.12) then
        call   gas_c2h6(t,s,h,cp,r)
      elseif(igas.eq.13) then
        call   gas_c3h8(t,s,h,cp,r)
      elseif(igas.eq.14) then
        call  gas_c4h10(t,s,h,cp,r)
      elseif(igas.eq.18) then
        call  gas_c8h18(t,s,h,cp,r)
      elseif(igas.eq.24) then
        call gas_c14h30(t,s,h,cp,r)
      else
        write(*,*) 'GASFUN: undefined gas index:', igas
        stop
      endif
      s_t = cp/t
      h_t = cp
      return
      end ! gasfun


      subroutine gaschem(igas, nchon)
c--------------------------------------------------------------------
c     Returns number of C,H,O,N atoms in gas molecule,
c     for the gases implemented in subroutine gasfun above.
c
c  Input:
c     igas   index specifying the gas (see if blocks below for list)
c
c  Output:
c     nchon(.)  number of C,H,O,N atoms in gas molecule
c
c--------------------------------------------------------------------
      data kc, kh, ko, kn / 1 , 2 , 3 , 4 /
      integer nchon(4)
      nchon(1) = 0
      nchon(2) = 0
      nchon(3) = 0
      nchon(4) = 0
      if    (igas.eq. 1) then  !   N2
        nchon(kn) = 2
      elseif(igas.eq. 2) then  !   O2
        nchon(ko) = 2
      elseif(igas.eq. 3) then  !   CO2
        nchon(kc) = 1
        nchon(ko) = 2
      elseif(igas.eq. 4) then  !   H2O
        nchon(kh) = 2
        nchon(ko) = 1
      elseif(igas.eq. 5) then  !   Ar

      elseif(igas.eq.11) then  !   CH4
        nchon(kc) = 1
        nchon(kh) = 4
      elseif(igas.eq.12) then  !   C2H6
        nchon(kc) = 2
        nchon(kh) = 6
      elseif(igas.eq.13) then  !   C3H8
        nchon(kc) = 3
        nchon(kh) = 8
      elseif(igas.eq.14) then  !   C4H10
        nchon(kc) = 4
        nchon(kh) = 10
      elseif(igas.eq.18) then  !   C8H18
        nchon(kc) = 8
        nchon(kh) = 18
      elseif(igas.eq.24) then  !   C14H30
        nchon(kc) = 14
        nchon(kh) = 30
      else
        write(*,*) 'GASCHEM: undefined gas index:', igas
        stop
      endif
      return
      end ! gaschem




      subroutine gas_N2(t1, s1,h1,cp1,r1)
      parameter (ndim= 52)
      real t(ndim), tl(ndim), cp(ndim),
     &     cpt(ndim), h(ndim), s(ndim)
      data r, hform /   296.94    ,   0.0000     /
      data t /
     &  175.00e0,  200.00e0,  225.00e0,  250.00e0,  275.00e0,  300.00e0,
     &  325.00e0,  350.00e0,  375.00e0,  400.00e0,  450.00e0,  500.00e0,
     &  550.00e0,  600.00e0,  650.00e0,  700.00e0,  750.00e0,  800.00e0,
     &  850.00e0,  900.00e0,  950.00e0, 1000.00e0, 1050.00e0, 1100.00e0,
     & 1150.00e0, 1200.00e0, 1250.00e0, 1300.00e0, 1350.00e0, 1400.00e0,
     & 1500.00e0, 1600.00e0, 1700.00e0, 1800.00e0, 1900.00e0, 2000.00e0,
     & 2100.00e0, 2200.00e0, 2300.00e0, 2400.00e0, 2500.00e0, 2600.00e0,
     & 2700.00e0, 2800.00e0, 2900.00e0, 3000.00e0, 3500.00e0, 4000.00e0,
     & 4500.00e0, 5000.00e0, 5500.00e0, 6000.00e0/
      data tl /
     & 5.16479e0, 5.29832e0, 5.41610e0, 5.52146e0, 5.61677e0, 5.70378e0,
     & 5.78383e0, 5.85793e0, 5.92693e0, 5.99146e0, 6.10925e0, 6.21461e0,
     & 6.30992e0, 6.39693e0, 6.47697e0, 6.55108e0, 6.62007e0, 6.68461e0,
     & 6.74524e0, 6.80239e0, 6.85646e0, 6.90776e0, 6.95655e0, 7.00307e0,
     & 7.04752e0, 7.09008e0, 7.13090e0, 7.17012e0, 7.20786e0, 7.24423e0,
     & 7.31322e0, 7.37776e0, 7.43838e0, 7.49554e0, 7.54961e0, 7.60090e0,
     & 7.64969e0, 7.69621e0, 7.74066e0, 7.78322e0, 7.82405e0, 7.86327e0,
     & 7.90101e0, 7.93737e0, 7.97247e0, 8.00637e0, 8.16052e0, 8.29405e0,
     & 8.41183e0, 8.51719e0, 8.61250e0, 8.69951e0/
      data cp /
     & 1039.00e0, 1039.00e0, 1039.00e0, 1039.10e0, 1039.30e0, 1039.60e0,
     & 1040.20e0, 1041.00e0, 1042.20e0, 1043.80e0, 1049.00e0, 1056.00e0,
     & 1065.00e0, 1075.00e0, 1086.00e0, 1098.00e0, 1110.00e0, 1122.00e0,
     & 1134.00e0, 1146.00e0, 1157.00e0, 1167.00e0, 1177.00e0, 1187.00e0,
     & 1196.00e0, 1204.00e0, 1212.00e0, 1219.00e0, 1226.00e0, 1232.00e0,
     & 1244.00e0, 1254.00e0, 1263.00e0, 1271.00e0, 1278.00e0, 1284.00e0,
     & 1290.00e0, 1295.00e0, 1300.00e0, 1304.00e0, 1307.00e0, 1311.00e0,
     & 1314.00e0, 1317.00e0, 1320.00e0, 1323.00e0, 1333.00e0, 1342.00e0,
     & 1349.00e0, 1355.00e0, 1362.00e0, 1369.00e0/
      data cpt /
     & 0.00051e0,-0.00051e0, 0.00154e0, 0.00638e0, 0.00894e0, 0.01784e0,
     & 0.02770e0, 0.03935e0, 0.05488e0, 0.07713e0, 0.12347e0, 0.16099e0,
     & 0.19255e0, 0.20879e0, 0.23230e0, 0.24203e0, 0.23959e0, 0.23962e0,
     & 0.24193e0, 0.23265e0, 0.20745e0, 0.19753e0, 0.20243e0, 0.19275e0,
     & 0.16659e0, 0.16090e0, 0.14980e0, 0.13990e0, 0.13058e0, 0.11776e0,
     & 0.11229e0, 0.09307e0, 0.08542e0, 0.07524e0, 0.06361e0, 0.06033e0,
     & 0.05506e0, 0.04944e0, 0.04717e0, 0.03186e0, 0.03539e0, 0.03659e0,
     & 0.02826e0, 0.03036e0, 0.03030e0, 0.02844e0, 0.01722e0, 0.01667e0,
     & 0.01211e0, 0.01288e0, 0.01437e0, 0.01363e0/
      data h /
     &-0.12781e6,-0.10184e6,-0.07586e6,-0.04989e6,-0.02391e6, 0.00208e6,
     & 0.02808e6, 0.05409e6, 0.08013e6, 0.10620e6, 0.15851e6, 0.21113e6,
     & 0.26415e6, 0.31765e6, 0.37167e6, 0.42626e6, 0.48146e6, 0.53726e6,
     & 0.59366e6, 0.65067e6, 0.70825e6, 0.76635e6, 0.82495e6, 0.88405e6,
     & 0.94363e6, 1.00363e6, 1.06403e6, 1.12481e6, 1.18594e6, 1.24739e6,
     & 1.37119e6, 1.49611e6, 1.62197e6, 1.74868e6, 1.87613e6, 2.00424e6,
     & 2.13294e6, 2.26220e6, 2.39195e6, 2.52216e6, 2.65271e6, 2.78361e6,
     & 2.91486e6, 3.04641e6, 3.17826e6, 3.31041e6, 3.97465e6, 4.64341e6,
     & 5.31625e6, 5.99224e6, 6.67146e6, 7.35422e6/
      data s /
     &-0.55312e3,-0.41438e3,-0.29201e3,-0.18253e3,-0.08349e3, 0.00695e3,
     & 0.09019e3, 0.16730e3, 0.23916e3, 0.30647e3, 0.42969e3, 0.54056e3,
     & 0.64162e3, 0.73471e3, 0.82118e3, 0.90210e3, 0.97826e3, 1.05028e3,
     & 1.11866e3, 1.18382e3, 1.24608e3, 1.30569e3, 1.36287e3, 1.41785e3,
     & 1.47082e3, 1.52189e3, 1.57121e3, 1.61888e3, 1.66502e3, 1.70971e3,
     & 1.79513e3, 1.87574e3, 1.95204e3, 2.02446e3, 2.09337e3, 2.15908e3,
     & 2.22187e3, 2.28200e3, 2.33968e3, 2.39510e3, 2.44839e3, 2.49973e3,
     & 2.54926e3, 2.59710e3, 2.64337e3, 2.68817e3, 2.89293e3, 3.07152e3,
     & 3.23001e3, 3.37245e3, 3.50192e3, 3.62073e3/
c- - - - - - - - - - -
      ilow = 1
      i = ndim
c- - - - - - - - - - -
 10   continue
      if(i .le. ilow+1) go to 11
 
      imid = (i+ilow)/2
      if(t1 .lt. t(imid)) then
       i = imid
      else
       ilow = imid
      endif
      go to 10
c- - - - - - - - - - -
 11   continue
 
      dt = t(i) - t(i-1)
      f = (t1 - t(i-1)) / dt
 
      dtl = tl(i) - tl(i-1)
      fl = (log(t1) - tl(i-1)) / dtl
c- - - - - - - - - - -
      s1    =          (1.0-fl)* s(i-1) 
     &               +      fl * s(i) 
     & + fl*(1.0-fl)*( (1.0-fl)*(dtl*cp(i-1) - s(i) + s(i-1))
     &                    - fl *(dtl*cp(i)   - s(i) + s(i-1)) )
 
      h1  = hform  + (1.0-f)* h(i-1) 
     &             +      f * h(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cp(i-1) - h(i) + h(i-1))
     &                  - f *(dt*cp(i)   - h(i) + h(i-1)) )
 
      cp1   =        (1.0-f)* cp(i-1) 
     &             +      f * cp(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cpt(i-1) - cp(i) + cp(i-1))
     &                  - f *(dt*cpt(i)   - cp(i) + cp(i-1)) )
 
      r1 = r
c- - - - - - - - - - -
      return
      end ! gas_N2


      subroutine gas_O2(t1, s1,h1,cp1,r1)
      parameter (ndim= 52)
      real t(ndim), tl(ndim), cp(ndim),
     &     cpt(ndim), h(ndim), s(ndim)
      data r, hform /   259.82    ,   0.0000     /
      data t /
     &  175.00e0,  200.00e0,  225.00e0,  250.00e0,  275.00e0,  300.00e0,
     &  325.00e0,  350.00e0,  375.00e0,  400.00e0,  450.00e0,  500.00e0,
     &  550.00e0,  600.00e0,  650.00e0,  700.00e0,  750.00e0,  800.00e0,
     &  850.00e0,  900.00e0,  950.00e0, 1000.00e0, 1050.00e0, 1100.00e0,
     & 1150.00e0, 1200.00e0, 1250.00e0, 1300.00e0, 1350.00e0, 1400.00e0,
     & 1500.00e0, 1600.00e0, 1700.00e0, 1800.00e0, 1900.00e0, 2000.00e0,
     & 2100.00e0, 2200.00e0, 2300.00e0, 2400.00e0, 2500.00e0, 2600.00e0,
     & 2700.00e0, 2800.00e0, 2900.00e0, 3000.00e0, 3500.00e0, 4000.00e0,
     & 4500.00e0, 5000.00e0, 5500.00e0, 6000.00e0/
      data tl /
     & 5.16479e0, 5.29832e0, 5.41610e0, 5.52146e0, 5.61677e0, 5.70378e0,
     & 5.78383e0, 5.85793e0, 5.92693e0, 5.99146e0, 6.10925e0, 6.21461e0,
     & 6.30992e0, 6.39693e0, 6.47697e0, 6.55108e0, 6.62007e0, 6.68461e0,
     & 6.74524e0, 6.80239e0, 6.85646e0, 6.90776e0, 6.95655e0, 7.00307e0,
     & 7.04752e0, 7.09008e0, 7.13090e0, 7.17012e0, 7.20786e0, 7.24423e0,
     & 7.31322e0, 7.37776e0, 7.43838e0, 7.49554e0, 7.54961e0, 7.60090e0,
     & 7.64969e0, 7.69621e0, 7.74066e0, 7.78322e0, 7.82405e0, 7.86327e0,
     & 7.90101e0, 7.93737e0, 7.97247e0, 8.00637e0, 8.16052e0, 8.29405e0,
     & 8.41183e0, 8.51719e0, 8.61250e0, 8.69951e0/
      data cp /
     &  910.00e0,  910.30e0,  911.30e0,  913.00e0,  915.60e0,  918.90e0,
     &  923.00e0,  928.00e0,  933.80e0,  940.50e0,  956.50e0,  972.00e0,
     &  988.00e0, 1003.00e0, 1017.00e0, 1031.00e0, 1043.00e0, 1054.00e0,
     & 1065.00e0, 1074.00e0, 1082.00e0, 1090.00e0, 1097.00e0, 1103.00e0,
     & 1109.00e0, 1115.00e0, 1120.00e0, 1125.00e0, 1130.00e0, 1134.00e0,
     & 1143.00e0, 1151.00e0, 1158.00e0, 1166.00e0, 1173.00e0, 1181.00e0,
     & 1188.00e0, 1195.00e0, 1202.00e0, 1209.00e0, 1216.00e0, 1223.00e0,
     & 1230.00e0, 1236.00e0, 1243.00e0, 1249.00e0, 1276.00e0, 1299.00e0,
     & 1316.00e0, 1328.00e0, 1337.00e0, 1344.00e0/
      data cpt /
     &-0.00237e0, 0.02637e0, 0.05289e0, 0.08608e0, 0.11879e0, 0.14675e0,
     & 0.18219e0, 0.21647e0, 0.24793e0, 0.29183e0, 0.32120e0, 0.31338e0,
     & 0.31528e0, 0.28549e0, 0.28275e0, 0.26349e0, 0.22328e0, 0.22339e0,
     & 0.20315e0, 0.16402e0, 0.16075e0, 0.15296e0, 0.12742e0, 0.11736e0,
     & 0.12314e0, 0.11007e0, 0.09658e0, 0.10360e0, 0.08900e0, 0.08040e0,
     & 0.08963e0, 0.07109e0, 0.07602e0, 0.07485e0, 0.07458e0, 0.07681e0,
     & 0.06817e0, 0.07050e0, 0.06983e0, 0.07016e0, 0.06951e0, 0.07179e0,
     & 0.06334e0, 0.06486e0, 0.06722e0, 0.05625e0, 0.05089e0, 0.04019e0,
     & 0.02834e0, 0.02045e0, 0.01585e0, 0.01215e0/
      data h /
     &-0.11227e6,-0.08952e6,-0.06675e6,-0.04395e6,-0.02109e6, 0.00184e6,
     & 0.02486e6, 0.04800e6, 0.07127e6, 0.09469e6, 0.14211e6, 0.19033e6,
     & 0.23933e6, 0.28911e6, 0.33961e6, 0.39081e6, 0.44267e6, 0.49509e6,
     & 0.54807e6, 0.60156e6, 0.65546e6, 0.70976e6, 0.76444e6, 0.81944e6,
     & 0.87474e6, 0.93034e6, 0.98622e6, 1.04234e6, 1.09872e6, 1.15532e6,
     & 1.26917e6, 1.38388e6, 1.49933e6, 1.61553e6, 1.73248e6, 1.85018e6,
     & 1.96863e6, 2.08778e6, 2.20763e6, 2.32818e6, 2.44943e6, 2.57138e6,
     & 2.69404e6, 2.81734e6, 2.94128e6, 3.06589e6, 3.69726e6, 4.34123e6,
     & 4.99523e6, 5.65639e6, 6.32274e6, 6.99306e6/
      data s /
     &-0.48568e3,-0.36416e3,-0.25689e3,-0.16079e3,-0.07366e3, 0.00615e3,
     & 0.07985e3, 0.14843e3, 0.21265e3, 0.27312e3, 0.38481e3, 0.48639e3,
     & 0.57978e3, 0.66641e3, 0.74724e3, 0.82313e3, 0.89468e3, 0.96235e3,
     & 1.02658e3, 1.08772e3, 1.14600e3, 1.20170e3, 1.25506e3, 1.30623e3,
     & 1.35539e3, 1.40272e3, 1.44834e3, 1.49237e3, 1.53492e3, 1.57609e3,
     & 1.65463e3, 1.72866e3, 1.79865e3, 1.86507e3, 1.92829e3, 1.98866e3,
     & 2.04646e3, 2.10189e3, 2.15516e3, 2.20646e3, 2.25596e3, 2.30379e3,
     & 2.35008e3, 2.39492e3, 2.43841e3, 2.48066e3, 2.67526e3, 2.84720e3,
     & 3.00124e3, 3.14055e3, 3.26756e3, 3.38421e3/
c- - - - - - - - - - -
      ilow = 1
      i = ndim
c- - - - - - - - - - -
 10   continue
      if(i .le. ilow+1) go to 11
 
      imid = (i+ilow)/2
      if(t1 .lt. t(imid)) then
       i = imid
      else
       ilow = imid
      endif
      go to 10
c- - - - - - - - - - -
 11   continue
 
      dt = t(i) - t(i-1)
      f = (t1 - t(i-1)) / dt
 
      dtl = tl(i) - tl(i-1)
      fl = (log(t1) - tl(i-1)) / dtl
c- - - - - - - - - - -
      s1    =          (1.0-fl)* s(i-1) 
     &               +      fl * s(i) 
     & + fl*(1.0-fl)*( (1.0-fl)*(dtl*cp(i-1) - s(i) + s(i-1))
     &                    - fl *(dtl*cp(i)   - s(i) + s(i-1)) )
 
      h1  = hform  + (1.0-f)* h(i-1) 
     &             +      f * h(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cp(i-1) - h(i) + h(i-1))
     &                  - f *(dt*cp(i)   - h(i) + h(i-1)) )
 
      cp1   =        (1.0-f)* cp(i-1) 
     &             +      f * cp(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cpt(i-1) - cp(i) + cp(i-1))
     &                  - f *(dt*cpt(i)   - cp(i) + cp(i-1)) )
 
      r1 = r
c- - - - - - - - - - -
      return
      end ! gas_O2


      subroutine gas_Ar(t1, s1,h1,cp1,r1)
      parameter (ndim= 24)
      real t(ndim), tl(ndim), cp(ndim),
     &     cpt(ndim), h(ndim), s(ndim)
      data r, hform /   208.00    ,   0.0000     /
      data t /
     &  175.00e0,  200.00e0,  250.00e0,  300.00e0,  400.00e0,  500.00e0,
     &  600.00e0,  700.00e0,  800.00e0,  900.00e0, 1000.00e0, 1200.00e0,
     & 1400.00e0, 1600.00e0, 1800.00e0, 2000.00e0, 2200.00e0, 2400.00e0,
     & 2600.00e0, 2800.00e0, 3000.00e0, 4000.00e0, 5000.00e0, 6000.00e0/
      data tl /
     & 5.16479e0, 5.29832e0, 5.52146e0, 5.70378e0, 5.99146e0, 6.21461e0,
     & 6.39693e0, 6.55108e0, 6.68461e0, 6.80239e0, 6.90776e0, 7.09008e0,
     & 7.24423e0, 7.37776e0, 7.49554e0, 7.60090e0, 7.69621e0, 7.78322e0,
     & 7.86327e0, 7.93737e0, 8.00637e0, 8.29405e0, 8.51719e0, 8.69951e0/
      data cp /
     &  520.00e0,  520.00e0,  520.00e0,  520.00e0,  520.00e0,  520.00e0,
     &  520.00e0,  520.00e0,  520.00e0,  520.00e0,  520.00e0,  520.00e0,
     &  520.00e0,  520.00e0,  520.00e0,  520.00e0,  520.00e0,  520.00e0,
     &  520.00e0,  520.00e0,  520.00e0,  520.00e0,  520.00e0,  520.00e0/
      data cpt /
     & 0.00000e0, 0.00000e0, 0.00000e0, 0.00000e0, 0.00000e0, 0.00000e0,
     & 0.00000e0, 0.00000e0, 0.00000e0, 0.00000e0, 0.00000e0, 0.00000e0,
     & 0.00000e0, 0.00000e0, 0.00000e0, 0.00000e0, 0.00000e0, 0.00000e0,
     & 0.00000e0, 0.00000e0, 0.00000e0, 0.00000e0, 0.00000e0, 0.00000e0/
      data h /
     &-0.06396e6,-0.05096e6,-0.02496e6, 0.00104e6, 0.05304e6, 0.10504e6,
     & 0.15704e6, 0.20904e6, 0.26104e6, 0.31304e6, 0.36504e6, 0.46904e6,
     & 0.57304e6, 0.67704e6, 0.78104e6, 0.88504e6, 0.98904e6, 1.09304e6,
     & 1.19704e6, 1.30104e6, 1.40504e6, 1.92504e6, 2.44504e6, 2.96504e6/
      data s /
     &-0.27680e3,-0.20736e3,-0.09133e3, 0.00348e3, 0.15307e3, 0.26911e3,
     & 0.36391e3, 0.44407e3, 0.51351e3, 0.57476e3, 0.62954e3, 0.72435e3,
     & 0.80451e3, 0.87395e3, 0.93519e3, 0.98998e3, 1.03954e3, 1.08479e3,
     & 1.12641e3, 1.16495e3, 1.20082e3, 1.35042e3, 1.46645e3, 1.56126e3/
c- - - - - - - - - - -
      ilow = 1
      i = ndim
c- - - - - - - - - - -
 10   continue
      if(i .le. ilow+1) go to 11
 
      imid = (i+ilow)/2
      if(t1 .lt. t(imid)) then
       i = imid
      else
       ilow = imid
      endif
      go to 10
c- - - - - - - - - - -
 11   continue
 
      dt = t(i) - t(i-1)
      f = (t1 - t(i-1)) / dt
 
      dtl = tl(i) - tl(i-1)
      fl = (log(t1) - tl(i-1)) / dtl
c- - - - - - - - - - -
      s1    =          (1.0-fl)* s(i-1) 
     &               +      fl * s(i) 
     & + fl*(1.0-fl)*( (1.0-fl)*(dtl*cp(i-1) - s(i) + s(i-1))
     &                    - fl *(dtl*cp(i)   - s(i) + s(i-1)) )
 
      h1  = hform  + (1.0-f)* h(i-1) 
     &             +      f * h(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cp(i-1) - h(i) + h(i-1))
     &                  - f *(dt*cp(i)   - h(i) + h(i-1)) )
 
      cp1   =        (1.0-f)* cp(i-1) 
     &             +      f * cp(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cpt(i-1) - cp(i) + cp(i-1))
     &                  - f *(dt*cpt(i)   - cp(i) + cp(i-1)) )
 
      r1 = r
c- - - - - - - - - - -
      return
      end ! gas_Ar


      subroutine gas_CO2(t1, s1,h1,cp1,r1)
      parameter (ndim= 52)
      real t(ndim), tl(ndim), cp(ndim),
     &     cpt(ndim), h(ndim), s(ndim)
      data r, hform /   188.96    , -0.89430E+07 /
      data t /
     &  175.00e0,  200.00e0,  225.00e0,  250.00e0,  275.00e0,  300.00e0,
     &  325.00e0,  350.00e0,  375.00e0,  400.00e0,  450.00e0,  500.00e0,
     &  550.00e0,  600.00e0,  650.00e0,  700.00e0,  750.00e0,  800.00e0,
     &  850.00e0,  900.00e0,  950.00e0, 1000.00e0, 1050.00e0, 1100.00e0,
     & 1150.00e0, 1200.00e0, 1250.00e0, 1300.00e0, 1350.00e0, 1400.00e0,
     & 1500.00e0, 1600.00e0, 1700.00e0, 1800.00e0, 1900.00e0, 2000.00e0,
     & 2100.00e0, 2200.00e0, 2300.00e0, 2400.00e0, 2500.00e0, 2600.00e0,
     & 2700.00e0, 2800.00e0, 2900.00e0, 3000.00e0, 3500.00e0, 4000.00e0,
     & 4500.00e0, 5000.00e0, 5500.00e0, 6000.00e0/
      data tl /
     & 5.16479e0, 5.29832e0, 5.41610e0, 5.52146e0, 5.61677e0, 5.70378e0,
     & 5.78383e0, 5.85793e0, 5.92693e0, 5.99146e0, 6.10925e0, 6.21461e0,
     & 6.30992e0, 6.39693e0, 6.47697e0, 6.55108e0, 6.62007e0, 6.68461e0,
     & 6.74524e0, 6.80239e0, 6.85646e0, 6.90776e0, 6.95655e0, 7.00307e0,
     & 7.04752e0, 7.09008e0, 7.13090e0, 7.17012e0, 7.20786e0, 7.24423e0,
     & 7.31322e0, 7.37776e0, 7.43838e0, 7.49554e0, 7.54961e0, 7.60090e0,
     & 7.64969e0, 7.69621e0, 7.74066e0, 7.78322e0, 7.82405e0, 7.86327e0,
     & 7.90101e0, 7.93737e0, 7.97247e0, 8.00637e0, 8.16052e0, 8.29405e0,
     & 8.41183e0, 8.51719e0, 8.61250e0, 8.69951e0/
      data cp /
     &  709.00e0,  735.00e0,  763.00e0,  791.00e0,  819.00e0,  846.00e0,
     &  871.00e0,  895.00e0,  918.00e0,  939.00e0,  978.00e0, 1014.00e0,
     & 1046.00e0, 1075.00e0, 1102.00e0, 1126.00e0, 1148.00e0, 1168.00e0,
     & 1187.00e0, 1204.00e0, 1220.00e0, 1234.00e0, 1247.00e0, 1259.00e0,
     & 1270.00e0, 1280.00e0, 1290.00e0, 1298.00e0, 1306.00e0, 1313.00e0,
     & 1326.00e0, 1338.00e0, 1348.00e0, 1356.00e0, 1364.00e0, 1371.00e0,
     & 1377.00e0, 1383.00e0, 1388.00e0, 1393.00e0, 1397.00e0, 1401.00e0,
     & 1404.00e0, 1408.00e0, 1411.00e0, 1414.00e0, 1427.00e0, 1437.00e0,
     & 1446.00e0, 1455.00e0, 1465.00e0, 1476.00e0/
      data cpt /
     & 0.98903e0, 1.09097e0, 1.12708e0, 1.12072e0, 1.11005e0, 1.03908e0,
     & 0.97362e0, 0.94644e0, 0.88061e0, 0.81113e0, 0.75200e0, 0.68089e0,
     & 0.60444e0, 0.56135e0, 0.51016e0, 0.45799e0, 0.41786e0, 0.39056e0,
     & 0.35990e0, 0.32984e0, 0.30074e0, 0.26719e0, 0.25051e0, 0.23076e0,
     & 0.20646e0, 0.20340e0, 0.17993e0, 0.15689e0, 0.15250e0, 0.13311e0,
     & 0.12633e0, 0.11157e0, 0.08738e0, 0.07891e0, 0.07699e0, 0.06313e0,
     & 0.06050e0, 0.05486e0, 0.05006e0, 0.04490e0, 0.04035e0, 0.03369e0,
     & 0.03490e0, 0.03672e0, 0.02822e0, 0.03038e0, 0.02228e0, 0.01851e0,
     & 0.01767e0, 0.01882e0, 0.02106e0, 0.02294e0/
      data h /
     &-0.09544e6,-0.07740e6,-0.05868e6,-0.03925e6,-0.01913e6, 0.00169e6,
     & 0.02316e6, 0.04523e6, 0.06790e6, 0.09111e6, 0.13905e6, 0.18887e6,
     & 0.24038e6, 0.29342e6, 0.34785e6, 0.40356e6, 0.46042e6, 0.51833e6,
     & 0.57721e6, 0.63699e6, 0.69760e6, 0.75895e6, 0.82098e6, 0.88364e6,
     & 0.94687e6, 1.01062e6, 1.07487e6, 1.13958e6, 1.20468e6, 1.27016e6,
     & 1.40211e6, 1.53532e6, 1.66964e6, 1.80485e6, 1.94085e6, 2.07761e6,
     & 2.21502e6, 2.35302e6, 2.49157e6, 2.63063e6, 2.77013e6, 2.91004e6,
     & 3.05029e6, 3.19089e6, 3.33184e6, 3.47309e6, 4.18351e6, 4.89959e6,
     & 5.62036e6, 6.34558e6, 7.07554e6, 7.81075e6/
      data s /
     &-0.40984e3,-0.31351e3,-0.22533e3,-0.14349e3,-0.06678e3, 0.00565e3,
     & 0.07437e3, 0.13980e3, 0.20234e3, 0.26226e3, 0.37514e3, 0.48008e3,
     & 0.57826e3, 0.67053e3, 0.75766e3, 0.84022e3, 0.91867e3, 0.99340e3,
     & 1.06479e3, 1.13313e3, 1.19866e3, 1.26160e3, 1.32212e3, 1.38042e3,
     & 1.43663e3, 1.49089e3, 1.54335e3, 1.59410e3, 1.64324e3, 1.69087e3,
     & 1.78190e3, 1.86787e3, 1.94930e3, 2.02658e3, 2.10011e3, 2.17026e3,
     & 2.23729e3, 2.30149e3, 2.36308e3, 2.42226e3, 2.47921e3, 2.53408e3,
     & 2.58701e3, 2.63814e3, 2.68761e3, 2.73549e3, 2.95449e3, 3.14571e3,
     & 3.31549e3, 3.46830e3, 3.60744e3, 3.73537e3/
c- - - - - - - - - - -
      ilow = 1
      i = ndim
c- - - - - - - - - - -
 10   continue
      if(i .le. ilow+1) go to 11
 
      imid = (i+ilow)/2
      if(t1 .lt. t(imid)) then
       i = imid
      else
       ilow = imid
      endif
      go to 10
c- - - - - - - - - - -
 11   continue
 
      dt = t(i) - t(i-1)
      f = (t1 - t(i-1)) / dt
 
      dtl = tl(i) - tl(i-1)
      fl = (log(t1) - tl(i-1)) / dtl
c- - - - - - - - - - -
      s1    =          (1.0-fl)* s(i-1) 
     &               +      fl * s(i) 
     & + fl*(1.0-fl)*( (1.0-fl)*(dtl*cp(i-1) - s(i) + s(i-1))
     &                    - fl *(dtl*cp(i)   - s(i) + s(i-1)) )
 
      h1  = hform  + (1.0-f)* h(i-1) 
     &             +      f * h(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cp(i-1) - h(i) + h(i-1))
     &                  - f *(dt*cp(i)   - h(i) + h(i-1)) )
 
      cp1   =        (1.0-f)* cp(i-1) 
     &             +      f * cp(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cpt(i-1) - cp(i) + cp(i-1))
     &                  - f *(dt*cpt(i)   - cp(i) + cp(i-1)) )
 
      r1 = r
c- - - - - - - - - - -
      return
      end ! gas_CO2


      subroutine gas_H2O(t1, s1,h1,cp1,r1)
      parameter (ndim= 52)
      real t(ndim), tl(ndim), cp(ndim),
     &     cpt(ndim), h(ndim), s(ndim)
      data r, hform /   461.91    , -0.13433E+08 /
      data t /
     &  175.00e0,  200.00e0,  225.00e0,  250.00e0,  275.00e0,  300.00e0,
     &  325.00e0,  350.00e0,  375.00e0,  400.00e0,  450.00e0,  500.00e0,
     &  550.00e0,  600.00e0,  650.00e0,  700.00e0,  750.00e0,  800.00e0,
     &  850.00e0,  900.00e0,  950.00e0, 1000.00e0, 1050.00e0, 1100.00e0,
     & 1150.00e0, 1200.00e0, 1250.00e0, 1300.00e0, 1350.00e0, 1400.00e0,
     & 1500.00e0, 1600.00e0, 1700.00e0, 1800.00e0, 1900.00e0, 2000.00e0,
     & 2100.00e0, 2200.00e0, 2300.00e0, 2400.00e0, 2500.00e0, 2600.00e0,
     & 2700.00e0, 2800.00e0, 2900.00e0, 3000.00e0, 3500.00e0, 4000.00e0,
     & 4500.00e0, 5000.00e0, 5500.00e0, 6000.00e0/
      data tl /
     & 5.16479e0, 5.29832e0, 5.41610e0, 5.52146e0, 5.61677e0, 5.70378e0,
     & 5.78383e0, 5.85793e0, 5.92693e0, 5.99146e0, 6.10925e0, 6.21461e0,
     & 6.30992e0, 6.39693e0, 6.47697e0, 6.55108e0, 6.62007e0, 6.68461e0,
     & 6.74524e0, 6.80239e0, 6.85646e0, 6.90776e0, 6.95655e0, 7.00307e0,
     & 7.04752e0, 7.09008e0, 7.13090e0, 7.17012e0, 7.20786e0, 7.24423e0,
     & 7.31322e0, 7.37776e0, 7.43838e0, 7.49554e0, 7.54961e0, 7.60090e0,
     & 7.64969e0, 7.69621e0, 7.74066e0, 7.78322e0, 7.82405e0, 7.86327e0,
     & 7.90101e0, 7.93737e0, 7.97247e0, 8.00637e0, 8.16052e0, 8.29405e0,
     & 8.41183e0, 8.51719e0, 8.61250e0, 8.69951e0/
      data cp /
     & 1850.00e0, 1850.40e0, 1851.80e0, 1854.50e0, 1858.50e0, 1863.80e0,
     & 1870.70e0, 1879.00e0, 1888.90e0, 1900.40e0, 1926.00e0, 1954.00e0,
     & 1984.00e0, 2015.00e0, 2047.00e0, 2080.00e0, 2113.00e0, 2147.00e0,
     & 2182.00e0, 2217.00e0, 2252.00e0, 2288.00e0, 2323.00e0, 2358.00e0,
     & 2392.00e0, 2425.00e0, 2458.00e0, 2490.00e0, 2521.00e0, 2552.00e0,
     & 2609.00e0, 2662.00e0, 2711.00e0, 2756.00e0, 2798.00e0, 2836.00e0,
     & 2872.00e0, 2904.00e0, 2934.00e0, 2962.00e0, 2987.00e0, 3011.00e0,
     & 3033.00e0, 3053.00e0, 3072.00e0, 3090.00e0, 3163.00e0, 3217.00e0,
     & 3258.00e0, 3292.00e0, 3322.00e0, 3350.00e0/
      data cpt /
     &-0.00242e0, 0.03443e0, 0.08071e0, 0.13473e0, 0.18437e0, 0.24378e0,
     & 0.30451e0, 0.36219e0, 0.43075e0, 0.48280e0, 0.53766e0, 0.58254e0,
     & 0.61218e0, 0.62876e0, 0.65278e0, 0.66010e0, 0.66683e0, 0.69258e0,
     & 0.70284e0, 0.69605e0, 0.71296e0, 0.71213e0, 0.69854e0, 0.69372e0,
     & 0.66657e0, 0.65999e0, 0.65345e0, 0.62620e0, 0.62173e0, 0.60686e0,
     & 0.54538e0, 0.51162e0, 0.46813e0, 0.43584e0, 0.39849e0, 0.37018e0,
     & 0.34078e0, 0.30669e0, 0.29246e0, 0.26348e0, 0.24364e0, 0.23196e0,
     & 0.20851e0, 0.19399e0, 0.18553e0, 0.17388e0, 0.12372e0, 0.09322e0,
     & 0.07338e0, 0.06326e0, 0.05758e0, 0.05442e0/
      data h /
     &-0.22806e6,-0.18181e6,-0.13553e6,-0.08921e6,-0.04280e6, 0.00373e6,
     & 0.05041e6, 0.09727e6, 0.14437e6, 0.19173e6, 0.28738e6, 0.38437e6,
     & 0.48282e6, 0.58279e6, 0.68433e6, 0.78751e6, 0.89233e6, 0.99882e6,
     & 1.10705e6, 1.21702e6, 1.32874e6, 1.44224e6, 1.55752e6, 1.67455e6,
     & 1.79330e6, 1.91373e6, 2.03581e6, 2.15951e6, 2.28479e6, 2.41162e6,
     & 2.66972e6, 2.93330e6, 3.20198e6, 3.47536e6, 3.75309e6, 4.03481e6,
     & 4.32024e6, 4.60907e6, 4.90098e6, 5.19580e6, 5.49327e6, 5.79318e6,
     & 6.09540e6, 6.39971e6, 6.70597e6, 7.01408e6, 8.57837e6,10.17401e6,
     &11.79317e6,13.43088e6,15.08450e6,16.75256e6/
      data s /
     &-0.98669e3,-0.73964e3,-0.52163e3,-0.32639e3,-0.14946e3, 0.01247e3,
     & 0.16191e3, 0.30084e3, 0.43081e3, 0.55307e3, 0.77836e3, 0.98271e3,
     & 1.17034e3, 1.34430e3, 1.50684e3, 1.65974e3, 1.80437e3, 1.94182e3,
     & 2.07303e3, 2.19874e3, 2.31954e3, 2.43597e3, 2.54845e3, 2.65733e3,
     & 2.76290e3, 2.86540e3, 2.96506e3, 3.06210e3, 3.15665e3, 3.24890e3,
     & 3.42694e3, 3.59704e3, 3.75991e3, 3.91616e3, 4.06631e3, 4.21080e3,
     & 4.35006e3, 4.48441e3, 4.61417e3, 4.73964e3, 4.86107e3, 4.97869e3,
     & 5.09275e3, 5.20342e3, 5.31088e3, 5.41534e3, 5.89746e3, 6.32352e3,
     & 6.70489e3, 7.04996e3, 7.36515e3, 7.65541e3/
c- - - - - - - - - - -
      ilow = 1
      i = ndim
c- - - - - - - - - - -
 10   continue
      if(i .le. ilow+1) go to 11
 
      imid = (i+ilow)/2
      if(t1 .lt. t(imid)) then
       i = imid
      else
       ilow = imid
      endif
      go to 10
c- - - - - - - - - - -
 11   continue
 
      dt = t(i) - t(i-1)
      f = (t1 - t(i-1)) / dt
 
      dtl = tl(i) - tl(i-1)
      fl = (log(t1) - tl(i-1)) / dtl
c- - - - - - - - - - -
      s1    =          (1.0-fl)* s(i-1) 
     &               +      fl * s(i) 
     & + fl*(1.0-fl)*( (1.0-fl)*(dtl*cp(i-1) - s(i) + s(i-1))
     &                    - fl *(dtl*cp(i)   - s(i) + s(i-1)) )
 
      h1  = hform  + (1.0-f)* h(i-1) 
     &             +      f * h(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cp(i-1) - h(i) + h(i-1))
     &                  - f *(dt*cp(i)   - h(i) + h(i-1)) )
 
      cp1   =        (1.0-f)* cp(i-1) 
     &             +      f * cp(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cpt(i-1) - cp(i) + cp(i-1))
     &                  - f *(dt*cpt(i)   - cp(i) + cp(i-1)) )
 
      r1 = r
c- - - - - - - - - - -
      return
      end ! gas_H2O


      subroutine gas_CH4(t1, s1,h1,cp1,r1)
      parameter (ndim= 23)
      real t(ndim), tl(ndim), cp(ndim),
     &     cpt(ndim), h(ndim), s(ndim)
      data r, hform /   519.65    , -0.46750E+07 /
      data t /
     &  200.00e0,  225.00e0,  250.00e0,  275.00e0,  300.00e0,  325.00e0,
     &  350.00e0,  375.00e0,  400.00e0,  450.00e0,  500.00e0,  550.00e0,
     &  600.00e0,  650.00e0,  700.00e0,  750.00e0,  800.00e0,  850.00e0,
     &  900.00e0,  950.00e0, 1000.00e0, 1050.00e0, 1100.00e0/
      data tl /
     & 5.29832e0, 5.41610e0, 5.52146e0, 5.61677e0, 5.70378e0, 5.78383e0,
     & 5.85793e0, 5.92693e0, 5.99146e0, 6.10925e0, 6.21461e0, 6.30992e0,
     & 6.39693e0, 6.47697e0, 6.55108e0, 6.62007e0, 6.68461e0, 6.74524e0,
     & 6.80239e0, 6.85646e0, 6.90776e0, 6.95655e0, 7.00307e0/
      data cp /
     & 2087.00e0, 2106.20e0, 2141.40e0, 2186.90e0, 2240.80e0, 2301.90e0,
     & 2369.50e0, 2443.00e0, 2522.00e0, 2703.00e0, 2889.00e0, 3074.00e0,
     & 3256.00e0, 3432.00e0, 3602.00e0, 3766.00e0, 3923.00e0, 4072.00e0,
     & 4214.00e0, 4348.00e0, 4475.00e0, 4595.00e0, 4708.00e0/
      data cpt /
     & 0.41978e0, 1.11622e0, 1.64336e0, 1.99433e0, 2.30727e0, 2.57658e0,
     & 2.83043e0, 3.03371e0, 3.33473e0, 3.74418e0, 3.70855e0, 3.68161e0,
     & 3.58502e0, 3.45832e0, 3.34170e0, 3.21489e0, 3.05874e0, 2.91014e0,
     & 2.76068e0, 2.60713e0, 2.47078e0, 2.32974e0, 2.19026e0/
      data h /
     &-0.21038e6,-0.15801e6,-0.10494e6,-0.05085e6, 0.00448e6, 0.06125e6,
     & 0.11963e6, 0.17977e6, 0.24182e6, 0.37236e6, 0.51217e6, 0.66125e6,
     & 0.81952e6, 0.98674e6, 1.16262e6, 1.34684e6, 1.53910e6, 1.73901e6,
     & 1.94619e6, 2.16027e6, 2.38087e6, 2.60765e6, 2.84026e6/
      data s /
     &-0.85408e3,-0.60735e3,-0.38373e3,-0.17757e3, 0.01497e3, 0.19670e3,
     & 0.36972e3, 0.53568e3, 0.69583e3, 1.00314e3, 1.29757e3, 1.58161e3,
     & 1.85692e3, 2.12453e3, 2.38512e3, 2.63926e3, 2.88737e3, 3.12971e3,
     & 3.36651e3, 3.59798e3, 3.82426e3, 4.04553e3, 4.26192e3/
c- - - - - - - - - - -
      ilow = 1
      i = ndim
c- - - - - - - - - - -
 10   continue
      if(i .le. ilow+1) go to 11
 
      imid = (i+ilow)/2
      if(t1 .lt. t(imid)) then
       i = imid
      else
       ilow = imid
      endif
      go to 10
c- - - - - - - - - - -
 11   continue
 
      dt = t(i) - t(i-1)
      f = (t1 - t(i-1)) / dt
 
      dtl = tl(i) - tl(i-1)
      fl = (log(t1) - tl(i-1)) / dtl
c- - - - - - - - - - -
      s1    =          (1.0-fl)* s(i-1) 
     &               +      fl * s(i) 
     & + fl*(1.0-fl)*( (1.0-fl)*(dtl*cp(i-1) - s(i) + s(i-1))
     &                    - fl *(dtl*cp(i)   - s(i) + s(i-1)) )
 
      h1  = hform  + (1.0-f)* h(i-1) 
     &             +      f * h(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cp(i-1) - h(i) + h(i-1))
     &                  - f *(dt*cp(i)   - h(i) + h(i-1)) )
 
      cp1   =        (1.0-f)* cp(i-1) 
     &             +      f * cp(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cpt(i-1) - cp(i) + cp(i-1))
     &                  - f *(dt*cpt(i)   - cp(i) + cp(i-1)) )
 
      r1 = r
c- - - - - - - - - - -
      return
      end ! gas_CH4


      subroutine gas_C2H6(t1, s1,h1,cp1,r1)
      parameter (ndim= 17)
      real t(ndim), tl(ndim), cp(ndim),
     &     cpt(ndim), h(ndim), s(ndim)
      data r, hform /   277.15    , -0.27930E+07 /
      data t /
     &  250.00e0,  275.00e0,  300.00e0,  325.00e0,  350.00e0,  375.00e0,
     &  400.00e0,  450.00e0,  500.00e0,  550.00e0,  600.00e0,  650.00e0,
     &  700.00e0,  750.00e0,  800.00e0,  850.00e0,  900.00e0/
      data tl /
     & 5.52146e0, 5.61677e0, 5.70378e0, 5.78383e0, 5.85793e0, 5.92693e0,
     & 5.99146e0, 6.10925e0, 6.21461e0, 6.30992e0, 6.39693e0, 6.47697e0,
     & 6.55108e0, 6.62007e0, 6.68461e0, 6.74524e0, 6.80239e0/
      data cp /
     & 1535.00e0, 1651.00e0, 1766.00e0, 1878.00e0, 1987.00e0, 2095.00e0,
     & 2199.00e0, 2402.00e0, 2596.00e0, 2782.00e0, 2958.00e0, 3126.00e0,
     & 3286.00e0, 3438.00e0, 3581.00e0, 3717.00e0, 3846.00e0/
      data cpt /
     & 4.65042e0, 4.62958e0, 4.55127e0, 4.40533e0, 4.34739e0, 4.24511e0,
     & 4.11215e0, 3.97684e0, 3.80048e0, 3.62125e0, 3.43452e0, 3.28066e0,
     & 3.12283e0, 2.94802e0, 2.78509e0, 2.65164e0, 2.50836e0/
      data h /
     &-0.07902e6,-0.03919e6, 0.00352e6, 0.04908e6, 0.09740e6, 0.14843e6,
     & 0.20211e6, 0.31716e6, 0.44215e6, 0.57664e6, 0.72017e6, 0.87231e6,
     & 1.03264e6, 1.20078e6, 1.37628e6, 1.55876e6, 1.74787e6/
      data s /
     &-0.28856e3,-0.13682e3, 0.01178e3, 0.15759e3, 0.30076e3, 0.44154e3,
     & 0.58009e3, 0.85089e3, 1.11408e3, 1.37030e3, 1.61998e3, 1.86343e3,
     & 2.10099e3, 2.33294e3, 2.55943e3, 2.78064e3, 2.99678e3/
c- - - - - - - - - - -
      ilow = 1
      i = ndim
c- - - - - - - - - - -
 10   continue
      if(i .le. ilow+1) go to 11
 
      imid = (i+ilow)/2
      if(t1 .lt. t(imid)) then
       i = imid
      else
       ilow = imid
      endif
      go to 10
c- - - - - - - - - - -
 11   continue
 
      dt = t(i) - t(i-1)
      f = (t1 - t(i-1)) / dt
 
      dtl = tl(i) - tl(i-1)
      fl = (log(t1) - tl(i-1)) / dtl
c- - - - - - - - - - -
      s1    =          (1.0-fl)* s(i-1) 
     &               +      fl * s(i) 
     & + fl*(1.0-fl)*( (1.0-fl)*(dtl*cp(i-1) - s(i) + s(i-1))
     &                    - fl *(dtl*cp(i)   - s(i) + s(i-1)) )
 
      h1  = hform  + (1.0-f)* h(i-1) 
     &             +      f * h(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cp(i-1) - h(i) + h(i-1))
     &                  - f *(dt*cp(i)   - h(i) + h(i-1)) )
 
      cp1   =        (1.0-f)* cp(i-1) 
     &             +      f * cp(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cpt(i-1) - cp(i) + cp(i-1))
     &                  - f *(dt*cpt(i)   - cp(i) + cp(i-1)) )
 
      r1 = r
c- - - - - - - - - - -
      return
      end ! gas_C2H6


      subroutine gas_C3H8(t1, s1,h1,cp1,r1)
      parameter (ndim= 17)
      real t(ndim), tl(ndim), cp(ndim),
     &     cpt(ndim), h(ndim), s(ndim)
      data r, hform /   188.50    , -0.23570E+07 /
      data t /
     &  250.00e0,  275.00e0,  300.00e0,  325.00e0,  350.00e0,  375.00e0,
     &  400.00e0,  450.00e0,  500.00e0,  550.00e0,  600.00e0,  650.00e0,
     &  700.00e0,  750.00e0,  800.00e0,  850.00e0,  900.00e0/
      data tl /
     & 5.52146e0, 5.61677e0, 5.70378e0, 5.78383e0, 5.85793e0, 5.92693e0,
     & 5.99146e0, 6.10925e0, 6.21461e0, 6.30992e0, 6.39693e0, 6.47697e0,
     & 6.55108e0, 6.62007e0, 6.68461e0, 6.74524e0, 6.80239e0/
      data cp /
     & 1448.00e0, 1564.00e0, 1679.00e0, 1791.00e0, 1900.00e0, 2008.00e0,
     & 2112.00e0, 2315.00e0, 2509.00e0, 2695.00e0, 2871.00e0, 3039.00e0,
     & 3199.00e0, 3351.00e0, 3494.00e0, 3630.00e0, 3759.00e0/
      data cpt /
     & 4.65042e0, 4.62958e0, 4.55127e0, 4.40534e0, 4.34738e0, 4.24511e0,
     & 4.11216e0, 3.97684e0, 3.80048e0, 3.62125e0, 3.43452e0, 3.28066e0,
     & 3.12283e0, 2.94802e0, 2.78509e0, 2.65164e0, 2.50836e0/
      data h /
     &-0.07484e6,-0.03719e6, 0.00335e6, 0.04673e6, 0.09287e6, 0.14173e6,
     & 0.19323e6, 0.30394e6, 0.42457e6, 0.55471e6, 0.69390e6, 0.84168e6,
     & 0.99767e6, 1.16145e6, 1.33261e6, 1.51074e6, 1.69549e6/
      data s /
     &-0.27328e3,-0.12983e3, 0.01120e3, 0.15004e3, 0.28676e3, 0.42155e3,
     & 0.55448e3, 0.81503e3, 1.06905e3, 1.31698e3, 1.55909e3, 1.79558e3,
     & 2.02670e3, 2.25264e3, 2.47352e3, 2.68945e3, 2.90062e3/
c- - - - - - - - - - -
      ilow = 1
      i = ndim
c- - - - - - - - - - -
 10   continue
      if(i .le. ilow+1) go to 11
 
      imid = (i+ilow)/2
      if(t1 .lt. t(imid)) then
       i = imid
      else
       ilow = imid
      endif
      go to 10
c- - - - - - - - - - -
 11   continue
 
      dt = t(i) - t(i-1)
      f = (t1 - t(i-1)) / dt
 
      dtl = tl(i) - tl(i-1)
      fl = (log(t1) - tl(i-1)) / dtl
c- - - - - - - - - - -
      s1    =          (1.0-fl)* s(i-1) 
     &               +      fl * s(i) 
     & + fl*(1.0-fl)*( (1.0-fl)*(dtl*cp(i-1) - s(i) + s(i-1))
     &                    - fl *(dtl*cp(i)   - s(i) + s(i-1)) )
 
      h1  = hform  + (1.0-f)* h(i-1) 
     &             +      f * h(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cp(i-1) - h(i) + h(i-1))
     &                  - f *(dt*cp(i)   - h(i) + h(i-1)) )
 
      cp1   =        (1.0-f)* cp(i-1) 
     &             +      f * cp(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cpt(i-1) - cp(i) + cp(i-1))
     &                  - f *(dt*cpt(i)   - cp(i) + cp(i-1)) )
 
      r1 = r
c- - - - - - - - - - -
      return
      end ! gas_C3H8


      subroutine gas_C4H10(t1, s1,h1,cp1,r1)
      parameter (ndim= 17)
      real t(ndim), tl(ndim), cp(ndim),
     &     cpt(ndim), h(ndim), s(ndim)
      data r, hform /   143.30    , -0.21640E+07 /
      data t /
     &  250.00e0,  275.00e0,  300.00e0,  325.00e0,  350.00e0,  375.00e0,
     &  400.00e0,  450.00e0,  500.00e0,  550.00e0,  600.00e0,  650.00e0,
     &  700.00e0,  750.00e0,  800.00e0,  850.00e0,  900.00e0/
      data tl /
     & 5.52146e0, 5.61677e0, 5.70378e0, 5.78383e0, 5.85793e0, 5.92693e0,
     & 5.99146e0, 6.10925e0, 6.21461e0, 6.30992e0, 6.39693e0, 6.47697e0,
     & 6.55108e0, 6.62007e0, 6.68461e0, 6.74524e0, 6.80239e0/
      data cp /
     & 1485.00e0, 1601.00e0, 1716.00e0, 1828.00e0, 1937.00e0, 2045.00e0,
     & 2149.00e0, 2352.00e0, 2546.00e0, 2732.00e0, 2908.00e0, 3076.00e0,
     & 3236.00e0, 3388.00e0, 3531.00e0, 3667.00e0, 3796.00e0/
      data cpt /
     & 4.65042e0, 4.62958e0, 4.55127e0, 4.40533e0, 4.34739e0, 4.24512e0,
     & 4.11215e0, 3.97684e0, 3.80048e0, 3.62125e0, 3.43452e0, 3.28066e0,
     & 3.12283e0, 2.94802e0, 2.78509e0, 2.65164e0, 2.50836e0/
      data h /
     &-0.07662e6,-0.03804e6, 0.00342e6, 0.04773e6, 0.09480e6, 0.14458e6,
     & 0.19701e6, 0.30956e6, 0.43205e6, 0.56404e6, 0.70507e6, 0.85471e6,
     & 1.01254e6, 1.17818e6, 1.35118e6, 1.53116e6, 1.71777e6/
      data s /
     &-0.27978e3,-0.13280e3, 0.01145e3, 0.15325e3, 0.29271e3, 0.43005e3,
     & 0.56537e3, 0.83028e3, 1.08820e3, 1.33966e3, 1.58498e3, 1.82443e3,
     & 2.05829e3, 2.28679e3, 2.51005e3, 2.72823e3, 2.94152e3/
c- - - - - - - - - - -
      ilow = 1
      i = ndim
c- - - - - - - - - - -
 10   continue
      if(i .le. ilow+1) go to 11
 
      imid = (i+ilow)/2
      if(t1 .lt. t(imid)) then
       i = imid
      else
       ilow = imid
      endif
      go to 10
c- - - - - - - - - - -
 11   continue
 
      dt = t(i) - t(i-1)
      f = (t1 - t(i-1)) / dt
 
      dtl = tl(i) - tl(i-1)
      fl = (log(t1) - tl(i-1)) / dtl
c- - - - - - - - - - -
      s1    =          (1.0-fl)* s(i-1) 
     &               +      fl * s(i) 
     & + fl*(1.0-fl)*( (1.0-fl)*(dtl*cp(i-1) - s(i) + s(i-1))
     &                    - fl *(dtl*cp(i)   - s(i) + s(i-1)) )
 
      h1  = hform  + (1.0-f)* h(i-1) 
     &             +      f * h(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cp(i-1) - h(i) + h(i-1))
     &                  - f *(dt*cp(i)   - h(i) + h(i-1)) )
 
      cp1   =        (1.0-f)* cp(i-1) 
     &             +      f * cp(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cpt(i-1) - cp(i) + cp(i-1))
     &                  - f *(dt*cpt(i)   - cp(i) + cp(i-1)) )
 
      r1 = r
c- - - - - - - - - - -
      return
      end ! gas_C4H10


      subroutine gas_C8H18(t1, s1,h1,cp1,r1)
      parameter (ndim= 17)
      real t(ndim), tl(ndim), cp(ndim),
     &     cpt(ndim), h(ndim), s(ndim)
      data r, hform /   72.900    , -0.18280E+07 /
      data t /
     &  250.00e0,  275.00e0,  300.00e0,  325.00e0,  350.00e0,  375.00e0,
     &  400.00e0,  450.00e0,  500.00e0,  550.00e0,  600.00e0,  650.00e0,
     &  700.00e0,  750.00e0,  800.00e0,  850.00e0,  900.00e0/
      data tl /
     & 5.52146e0, 5.61677e0, 5.70378e0, 5.78383e0, 5.85793e0, 5.92693e0,
     & 5.99146e0, 6.10925e0, 6.21461e0, 6.30992e0, 6.39693e0, 6.47697e0,
     & 6.55108e0, 6.62007e0, 6.68461e0, 6.74524e0, 6.80239e0/
      data cp /
     & 1480.00e0, 1596.00e0, 1711.00e0, 1823.00e0, 1932.00e0, 2040.00e0,
     & 2144.00e0, 2347.00e0, 2541.00e0, 2727.00e0, 2903.00e0, 3071.00e0,
     & 3231.00e0, 3383.00e0, 3526.00e0, 3662.00e0, 3791.00e0/
      data cpt /
     & 4.65042e0, 4.62958e0, 4.55127e0, 4.40533e0, 4.34739e0, 4.24511e0,
     & 4.11215e0, 3.97684e0, 3.80048e0, 3.62125e0, 3.43452e0, 3.28066e0,
     & 3.12283e0, 2.94802e0, 2.78509e0, 2.65164e0, 2.50836e0/
      data h /
     &-0.07638e6,-0.03793e6, 0.00341e6, 0.04760e6, 0.09454e6, 0.14419e6,
     & 0.19650e6, 0.30880e6, 0.43104e6, 0.56278e6, 0.70356e6, 0.85295e6,
     & 1.01053e6, 1.17592e6, 1.34867e6, 1.52840e6, 1.71476e6/
      data s /
     &-0.27890e3,-0.13240e3, 0.01141e3, 0.15282e3, 0.29191e3, 0.42890e3,
     & 0.56390e3, 0.82822e3, 1.08561e3, 1.33659e3, 1.58148e3, 1.82054e3,
     & 2.05402e3, 2.28217e3, 2.50512e3, 2.72299e3, 2.93599e3/
c- - - - - - - - - - -
      ilow = 1
      i = ndim
c- - - - - - - - - - -
 10   continue
      if(i .le. ilow+1) go to 11
 
      imid = (i+ilow)/2
      if(t1 .lt. t(imid)) then
       i = imid
      else
       ilow = imid
      endif
      go to 10
c- - - - - - - - - - -
 11   continue
 
      dt = t(i) - t(i-1)
      f = (t1 - t(i-1)) / dt
 
      dtl = tl(i) - tl(i-1)
      fl = (log(t1) - tl(i-1)) / dtl
c- - - - - - - - - - -
      s1    =          (1.0-fl)* s(i-1) 
     &               +      fl * s(i) 
     & + fl*(1.0-fl)*( (1.0-fl)*(dtl*cp(i-1) - s(i) + s(i-1))
     &                    - fl *(dtl*cp(i)   - s(i) + s(i-1)) )
 
      h1  = hform  + (1.0-f)* h(i-1) 
     &             +      f * h(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cp(i-1) - h(i) + h(i-1))
     &                  - f *(dt*cp(i)   - h(i) + h(i-1)) )
 
      cp1   =        (1.0-f)* cp(i-1) 
     &             +      f * cp(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cpt(i-1) - cp(i) + cp(i-1))
     &                  - f *(dt*cpt(i)   - cp(i) + cp(i-1)) )
 
      r1 = r
c- - - - - - - - - - -
      return
      end ! gas_C8H18


      subroutine gas_C14H30(t1, s1,h1,cp1,r1)
      parameter (ndim= 17)
      real t(ndim), tl(ndim), cp(ndim),
     &     cpt(ndim), h(ndim), s(ndim)
      data r, hform /   167.00    , -0.16700E+07 /
      data t /
     &  250.00e0,  275.00e0,  300.00e0,  325.00e0,  350.00e0,  375.00e0,
     &  400.00e0,  450.00e0,  500.00e0,  550.00e0,  600.00e0,  650.00e0,
     &  700.00e0,  750.00e0,  800.00e0,  850.00e0,  900.00e0/
      data tl /
     & 5.52146e0, 5.61677e0, 5.70378e0, 5.78383e0, 5.85793e0, 5.92693e0,
     & 5.99146e0, 6.10925e0, 6.21461e0, 6.30992e0, 6.39693e0, 6.47697e0,
     & 6.55108e0, 6.62007e0, 6.68461e0, 6.74524e0, 6.80239e0/
      data cp /
     & 1469.00e0, 1585.00e0, 1700.00e0, 1812.00e0, 1921.00e0, 2029.00e0,
     & 2133.00e0, 2336.00e0, 2530.00e0, 2716.00e0, 2892.00e0, 3060.00e0,
     & 3220.00e0, 3372.00e0, 3515.00e0, 3651.00e0, 3780.00e0/
      data cpt /
     & 4.65042e0, 4.62958e0, 4.55127e0, 4.40533e0, 4.34739e0, 4.24511e0,
     & 4.11215e0, 3.97684e0, 3.80048e0, 3.62125e0, 3.43452e0, 3.28066e0,
     & 3.12283e0, 2.94802e0, 2.78509e0, 2.65164e0, 2.50836e0/
      data h /
     &-0.07585e6,-0.03768e6, 0.00339e6, 0.04730e6, 0.09396e6, 0.14334e6,
     & 0.19538e6, 0.30713e6, 0.42882e6, 0.56000e6, 0.70024e6, 0.84907e6,
     & 1.00611e6, 1.17094e6, 1.34315e6, 1.52233e6, 1.70814e6/
      data s /
     &-0.27697e3,-0.13152e3, 0.01134e3, 0.15186e3, 0.29014e3, 0.42638e3,
     & 0.56066e3, 0.82368e3, 1.07992e3, 1.32985e3, 1.57379e3, 1.81196e3,
     & 2.04463e3, 2.27202e3, 2.49425e3, 2.71146e3, 2.92383e3/
c- - - - - - - - - - -
      ilow = 1
      i = ndim
c- - - - - - - - - - -
 10   continue
      if(i .le. ilow+1) go to 11
 
      imid = (i+ilow)/2
      if(t1 .lt. t(imid)) then
       i = imid
      else
       ilow = imid
      endif
      go to 10
c- - - - - - - - - - -
 11   continue
 
      dt = t(i) - t(i-1)
      f = (t1 - t(i-1)) / dt
 
      dtl = tl(i) - tl(i-1)
      fl = (log(t1) - tl(i-1)) / dtl
c- - - - - - - - - - -
      s1    =          (1.0-fl)* s(i-1) 
     &               +      fl * s(i) 
     & + fl*(1.0-fl)*( (1.0-fl)*(dtl*cp(i-1) - s(i) + s(i-1))
     &                    - fl *(dtl*cp(i)   - s(i) + s(i-1)) )
 
      h1  = hform  + (1.0-f)* h(i-1) 
     &             +      f * h(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cp(i-1) - h(i) + h(i-1))
     &                  - f *(dt*cp(i)   - h(i) + h(i-1)) )
 
      cp1   =        (1.0-f)* cp(i-1) 
     &             +      f * cp(i) 
     & + f*(1.0-f)*( (1.0-f)*(dt*cpt(i-1) - cp(i) + cp(i-1))
     &                  - f *(dt*cpt(i)   - cp(i) + cp(i-1)) )
 
      r1 = r
c- - - - - - - - - - -
      return
      end ! gas_C14H30
