
      program gastest
c------------------------------------------------------------------
c     Exercises the various gas_* routines for air and for the
c     selected fuel (via ifuel).  Steps through engine setting
c     station values.  
c------------------------------------------------------------------
      parameter (n=6)
      real alpha(n), beta(n), gamma(n), lambda(n)
      real m0

      real alf(n)

c
c---- air fractions  N2      O2      CO2    H2O      Ar       fuel
      data alpha / 0.781 , 0.209 , 0.0004, 0.0   , 0.00965,  0.0 /
c
c---- fuel fractions
      data beta  / 0.0   , 0.0   , 0.0   , 0.0   , 0.0    ,  1.0 /
c
      nair = n-1
c
c=====================================================================
c---- set fuel, put in last constituent slot n
c      ifuel = 11  ! methane
c      ifuel = 12  ! ethane
c      ifuel = 13  ! propane
c      ifuel = 14  ! butane
c      ifuel = 18  ! octane  
c      ifuel = 24  ! kerosene  (C14H30)
c
c      hfuel = 55.0e6  ! methane 
c      hfuel = 52.0e6  ! ethane  
c      hfuel = 49.0e6  ! propane 
c      hfuel = 46.0e6  ! butane  
c      hfuel = 44.0e6  ! octane  
c      hfuel = 43.0e6  ! kerosene  (C14H30)
c
      ifuel = 11
      hfuel = 55.0e6
c
      call gasfuel(ifuel,gamma,n)
c
cc---------------------------------------------------------------------
cc---- evaluate properties over temperatures for chosen constituent i
cc-     (see gasfun.f header for list of constituents)
c      i = 1
c      nt = 501
c      do it = 1, nt
c        t = 250.0 + (2000.0-250.0)*float(it-1)/float(nt-1)
c        tl = log(t)
c        call gasfun(i,t, s,dsdt,h,dhdt,cp,r)
c        write(1,4444) t,tl,s,dsdt,h,dhdt,cp,r
c 4444   format(1x,9g14.6)
c      enddo
c      stop
cc---------------------------------------------------------------------

cc--------------------------------------------------------
cc---- evaluate properties over temperatures for air
c      nt = 101
c      do it = 1, nt
c        frac = float(it-1) / float(nt-1)
c        t0 = 175.0 + frac*(2000.0-175.0)
cc
c        call gassum(alpha,nair, t0, s0,dsdt,h0,dhdt,cp0,r0)
c        gam0 = cp0/(cp0-r0)
c        a0 = sqrt(gam0*r0*t0)
cc
c        if(it.eq.1) h0ref = h0 - t0*r0*3.5
cc
c        write(*,'(1x,8g13.5)') t0, cp0, (h0-h0ref)/r0/3.5, gam0, a0
c      enddo
c      stop
c--------------------------------------------------------

c===============================================================
c---- specify reference state and Mach number
      t0 = 215.0
      p0 = 0.22e5
      m0 = 0.8

      call gassum(alpha,nair, t0, s0,dsdt,h0,dhdt,cp0,r0)
      gam0 = cp0/(cp0-r0)
      a0 = sqrt(gam0*r0*t0)
      u0 = m0*a0

c===============================================================
c---- total freestream
      hspec = h0 + 0.5*u0**2
      tguess = t0*(1.0 + 0.5*(gam0-1.0)*m0**2)
      call gas_tset(alpha,nair, hspec,tguess, t1)
c 
      call gassum(alpha,nair, t1, s1,dsdt,h1,dhdt,cp1,r1)
      p1 = p0 * exp((s1-s0)/r1)
c
c===============================================================
c---- diffuser
      hspec = h0 + 0.5*u0**2
      tguess = t0*(1.0 + 0.5*(gam0-1.0)*m0**2)
      call gas_tset(alpha,nair, hspec,tguess, t2)
c 
      epold = 0.99
      call gassum(alpha,nair, t2, s2,dsdt,h2,dhdt,cp2,r2)
      p2 = p0 * exp(epold*(s2-s0)/r2)
      gam2 = cp2/(cp2-r2)
c
c===============================================================
c---- compressor
      pic = 20.0
      epolc = 0.95
      call gassum(alpha,nair, t2, s2,dsdt,h2,dhdt,cp2,r2)
      call gas_prat(alpha,nair, p2,t2,h2,s2,cp2,r2, pic,epolc,
     &                          p3,t3,h3,s3,cp3,r3 )


      gam3 = cp3/(cp3-r3)
      gama = 0.5*(gam2+gam3)
      gamb = 0.5*(cp2+cp3)/(0.5*(cp2+cp3)-0.5*(r2+r3))

      p3 = p2 * exp( epolc*(s3-s2)/r2 )

      tauc  = t3/t2
      tauc2 = pic**((gam2-1.0)/(gam2*epolc))
      tauc3 = pic**((gam3-1.0)/(gam3*epolc))
      tauca = pic**((gama-1.0)/(gama*epolc))
      taucb = pic**((gamb-1.0)/(gamb*epolc))

      write(*,*)
      write(*,*) 'a0 m0 =', a0, m0
      write(*,*) 'u0    =', u0
      write(*,*) 'p0 t0 =', p0, t0

      write(*,*)
      write(*,*) 'p1 t1 =', p1, t1
      write(*,*) 'p2 t2 =', p2, t2
      write(*,*) 'p3 t3 =', p3, t3
      write(*,*) 'cp2 gam2 =', cp2, gam2
      write(*,*) 'cp3 gam3 =', cp3, gam3

      write(*,*) 'tauc   = ', tauc
      write(*,*) 'tauc2  = ', tauc2, tauc2/tauc
      write(*,*) 'tauc3  = ', tauc3, tauc3/tauc
      write(*,*) 'tauca  = ', tauca, tauca/tauc
      write(*,*) 'taucb  = ', taucb, taucb/tauc



c===============================================================
c---- combustor
      tf = t3
      t4 = 2000.0
      pib = 0.99
      call gas_burn(alpha,beta,gamma,n,ifuel, t3,tf,t4, f,lambda)
      call gassum(lambda,nair, t4, s4,dsdt,h4,dhdt,cp4,r4)



      gam4 = cp4/(cp4-r4)
      p4 = p3*pib
      cpa = 0.5*(cp3+cp4)
      hfuel34 = ((1.0+f)*t4 - t3)*cpa / f

      write(*,*)
      write(*,*) 'f  =', f
      write(*,*) 'hfuel   =', hfuel
      write(*,*) 'hfuel34 =', hfuel34


c===============================================================
c---- turbine
      delh = -(h3-h2) / (1.0 + f)
      epolt = 0.94

      epolti = 1.0/epolt
      call gas_delh(lambda,nair, p4,t4,h4,s4,cp4,r4, delh,epolti,
     &                           p5,t5,h5,s5,cp5,r5 )

      gam5 = cp5/(cp5-r5)
      gama = 0.5*(gam4+gam5)
      gamb = 0.5*(cp4+cp5)/(0.5*(cp4+cp5)-0.5*(r4+r5))

      pit  =  p5/p4
      pit4 = (t5/t4)**(gam4/(gam4-1.0)/epolt)
      pit5 = (t5/t4)**(gam5/(gam5-1.0)/epolt)
      pita = (t5/t4)**(gama/(gama-1.0)/epolt)
      pitb = (t5/t4)**(gamb/(gamb-1.0)/epolt)

      write(*,*)
      write(*,*) 'delh  =', delh
      write(*,*) 'p4 t4 =', p4, t4
      write(*,*) 'p5 t5 =', p5, t5
      write(*,*) 'cp4 gam4 =', cp4, gam4
      write(*,*) 'cp5 gam5 =', cp5, gam5

      write(*,*) 'pit  = ', pit
      write(*,*) 'pit4 = ', pit4, pit4/pit
      write(*,*) 'pit5 = ', pit5, pit5/pit
      write(*,*) 'pita = ', pita, pita/pit
      write(*,*) 'pitb = ', pitb, pitb/pit

      stop
      end

