
      subroutine getparm(lu,idim,jdim,
     &                   ni,nj,ispars,jspars,
     &                   parsi,parsj,dvarso)
c==================================================================
c     Reads *.tas file from logical unit lu.
c     Puts data into global arrays declared in the include files.
c==================================================================
      implicit real (a-h,l-z)

      integer lu, idim, jdim, ni, nj, naf
      real parsi(*), parsj(*), dvarso(*)

      include 'index.inc'
      include 'airf.inc'
      include 'arrs.inc'
      include 'constants.inc'

      character*256 line

      integer ninpx, ninp
      parameter(ninpx = 20+nmisx)
      real rinp(ninpx)

      real Npax(nmisx), Wpax(nmisx)

      logical letter 

c---- no missions defined yet
      nmission = 0


c---- initialize file-line counter
      iline = 0

c---- start reading file on logical unit lu
      call getcval(lu,iline,configname)
      call getcval(lu,iline,casename1)
      call getcval(lu,iline,casename2)

c------------------------------------------------------------------
c---- execution control flags and limits
      call getLval(lu,iline,Litprint)
      call getival(lu,iline,iterwmax)
      call getival(lu,iline,iterfmax)

cc    call getrval(lu,iline,wrlx1)
cc    call getrval(lu,iline,wrlx2)
cc    call getrval(lu,iline,wrlx3)
      wrlx1 = 0.5
      wrlx2 = 0.9
      wrlx3 = 0.5

      call getLval(lu,iline,Lopt)
      call getLval(lu,iline,Loprint)
      call getival(lu,iline,istepmax)
      call getival(lu,iline,istepout)
      call getrval(lu,iline,Wftol)

      call getLval(lu,iline,LlBFcon)
      call getLval(lu,iline,LWfmaxcon)
      call getLval(lu,iline,Lbmaxcon)
      call getLval(lu,iline,Lgtoccon)

cc    call getLval(lu,iline,LdBSLcon)
cc    call getLval(lu,iline,LdBCBcon)

      call getLval(lu,iline,Loutwrite)
      call getLval(lu,iline,Lfblwrite)
      call getLval(lu,iline,Ltrpwrite)

      call getLval(lu,iline,Lsavwrite)
cc    call getLval(lu,iline,Ltfwrite)
cc    call getLval(lu,iline,Lpfwrite)

      call getLval(lu,iline,Laswwrite)

c------------------------------------------------------------------

 1000 format(a)

c------------------------------------------------------------------
c---- read i-parameter values
      call getcval(lu,iline,line)
      do is = 1, istotal
        ni = idim+1
        call getrkey(line,cpars(is),parsi,ni)
        if    (ni .eq. -1) then
c------- read error
         go to 800
        elseif(ni .eq.  0) then
c------- no sequence values given... will use what's specified elsewhere in file
         ispars = 0
         ni = 1
         go to 15
        elseif(ni .gt.  0) then
c------- ni sequence values were given, save index "is" of which parameter it is
         ispars = is
         go to 15
        endif
      enddo
      write(*,1510) iline, line
 1510 format(/' Unrecognized i-parameter keyword in line', i5,':  ',a,
     &       /' Valid keywords cpars(.) are declared in index.inc'/)
      stop

 15   continue
      call getcval(lu,iline,pilabel)
      call getcval(lu,iline,picolor)

c------------------------------------------------------------------
c---- read j-parameter values
      call getcval(lu,iline,line)

      do js = 1, istotal
        nj = jdim+1
        call getrkey(line,cpars(js),parsj,nj)
        if    (nj .eq. -1) then
c------- read error
         go to 800
        elseif(nj .eq.  0) then
c------- no sequence values given... will use what's specified elsewhere in file
         jspars = 0
         nj = 1
         go to 25
        elseif(nj .gt.  0) then
c------- nj sequence values were given, save index "js" of which parameter it is
         jspars = js
         go to 25
        endif
      enddo
      write(*,2510) iline, line
2510  format(/' Unrecognized j-parameter keyword in line', i5,':  ',a,
     &       /' Valid keywords cpars(.) are declared in index.inc'/)
      stop

 25   continue
      call getcval(lu,iline,pjlabel)

c------------------------------------------------------------------
c---- read variables to be optimized, their initial values, and deltas
      do ko = 1, iototal
        dvarso(ko) = 0.0
      enddo

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c---- start optimization variable-reading loop
 30   continue
      call getcval(lu,iline,line)

c---- strip leading blanks
      call strip(line,inum)

c---- does the line begin with a possible keyword (a letter)?
      ichar1 = ichar(line(1:1))
      letter = (ichar1 .ge. ichar('a') .and. ichar1 .le. ichar('z'))
     &   .or.  (ichar1 .ge. ichar('A') .and. ichar1 .le. ichar('Z')) 
      if(.not.letter) then
c----- no keyword... step back to this line, and go keep reading file
       backspace(lu)
       iline = iline - 1
       go to 35
      endif

c---- test against all valid keywords
      do ko = 1, iototal
        ninp = 2
        call getrkey(line,cparo(ko),rinp,ninp)
        if    (ninp .eq. -1) then
c------- read error
         go to 800
        elseif(ninp .eq.  0) then
c------- valid opt. variable... but will not be optimized since no perturbation given
c-       go keep reading lines
         go to 30
        elseif(ninp .gt.  0) then
c------- valid opt. variable... save its specified initial perturbation, 
c-       go keep reading lines
         dvarso(ko) = rinp(1)
         go to 30
        endif
      enddo

      write(*,3510) iline, line
 3510 format(
     & /' Unrecognized optimized-variable keyword in line', i5,
     &    ':  ',a,
     & /' Valid keywords cparo(.) are declared in index.inc'/)
      stop
c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

c---- pick up here after all the optimization-variable lines
 35   continue

c------------------------------------------------------------------
c---- mission weighting fractions wOpt for computing overall fleet objective
      call getcval(lu,iline,line)
      ninp = ninpx
      call getrkey(line,' ', rinp,ninp)
      if(ninp .lt. 0) go to 800

c---- set number of missions
      if(ninp .gt. nmisx) then
       write(*,*) 
     &  'Number of missions limited to array limit  nmisx =', nmisx
      endif
      nmission = min( ninp , nmisx )

c---- normalize mission weights
      wsum = 0.
      do km = 1, nmission
        wsum = wsum + rinp(km)
      enddo
      if(wsum .eq. 0.0) then
       write(*,*) '* Warning:  Sum(wObj) = 0'
       rinp(1) = 1.0
       wsum = 1.0
      endif
      do km = 1, nmission
        parm(imwOpt,km) = rinp(km) / wsum
      enddo

c- - - - - - - - - - - - - - - -
c---- mission Range(s)
      call getcval(lu,iline,line)
      ninp = nmisx
      call getrkey(line,' ', rinp,ninp)
      if(ninp .lt. 0) go to 800
c---- pad with last value if too few numbers given
      do km = ninp+1, nmission
        rinp(km) = rinp(ninp)
      enddo

      do km = 1, nmission
        parm(imRange,km) = rinp(km)
      enddo

c- - - - - - - - - - - - - - - -
c---- mission Npax(s)
      call getcval(lu,iline,line)
      ninp = nmisx
      call getrkey(line,' ', rinp,ninp)
      if(ninp .lt. 0) go to 800
      do km = ninp+1, nmission
        rinp(km) = rinp(ninp)
      enddo

      do km = 1, nmission
        Npax(km) = rinp(km)
      enddo

c- - - - - - - - - - - - - - - -
c---- mission Wpax(s)
      call getcval(lu,iline,line)
      ninp = nmisx
      call getrkey(line,' ', rinp,ninp)
      if(ninp .lt. 0) go to 800
      do km = ninp+1, nmission
        rinp(km) = rinp(ninp)
      enddo

      do km = 1, nmission
        Wpax(km) = rinp(km)
      enddo

c---- set resulting mission Wpay(s)
      do km = 1, nmission
        parm(imWpay,km) = Npax(km)*Wpax(km)
      enddo

c- - - - - - - - - - - - - - - -
c---- mission altTO(s)
      call getcval(lu,iline,line)
      ninp = nmisx
      call getrkey(line,' ', rinp,ninp)
      if(ninp .lt. 0) go to 800
      do km = ninp+1, nmission
        rinp(km) = rinp(ninp)
      enddo

      do km = 1, nmission
        parm(imaltTO,km) = rinp(km)
      enddo

c- - - - - - - - - - - - - - - -
c---- mission T0TO(s)
      call getcval(lu,iline,line)
      ninp = nmisx
      call getrkey(line,' ', rinp,ninp)
      if(ninp .lt. 0) go to 800
      do km = ninp+1, nmission
        rinp(km) = rinp(ninp)
      enddo

      do km = 1, nmission
        parm(imT0TO,km) = rinp(km)
      enddo



c- - - - - - - - - - - - - - - -
      if(.false.) then
c----- mission-varying excrescence factors disabled in this version
c-     ( also commented out in woper.f )

c---- mission fexwing
      call getcval(lu,iline,line)
      ninp = nmisx
      call getrkey(line,' ', rinp,ninp)
      if(ninp .lt. 0) go to 800
      do km = ninp+1, nmission
        rinp(km) = rinp(ninp)
      enddo

      do km = 1, nmission
        parm(imfexcdw,km) = rinp(km)
      enddo

c- - - - - - - - - - - - - - - -
c---- mission fextail
      call getcval(lu,iline,line)
      ninp = nmisx
      call getrkey(line,' ', rinp,ninp)
      if(ninp .lt. 0) go to 800
      do km = ninp+1, nmission
        rinp(km) = rinp(ninp)
      enddo

      do km = 1, nmission
        parm(imfexcdt,km) = rinp(km)
      enddo

c- - - - - - - - - - - - - - - -
c---- mission fexfuse
      call getcval(lu,iline,line)
      ninp = nmisx
      call getrkey(line,' ', rinp,ninp)
      if(ninp .lt. 0) go to 800
      do km = ninp+1, nmission
        rinp(km) = rinp(ninp)
      enddo

      do km = 1, nmission
        parm(imfexcdf,km) = rinp(km)
      enddo

      endif

c- - - - - - - - - - - - - - - -
cc---- mission altCR
c      call getcval(lu,iline,line)
c      ninp = nmisx
c      call getrkey(line,' ', rinp,ninp)
c      if(ninp .lt. 0) go to 800
c      do km = ninp+1, nmission
c        rinp(km) = rinp(ninp)
c      enddo
c
c      do km = 1, nmission
c        parm(imT0TO,km) = rinp(km)
c      enddo

c------------------------------------------------------------------
      call getrval(lu,iline,altCR )

c------------------------------------------------------------------
      call getrval(lu,iline,clpmax)
c      call getrval(lu,iline,eTO)

      do km = 1, nmission
c        do ip = ipstatic, ipclimb1
c          para(iaspaneff,ip,km) = eTO
c        enddo

        altTO = parm(imaltTO,km)
        do ip = ipstatic, ipcutback
          para(iaalt   ,ip,km) = altTO      ! takeoff pressure altitude
          para(iaclpmax,ip,km) = clpmax     ! wing max cl_perp
        enddo

        ip = ipclimb1
        para(iaalt   ,ip,km) = altTO
        para(iaclpmax,ip,km) = clpmax

        ip = ipcruise1
        para(iaalt,ip,km) = altCR

        ip = ipdescentn
        para(iaalt   ,ip,km) = altTO
        para(iaclpmax,ip,km) = clpmax
c        para(iaspaneff,ip,km) = eTO
      enddo

      call getrval(lu,iline,parg(igcdefan))
      call getrval(lu,iline,parg(igCDgear))
      call getrval(lu,iline,parg(igCDspoil))
      call getrval(lu,iline,parg(igmuroll))
      call getrval(lu,iline,parg(igmubrake))
      call getrval(lu,iline,parg(ighobst))

      call getrval(lu,iline,parg(iglBFmax) )
      call getrval(lu,iline,parg(iggtocmin) )

      call getrval(lu,iline,parg(igdBSLmax))
      call getrval(lu,iline,parg(igdBCBmax))

c---- prescribed cutback sighting angle and sin(climb angle)
      call getrval(lu,iline,tCBdeg)
      call getrval(lu,iline,gCBdeg)
      call getrval(lu,iline,gDE1deg)
      call getrval(lu,iline,gDEndeg)
      do km = 1, nmission
        parm(imthCB,km)    = tCBdeg * pi/180.0
        parm(imgamVCB,km)  = gCBdeg * pi/180.0
        parm(imgamVDE1,km) = gDE1deg * pi/180.0
        parm(imgamVDEn,km) = gDEndeg * pi/180.0
      enddo

c-----------------------------------------
c---- sizing-load parameters
      call getrval(lu,iline,parg(igNlift))
      call getrval(lu,iline,parg(igNland))
      call getrval(lu,iline,parg(igVne))

      call getrval(lu,iline,cabinalt)
      cabinkm = cabinalt/1000.0
      call atmos(cabinkm,dum,parg(igpcabin),dum,dum,dum)

C-----------------------------------------
C---- cruise parameters
      call getrval(lu,iline,CL)
      call getrval(lu,iline,Mach)

      do km = 1, nmission
c------ constant CL for entire mission except start and end 
        do ip = ipclimb1+1, ipdescentn-1
          para(iaCL,ip,km) = CL
        enddo

c------ constant Mach only at cruise altitude
        do ip = ipclimbn, ipdescent1
          para(iaMach,ip,km) = Mach
        enddo
      enddo

C-----------------------------------------
C-- basic wing parameters  (sweep is also used for tails, for simplicity)

      call getrval(lu,iline,parg(igsweep))
      call getrval(lu,iline,parg(igAR))

      call getrval(lu,iline,parg(igbmax))

      call getrval(lu,iline,parg(iglambdas))
      call getrval(lu,iline,parg(iglambdat))
      parg(iglambdas) = max( parg(iglambdas) , 0.001 )
      parg(iglambdat) = max( parg(iglambdat) , 0.001 )

      call getival(lu,iline,pari(iiwplan))
      call getival(lu,iline,pari(iifwcen))

      call getrval(lu,iline,parg(igrWfmax))

      call getrval(lu,iline,parg(igfLo))
      call getrval(lu,iline,parg(igfLt))

      call getrval(lu,iline,parg(igzs))


c---- set span of wing root bo
      call getrval(lu,iline,yo)
      parg(igbo) = 2.0*yo

      call getrval(lu,iline,parg(igetas))

      call getrval(lu,iline,parg(igrVstrut))

C-----------------------------------------
c--- tail download parameter for wing structure sizing case
      call getrval(lu,iline,parg(igCLhNrat))

C-----------------------------------------
C-- wing spanload parameters

c--- takeoff
      call getrval(lu,iline,rcls)
      call getrval(lu,iline,rclt)
      call getrval(lu,iline,cmpo)
      call getrval(lu,iline,cmps)
      call getrval(lu,iline,cmpt)
      do km = 1, nmission
        do ip = 1, ipclimb1
          para(iarcls,ip,km) = rcls
          para(iarclt,ip,km) = rclt
          para(iacmpo,ip,km) = cmpo
          para(iacmps,ip,km) = cmps
          para(iacmpt,ip,km) = cmpt
        enddo
      enddo

c--- cruise
      call getrval(lu,iline,rcls)
      call getrval(lu,iline,rclt)
      call getrval(lu,iline,cmpo)
      call getrval(lu,iline,cmps)
      call getrval(lu,iline,cmpt)
      do km = 1, nmission
        do ip = ipclimb1+1, ipdescentn-1
          para(iarcls,ip,km) = rcls
          para(iarclt,ip,km) = rclt
          para(iacmpo,ip,km) = cmpo
          para(iacmps,ip,km) = cmps
          para(iacmpt,ip,km) = cmpt
        enddo
      enddo

c--- landing
      call getrval(lu,iline,rcls)
      call getrval(lu,iline,rclt)
      call getrval(lu,iline,cmpo)
      call getrval(lu,iline,cmps)
      call getrval(lu,iline,cmpt)
      do km = 1, nmission
        ip = ipdescentn
ccc     do ip = ipdescentn, ipdescentn
          para(iarcls,ip,km) = rcls
          para(iarclt,ip,km) = rclt
          para(iacmpo,ip,km) = cmpo
          para(iacmps,ip,km) = cmps
          para(iacmpt,ip,km) = cmpt
ccc     enddo
      enddo


C-----------------------------------------
C-- wing and tail structural box parameters
      call getrval(lu,iline,parg(igwbox) )

      call getrval(lu,iline,parg(ighboxo))
      call getrval(lu,iline,parg(ighboxs))

      call getrval(lu,iline,parg(igrh) )

      call getrval(lu,iline,parg(igXaxis) )

      call getrval(lu,iline,parg(ighstrut))

C-----------------------------------------
C-- weight fractions of flight surfaces and secondary wing components, 
C     as fractions of dry total wing weight
      call getrval(lu,iline,parg(igfflap))
      call getrval(lu,iline,parg(igfslat))
      call getrval(lu,iline,parg(igfaile))
      call getrval(lu,iline,parg(igflete))
      call getrval(lu,iline,parg(igfribs))
      call getrval(lu,iline,parg(igfspoi))
      call getrval(lu,iline,parg(igfwatt))

C-----------------------------------------
C-- horizontal and vertical tail parameters
      call getival(lu,iline,pari(iiHTsize))
      call getrval(lu,iline,parg(igVh))
      call getrval(lu,iline,parg(igCLhCGfwd))

      call getival(lu,iline,pari(iiVTsize))
      call getrval(lu,iline,parg(igVv))
      call getrval(lu,iline,parg(igCLveout ))

      call getival(lu,iline,pari(iixwmove))
      call getrval(lu,iline,parg(igCLhspec))
      call getrval(lu,iline,parg(igSMmin))

cc    call getrval(lu,iline,parg(igdCLhdCL))
      call getrval(lu,iline,parg(igdepsda))

cc    call getrval(lu,iline,parg(igdCLndCL))
      call getrval(lu,iline,parg(igdCLnda))

      call getrval(lu,iline,parg(igARh))
      call getrval(lu,iline,parg(igARv))

      call getrval(lu,iline,parg(iglambdah))
      call getrval(lu,iline,parg(iglambdav))

      call getrval(lu,iline,parg(igsweeph))

      parg(igsweeph) = parg(igsweep)   ! ###

      call getrval(lu,iline,parg(igsweepv))

      call getrval(lu,iline,yoh)
      call getrval(lu,iline,yov)
      parg(igboh) = 2.0*yoh
      parg(igbov) = 2.0*yov

      call getrval(lu,iline,parg(igfCDhcen))

      call getrval(lu,iline,parg(igCLhmax))
      call getrval(lu,iline,parg(igCLvmax))

      call getrval(lu,iline,parg(igfhadd))
      call getrval(lu,iline,parg(igfvadd))

      call getrval(lu,iline,parg(igwboxh))
      call getrval(lu,iline,parg(igwboxv))
      call getrval(lu,iline,parg(ighboxh))
      call getrval(lu,iline,parg(ighboxv))
      call getrval(lu,iline,parg(igrhh) )
      call getrval(lu,iline,parg(igrhv) )

      call getrval(lu,iline,parg(ignvtail))

c-----------------------------------------
c---- cabin and fuselage geometry parameters
      call getrval(lu,iline,parg(igRfuse))
      call getrval(lu,iline,parg(igdRfuse))
      call getrval(lu,iline,parg(igwfb))

ccc      call getrval(lu,iline,parg(ignfweb))
      parg(ignfweb) = 1.0

      call getrval(lu,iline,parg(ighfloor))

      call getrval(lu,iline,parg(iganose))
      call getrval(lu,iline,parg(igbtail))

      call getrval(lu,iline,parg(igxnose))
      call getrval(lu,iline,parg(igxend))
      call getrval(lu,iline,parg(igxblend1))
      call getrval(lu,iline,parg(igxblend2))
      call getrval(lu,iline,parg(igxshell1))
      call getrval(lu,iline,parg(igxshell2))
      call getrval(lu,iline,parg(igxconend))
      call getrval(lu,iline,parg(igxwbox))
      call getrval(lu,iline,parg(igxhbox))
      call getrval(lu,iline,parg(igxvbox))

      call getrval(lu,iline,parg(igzwing))
      call getrval(lu,iline,parg(igzhtail))

      call getival(lu,iline,pari(iiengloc))

      call getrval(lu,iline,parg(igxeng))
      call getrval(lu,iline,parg(igyeng))
      call getrval(lu,iline,parg(igneng))

      call getrval(lu,iline,parg(iglambdac))

      call getrval(lu,iline,parg(igfstring))
      call getrval(lu,iline,parg(igfframe))
      call getrval(lu,iline,parg(igffadd))

      call getrval(lu,iline,parg(igWfix))
      call getrval(lu,iline,parg(igxfix))

      call getrval(lu,iline,parg(igWpwindow))
      call getrval(lu,iline,parg(igWppinsul))
      call getrval(lu,iline,parg(igWppfloor))

      call getrval(lu,iline,parg(igrMh))
      call getrval(lu,iline,parg(igrMv))

      call getival(lu,iline,pari(iifclose))

      call getrval(lu,iline,parg(igCMVf1))
      call getrval(lu,iline,parg(igCLMf0))

      call getrval(lu,iline,fduo)
      call getrval(lu,iline,fdus)
      call getrval(lu,iline,fdut)

      do km = 1, nmission
        do ip = 1, iptotal
          para(iafduo,ip,km) = fduo
          para(iafdus,ip,km) = fdus
          para(iafdut,ip,km) = fdut
        enddo
      enddo

C-----------------------------------------
C-- landing gear weight locations and weight fractions
      call getrval(lu,iline,parg(igxhpesys))
      call getrval(lu,iline,parg(igxlgnose))
      call getrval(lu,iline,parg(igdxlgmain))

      call getrval(lu,iline,parg(igfhpesys))
      call getrval(lu,iline,parg(igflgnose))
      call getrval(lu,iline,parg(igflgmain))

C-----------------------------------------
C-- other weight fractions
      call getrval(lu,iline,parg(igxapu))
      call getrval(lu,iline,parg(igfapu))

      call getrval(lu,iline,parg(igfseat))
      call getrval(lu,iline,parg(igfpadd))

      call getrval(lu,iline,parg(igfeadd))
      call getrval(lu,iline,parg(igfpylon))

      call getrval(lu,iline,parg(igfreserve))

C-----------------------------------------
C-- allowable stresses
      call getrval(lu,iline,parg(igsigfac)  )

      call getrval(lu,iline,parg(igsigskin) )
      call getrval(lu,iline,parg(igsigbend) )
      call getrval(lu,iline,parg(igsigcap)  )
      call getrval(lu,iline,parg(igtauweb)  )
      call getrval(lu,iline,parg(igsigstrut))

      call getrval(lu,iline,parg(igrEshell) )

      call getrval(lu,iline,parg(igEcap)  )
      call getrval(lu,iline,parg(igEstrut))

C-----------------------------------------
C-- structural material densities
      call getrval(lu,iline,parg(igrhoskin)  )
      call getrval(lu,iline,parg(igrhobend)  )
      call getrval(lu,iline,parg(igrhocap)   )
      call getrval(lu,iline,parg(igrhoweb)   )
      call getrval(lu,iline,parg(igrhostrut) )

C----------------------------------------
C---- drag build-up parameters
ccc      call getival(lu,iline,nairf)
      nairf = 1

c---- read and process airfoil file(s)
      do iairf = 1, nairf
c------ read airfoil database filename, and strip off any leading blanks
        call getcval(lu,iline,airfile(iairf))
        call strip(airfile(iairf),naf)

c------ set up airfoil database arays
        call airtable(airfile(iairf),
     &       iAdim,jAdim,kAdim,lAdim,
     &       nAMa(iairf),nAcl(iairf),nAtau(iairf),nAfun(iairf),
     &       AMa(1,iairf),Acl(1,iairf),Atau(1,iairf), ARe(iairf),
     &          A(1,1,1,1,iairf),
     &        A_M(1,1,1,1,iairf),
     &       A_cl(1,1,1,1,iairf),
     &      A_tau(1,1,1,1,iairf),
     &     A_M_cl(1,1,1,1,iairf),
     &    A_M_tau(1,1,1,1,iairf),
     &   A_cl_tau(1,1,1,1,iairf),
     & A_M_cl_tau(1,1,1,1,iairf) )

      enddo

      nairf = 2
      iairf = 2
      airfile(iairf) = airfile(1)
cc    airfile(iairf) = 'Cl.air'
      call airtable(airfile(iairf),
     &       iAdim,jAdim,kAdim,lAdim,
     &       nAMa(iairf),nAcl(iairf),nAtau(iairf),nAfun(iairf),
     &       AMa(1,iairf),Acl(1,iairf),Atau(1,iairf), ARe(iairf),
     &          A(1,1,1,1,iairf),
     &        A_M(1,1,1,1,iairf),
     &       A_cl(1,1,1,1,iairf),
     &      A_tau(1,1,1,1,iairf),
     &     A_M_cl(1,1,1,1,iairf),
     &    A_M_tau(1,1,1,1,iairf),
     &   A_cl_tau(1,1,1,1,iairf),
     & A_M_cl_tau(1,1,1,1,iairf) )


      call getrval(lu,iline,cdfw)   
      call getrval(lu,iline,cdpw)   
      call getrval(lu,iline,Rerefw)

      call getrval(lu,iline,cdft)   
      call getrval(lu,iline,cdpt)   
      call getrval(lu,iline,Rereft) 

      call getrval(lu,iline,cdfs)   
      call getrval(lu,iline,cdps)   
      call getrval(lu,iline,Rerefs)

      call getrval(lu,iline,aRexp)

      call getrval(lu,iline,fexcdw) 
      call getrval(lu,iline,fexcdt) 
      call getrval(lu,iline,fexcdf) 

      call getrval(lu,iline,parg(igfBLIw))
      call getrval(lu,iline,parg(igfBLIf))
      call getival(lu,iline,pari(iiBLIc))

c- - - - - - - - - - - - - - - - - - - - - - - -
      do km = 1, nmission
        do ip = 1, iptotal
          para(iacdfw  ,ip,km) = cdfw
          para(iacdpw  ,ip,km) = cdpw
          para(iaRerefw,ip,km) = Rerefw

          para(iacdft  ,ip,km) = cdft
          para(iacdpt  ,ip,km) = cdpt
          para(iaRereft,ip,km) = Rereft

          para(iacdfs  ,ip,km) = cdfs
          para(iacdps  ,ip,km) = cdps
          para(iaRerefs,ip,km) = Rerefs

          para(iaaRexp ,ip,km) = aRexp

          para(iafexcdw,ip,km) = fexcdw
          para(iafexcdt,ip,km) = fexcdt
          para(iafexcdf,ip,km) = fexcdf

c         para(iafexcdw,ip,km) = parm(imfexcdw,km)
c         para(iafexcdt,ip,km) = parm(imfexcdt,km)
c         para(iafexcdf,ip,km) = parm(imfexcdf,km)

        enddo

      enddo

c-----------------------------------------
c---- fuel parameters
      call getival(lu,iline,pari(iifuel))
      call getrval(lu,iline,rhofuel)
      call getrval(lu,iline,Tfuel)

      parg(igrhofuel) = rhofuel
      do km = 1, nmission
        do ip = 1, iptotal
          pare(ieTfuel,ip,km) = Tfuel
        enddo
      enddo

c----- heating values for checking hfuel from variable-cp gas routines
c      hfuels = 55.0e6  ! methane 
c      hfuels = 52.0e6  ! ethane  
c      hfuels = 49.0e6  ! propane 
c      hfuels = 46.0e6  ! butane  
c      hfuels = 44.0e6  ! octane  
c      hfuels = 43.0e6  ! kerosene  (C14H30)
c      hfuels = 43.0e6

c-----------------------------------------
c---- engine parameters
      call getrval(lu,iline,Tmetal)
      call getrval(lu,iline,Tt4TO )
      call getrval(lu,iline,parg(igfTt4CL1))
      call getrval(lu,iline,parg(igfTt4CLn))
      call getrval(lu,iline,Tt4CR )

      call getrval(lu,iline,dTstrk)
      call getrval(lu,iline,Mtexit)
      call getrval(lu,iline,StA   )
      call getrval(lu,iline,efilm )
      call getrval(lu,iline,tfilm )
      call getrval(lu,iline,M4a   )
      call getrval(lu,iline,ruc   )

      parg(igTmetal) = Tmetal
      do km = 1, nmission
        T0TO = parm(imT0TO,km)
        do ip = 1, iptotal
          pare(ieTt4,ip,km) = Tt4CR
          pare(ieM4a,ip,km) = M4a
          pare(ieruc,ip,km) = ruc
          pare(iedTstrk,ip,km) = dTstrk
          pare(ieMtexit,ip,km) = Mtexit
          pare(ieStA,ip,km) = StA
          pare(ieefilm,ip,km) = efilm
          pare(ietfilm,ip,km) = tfilm
        enddo

        pare(ieT0 ,ipstatic ,km) = T0TO
        pare(ieT0 ,iprotate ,km) = T0TO
        pare(ieT0 ,iptakeoff,km) = T0TO

        pare(ieTt4,ipstatic ,km) = Tt4TO
        pare(ieTt4,iprotate ,km) = Tt4TO
        pare(ieTt4,iptakeoff,km) = Tt4TO
      enddo


      call getrval(lu,iline,OPR  )
      call getrval(lu,iline,pilc )
      pihc = OPR/pilc

      call getrval(lu,iline,pif  )
      call getrval(lu,iline,pid  )
      call getrval(lu,iline,pib  )
      call getrval(lu,iline,pifn )
      call getrval(lu,iline,pitn )
      call getrval(lu,iline,epolf)
      call getrval(lu,iline,epollc)
      call getrval(lu,iline,epolhc)
      call getrval(lu,iline,epolht)
      call getrval(lu,iline,epollt)

      call getrval(lu,iline,etab)

      call getrval(lu,iline,pifK)
      call getrval(lu,iline,epfK)

      call getrval(lu,iline,BPR  )
      call getrval(lu,iline,Gearf)
      call getrval(lu,iline,HTRf )
      call getrval(lu,iline,HTRlc)
      call getrval(lu,iline,HTRhc)

      call getrval(lu,iline,M2   )
      call getrval(lu,iline,M25  )

      call getrval(lu,iline,epsl)
      call getrval(lu,iline,epsh)

      do km = 1, nmission
        do ip = 1, iptotal
          pare(iepid,ip,km) = pid
          pare(iepib,ip,km) = pib
          pare(iepifn,ip,km) = pifn
          pare(iepitn,ip,km) = pitn
          pare(iepif,ip,km) = pif
          pare(iepilc,ip,km) = pilc
          pare(iepihc,ip,km) = pihc
          pare(ieepolf,ip,km) = epolf
          pare(ieepollc,ip,km) = epollc
          pare(ieepolhc,ip,km) = epolhc
          pare(ieepolht,ip,km) = epolht
          pare(ieepollt,ip,km) = epollt
          pare(ieetab,ip,km) = etab
          pare(iepifK,ip,km) = pifK
          pare(ieepfK,ip,km) = epfK
          pare(ieBPR,ip,km) = BPR
          pare(ieM2 ,ip,km) = M2
          pare(ieM25,ip,km) = M25
          pare(ieepsl,ip,km) = epsl
          pare(ieepsh,ip,km) = epsh
        enddo
      enddo

      call getrval(lu,iline,mofftpax)
      call getrval(lu,iline,mofftmMTO)
      call getrval(lu,iline,Pofftpax)
      call getrval(lu,iline,PofftmMTO)

      parg(igmofWpay) = mofftpax  / Wpax(1)
      parg(igmofWMTO) = mofftmMTO / gee
      parg(igPofWpay) = Pofftpax  / Wpax(1)
      parg(igPofWMTO) = PofftmMTO / gee


      call getrval(lu,iline,Tt9)
      call getrval(lu,iline,pt9)

      do km = 1, nmission
        do ip = 1, iptotal
          pare(ieTt9,ip,km) = Tt9
          pare(iept9,ip,km) = pt9
        enddo
      enddo

      call getrval(lu,iline,A7static  )
      call getrval(lu,iline,A7takeoff )
      call getrval(lu,iline,A7cutback )
      call getrval(lu,iline,A7climb1  )
      call getrval(lu,iline,A7climbn  )
      call getrval(lu,iline,A7descent1)
      call getrval(lu,iline,A7descentn)

      call getrval(lu,iline,A5static  )
      call getrval(lu,iline,A5takeoff )
      call getrval(lu,iline,A5cutback )
      call getrval(lu,iline,A5climb1  )
      call getrval(lu,iline,A5climbn  )
      call getrval(lu,iline,A5descent1)
      call getrval(lu,iline,A5descentn)

      do km = 1, nmission
        pare(ieA7fac,ipstatic ,km) = A7static
        pare(ieA7fac,iprotate ,km) = A7takeoff
        pare(ieA7fac,iptakeoff,km) = A7takeoff
        pare(ieA7fac,ipcutback,km) = A7cutback

        pare(ieA5fac,ipstatic ,km) = A5static
        pare(ieA5fac,iprotate ,km) = A5takeoff
        pare(ieA5fac,iptakeoff,km) = A5takeoff
        pare(ieA5fac,ipcutback,km) = A5cutback

        do ip = ipclimb1, ipclimbn
          frac = float(ip      -ipclimb1)
     &         / float(ipclimbn-ipclimb1)
          pare(ieA7fac,ip,km) = A7climb1*(1.0-frac) + A7climbn*frac
          pare(ieA5fac,ip,km) = A5climb1*(1.0-frac) + A5climbn*frac
        enddo
        do ip = ipcruise1, ipcruisen
          pare(ieA7fac,ip,km) = 1.0
          pare(ieA5fac,ip,km) = 1.0
        enddo
        do ip = ipdescent1, ipdescentn
          frac = float(ip        -ipdescent1)
     &         / float(ipdescentn-ipdescent1)
          pare(ieA7fac,ip,km) = A7descent1*(1.0-frac) + A7descentn*frac
          pare(ieA5fac,ip,km) = A5descent1*(1.0-frac) + A5descentn*frac
        enddo

        pare(ieA7fac,iptest,km) = A7static
        pare(ieA5fac,iptest,km) = A5static
      enddo

      parg(igGearf) = Gearf
      parg(igHTRf ) = HTRf
      parg(igHTRlc) = HTRlc
      parg(igHTRhc) = HTRhc

      call getrval(lu,iline,parg(igrSnace))
      call getrval(lu,iline,parg(igrVnace))

      call getival(lu,iline,pari(iiengwgt))

      return

c---------------------------------------------------------------
 800  continue
      write(*,8010) 'READ error on line', iline, line(1:60)
 8010 format(1x,a,i5,':  ', a)
      stop

 900  continue
      write(*,*) 'Unexpected end-of-file'
      stop

      end ! getparm
