c
c    Routines for reading logical, integer, real, or character data
c    from a formatted file.
c
c    Each routine tries to read from logical unit "lu"
c    the next non-blank, non-comment data line.
c    A comment line starts with   #  or  %  or  !  .
c    All data after "!" is ignored, except in string data.
c
c    Each routine also increments the counter "line" 
c    for each file line it tries to read.
c    If GETIVAL or GETRVAL has a read error, it will print 
c    the line number and the line which caused the error.
c
c
c
c    Usage:
c
c     lu = 1     ! or any positive integer except 5 or 6
c     open(lu,file='filename',status='old')
c     iline = 0
c     call getival(lu,iline,ival1)
c     call getival(lu,iline,ival2)
c     call getival(lu,iline,ival3)
c     call getrval(lu,iline,rval1)
c     call getrval(lu,iline,rval2)
c     call getival(lu,iline,ival4)
c       .
c       .
c     close(lu)
c


      subroutine getLval(lu,iline,Lval)
      integer lu, iline
      logical Lval

      character*128 line

 1000 format(a)
 10   read(lu,1000,end=800) line
      iline = iline + 1
      if(line.eq.' ') go to 10
      if(index('#%!',line(1:1)) .ne. 0) go to 10

c      write(*,*) line(1:60)

c      kpond = index(line,'#')
c      kpcnt = index(line,'%')
      kbang = index(line,'!')

      kend = len(line)
c      if(kpond.ne.0) kend = min(kpond-1,kend)
c      if(kpcnt.ne.0) kend = min(kpcnt-1,kend)
      if(kbang.ne.0) kend = min(kbang-1,kend)
      if(kend.le.0) go to 10
      if(line(1:kend).eq.' ') go to 10

      read(line(1:kend),*,err=900) Lval
      return

 800  continue
      write(*,*) 'GETLVAL: Unexpected end-of-file'
      stop

 900  continue
      write(*,2000) 'GETLVAL: Read error on line', iline, line
 2000 format(1x,a,i6,':  ',a)
      stop

      end ! getLval



      subroutine getival(lu,iline,ival)
      integer lu, iline
      integer ival

      character*128 line

 1000 format(a)
 10   read(lu,1000,end=800) line
      iline = iline + 1
      if(line.eq.' ') go to 10
      if(index('#%!',line(1:1)) .ne. 0) go to 10

c      write(*,*) line(1:60)

c      kpond = index(line,'#')
c      kpcnt = index(line,'%')
      kbang = index(line,'!')

      kend = len(line)
c      if(kpond.ne.0) kend = min(kpond-1,kend)
c      if(kpcnt.ne.0) kend = min(kpcnt-1,kend)
      if(kbang.ne.0) kend = min(kbang-1,kend)
      if(kend.le.0) go to 10
      if(line(1:kend).eq.' ') go to 10

      read(line(1:kend),*,err=900) ival
      return

 800  continue
      write(*,*) 'GETIVAL: Unexpected end-of-file'
      stop

 900  continue
      write(*,2000) 'GETIVAL: Read error on line', iline, line
 2000 format(1x,a,i6,':  ',a)
      stop

      end ! getival




      subroutine getrval(lu,iline,rval)
      integer lu, iline
      real rval

      character*128 line

 1000 format(a)
 10   read(lu,1000,end=800) line
      iline = iline + 1

      if(line.eq.' ') go to 10
      if(index('#%!',line(1:1)) .ne. 0) go to 10

c      write(*,*) line(1:60)

c      kpond = index(line,'#')
c      kpcnt = index(line,'%')
      kbang = index(line,'!')

      kend = len(line)
c      if(kpond.ne.0) kend = min(kpond-1,kend)
c      if(kpcnt.ne.0) kend = min(kpcnt-1,kend)
      if(kbang.ne.0) kend = min(kbang-1,kend)
      if(kend.le.0) go to 10
      if(line(1:kend).eq.' ') go to 10

      k1 = 1
      k2 = kend

      fac = 1.0
      kmul = index(line(k1:k2),'*')
      kdiv = index(line(k1:k2),'/')

      if    (kmul.ne.0) then
       k3 = k1 + kmul
       k4 = k2
       k2 = k1 + kmul - 2
       read(line(k3:k4),*,err=900) fac
      elseif(kdiv.ne.0) then
       k3 = k1 + kdiv
       k4 = k2
       k2 = K1 + kdiv - 2
       read(line(k3:k4),*,err=900) faci
       fac = 1.0/faci
      endif
c
      read(line(k1:k2),*,err=900) rval
      rval = rval * fac
      return

 800  continue
      write(*,*) 'GETRVAL: Unexpected end-of-file'
      stop

 900  continue
      write(*,2000) 'GETRVAL: Read error on line', iline, line
 2000 format(1x,a,i6,':  ',a)
      stop

      end ! getrval




      subroutine getcval(lu,iline,cval)
      integer lu, iline
      character*(*) cval

      character*128 line

 1000 format(a)
 10   read(lu,1000,end=800) line
      iline = iline + 1
      if(line.eq.' ') go to 10
      if(index('#%!',line(1:1)) .ne. 0) go to 10

c      write(*,*) line(1:60)

c      kpond = index(line,'#')
c      kpcnt = index(line,'%')
      kbang = index(line,'!')

      kend = len(line)
c      if(kpond.ne.0) kend = min(kpond-1,kend)
c      if(kpcnt.ne.0) kend = min(kpcnt-1,kend)
      if(kbang.ne.0) kend = min(kbang-1,kend)
      if(kend.le.0) go to 10
      if(line(1:kend).eq.' ') go to 10

      cval = line(1:kend)
      return

 800  continue
      write(*,*) 'GETCVAL: Unexpected end-of-file'
      stop
      end ! getcval



      subroutine getrkey(line,key,val,nval)
      character*(*) line
      character*(*) key
      real val(*)
      logical error
c--------------------------------------------------------------------------------
c     If "key" is a blank string, then up to "nval" numbers
c     are read from string "line" and returned in val(.) array.
c
c     If "key" is not a blank string, then the "nval" numbers 
c     will be read only if the "key" string is also present
c     at the beginning of "line", with the numbers following.
c
c     If "key" is not a blank string and there's no string match,
c     then the returned "nval" indicates action taken.
c
c     If "line" contains a "*" character, the number after it 
c     is used as a multiplier on all the preceding numbers.
c
c     If "line" contains a "/" character, the number after it 
c     is used as a divisor on all the preceding numbers.
c
c     Input:
c      line   character string containing keyword, data, multiplier/divisor
c      key    character string containing keyword
c      nval   max number of real numbers to be read
c
c     Output:
c      val(.)      real numbers read
c      nval = -2   "key" string not present in "line"
c      nval = -1   "key" present, but read error on val(.)
c      nval =  0   "key" present, no val(.) numbers present
c      nval >  0   "key" present, val(1:nval) read in 
c--------------------------------------------------------------------------------

c---- find position of last character before any of the comment delimiters
      kpond = index(line,'#')
      kpcnt = index(line,'%')
      kbang = index(line,'!')

      kend = len(line)
      if(kpond .ne. 0) kend = min(kpond-1,kend)
      if(kpcnt .ne. 0) kend = min(kpcnt-1,kend)
      if(kbang .ne. 0) kend = min(kbang-1,kend)
      if(kend .le. 0) go to 900

c---- if line starts with a number, then knum will be > 0
      knum = index('0123456789-+.',line(1:1))

c---- length of keyword string, including trailing blank
      nkey = index(key,' ')

cc      write(*,*) key

      if(knum .ne. 0  .or.  nkey .eq. 1) then
c----- no keyword string in "line", or blank "key"... just go read numbers
       k1 = 1
       k2 = kend
       go to 10

      elseif(nkey .lt. 1) then
       write(*,*) '? Bad keyword string passed in: ', key
       stop

      endif

c---- find location of keyword string in data line
      kkey = index(line(1:kend),key(1:nkey))
      if(kkey .eq. 0) then
c----- keyword is not in data line... no data returned
       nval = -2
       return
      endif

c---- data to be read is in line(k1:k2)
      k1 = kkey + nkey
      k2 = kend

 10   continue
      if(k2 .lt. k1) then
c----- no data present
       nval = 0
       return
      endif

c---- first see if there's a multiplier or divisor at the end of the data line
      kmul = index(line(k1:k2),'*')
      kdiv = index(line(k1:k2),'/')
      if    (kmul .ne. 0) then
       k3 = k1 + kmul
       k4 = k2
       k2 = k1 + kmul - 2
       read(line(k3:k4),*,err=900) fac

      elseif(kdiv .ne. 0) then
       k3 = K1 + kdiv
       k4 = k2
       k2 = k1 + kdiv - 2
       read(line(k3:k4),*,err=900) faci
       fac = 1.0/faci

      else
       fac = 1.0

      endif
c
c---- read up to nval data values, if any
      call getflt(line(k1:k2),val,nval,error)
      if(error) go to 900

      if(fac .ne. 1.0) then
c----- apply multiplier or divisor to all data values
       do i = 1, nval
         val(i) = val(i) * fac
       enddo
      endif

      return

 900  continue
      nval = -1
      return

      end ! getrkey
 




      SUBROUTINE GETINT(INPUT,A,N,ERROR)
      CHARACTER*(*) INPUT
      INTEGER A(*)
      LOGICAL ERROR
C----------------------------------------------------------
C     Parses character string INPUT into an array
C     of integer numbers returned in A(1...N)
C
C     Will attempt to extract no more than N numbers, 
C     unless N = 0, in which case all numbers present
C     in INPUT will be extracted.
C
C     N returns how many numbers were actually extracted.
C----------------------------------------------------------
      CHARACTER*130 REC
      CHARACTER*1 TAB
C
      TAB = CHAR(9)
C
C---- only first 128 characters in INPUT will be parsed
      ILEN = MIN( LEN(INPUT) , 128 )
      ILENP = ILEN + 2
C
C---- put input into local work string (which will be munched)
      REC(1:ILENP) = INPUT(1:ILEN) // ' ,'
C
C---- ignore everything after a "!" character
      K = INDEX(REC,'!')
      IF(K.GT.0) REC(1:ILEN) = REC(1:K-1)
C
C---- change tabs to spaces
 5    K = INDEX(REC(1:ILEN),TAB)
      IF(K.GT.0) THEN
       REC(K:K) = ' '
       GO TO 5
      ENDIF
C
      NINP = N
C
C---- count up how many numbers are to be extracted
      N = 0
      K = 1
      DO 10 IPASS=1, ILEN
C------ search for next space or comma starting with current index K
        KSPACE = INDEX(REC(K:ILENP),' ') + K - 1
        KCOMMA = INDEX(REC(K:ILENP),',') + K - 1
C
        IF(K.EQ.KSPACE) THEN
C------- just skip this space
         K = K+1
         GO TO 9
        ENDIF
C
        IF(K.EQ.KCOMMA) THEN
C------- comma found.. increment number count and keep looking
         N = N+1
         K = K+1
         GO TO 9
        ENDIF
C
C------ neither space nor comma found, so we ran into a number...
C-    ...increment number counter and keep looking after next space or comma
        N = N+1
        K = MIN(KSPACE,KCOMMA) + 1
C
  9     IF(K.GE.ILEN) GO TO 11
 10   CONTINUE
C
C---- decide on how many numbers to read, and go ahead and read them
 11   IF(NINP.GT.0) N = MIN( N, NINP )
      READ(REC(1:ILEN),*,ERR=20) (A(I),I=1,N)
      ERROR = .FALSE.
      RETURN
C
C---- bzzzt !!!
 20   CONTINUE
ccc   WRITE(*,*) 'GETINT: String-to-integer conversion error.'
      N = 0
      ERROR = .TRUE.
      RETURN
      END ! GETINT


      SUBROUTINE GETFLT(INPUT,A,N,ERROR)
      CHARACTER*(*) INPUT
      REAL A(*)
      LOGICAL ERROR
C----------------------------------------------------------
C     Parses character string INPUT into an array
C     of real numbers returned in A(1...N)
C
C     Will attempt to extract no more than N numbers, 
C     unless N = 0, in which case all numbers present
C     in INPUT will be extracted.
C
C     N returns how many numbers were actually extracted.
C----------------------------------------------------------
      CHARACTER*130 REC
      CHARACTER*1 TAB
C
      TAB = CHAR(9)
C
C---- only first 128 characters in INPUT will be parsed
      ILEN = MIN( LEN(INPUT) , 128 )
      ILENP = ILEN + 2
C
C---- put input into local work string (which will be munched)
      REC(1:ILENP) = INPUT(1:ILEN) // ' ,'
C
C---- ignore everything after a "!" character
      K = INDEX(REC,'!')
      IF(K.GT.0) REC(1:ILEN) = REC(1:K-1)
C
C---- change tabs to spaces
 5    K = INDEX(REC(1:ILEN),TAB)
      IF(K.GT.0) THEN
       REC(K:K) = ' '
       GO TO 5
      ENDIF
C
      NINP = N
C
C---- count up how many numbers are to be extracted
      N = 0
      K = 1
      DO 10 IPASS=1, ILEN
C------ search for next space or comma starting with current index K
        KSPACE = INDEX(REC(K:ILENP),' ') + K - 1
        KCOMMA = INDEX(REC(K:ILENP),',') + K - 1
C
        IF(K.EQ.KSPACE) THEN
C------- just skip this space
         K = K+1
         GO TO 9
        ENDIF
C
        IF(K.EQ.KCOMMA) THEN
C------- comma found.. increment number count and keep looking
         N = N+1
         K = K+1
         GO TO 9
        ENDIF
C
C------ neither space nor comma found, so we ran into a number...
C-    ...increment number counter and keep looking after next space or comma
        N = N+1
        K = MIN(KSPACE,KCOMMA) + 1
C
  9     IF(K.GE.ILEN) GO TO 11
 10   CONTINUE
C
C---- decide on how many numbers to read, and go ahead and read them
 11   IF(NINP.GT.0) N = MIN( N, NINP )
      READ(REC(1:ILEN),*,ERR=20) (A(I),I=1,N)
      ERROR = .FALSE.
      RETURN
C
C---- bzzzt !!!
 20   CONTINUE
ccc   WRITE(*,*) 'GETFLT: String-to-integer conversion error.'
      N = 0
      ERROR = .TRUE.
      RETURN
      END ! GETFLT



      SUBROUTINE STRIP(STRING,NS)
      CHARACTER*(*) STRING
C----------------------------------------------------
C     Strips leading blanks off STRING and returns 
C     length NS of non-blank part.
C----------------------------------------------------
      NLEN = LEN(STRING)
C
C---- find last non-blank character
      DO K2 = NLEN, 1, -1
        IF(STRING(K2:K2).NE.' ') GO TO 11
      ENDDO
      K2 = 0
   11 CONTINUE
C
C---- find first non-blank character
      DO K1 = 1, K2
        IF(STRING(K1:K1).NE.' ') GO TO 21
      ENDDO
   21 CONTINUE
C
C---- number of non-blank characters
      NS = K2 - K1 + 1
      IF(NS.EQ.0) RETURN
C
C---- shift STRING so first character is non-blank
      STRING(1:NS) = STRING(K1:K2)
C
C---- pad tail of STRING with blanks
      DO K = NS+1, NLEN
        STRING(K:K) = ' '
      ENDDO
C
      RETURN
      END





      SUBROUTINE BSTRIP(STRING,NS)
      CHARACTER*(*) STRING
C--------------------------------------------------
C     Strips all blanks from STRING and returns 
C     length NS of non-blank part.
C     If STRING is all blanks, just returns NS=0
C--------------------------------------------------
C
C---- first remove any leading blanks and get length to be processed
      CALL STRIP(STRING,NS)
C
C---- pass over STRING and strip out all interior blanks
      K = 1
C
 10   CONTINUE
      IF(K.GE.NS) THEN
       RETURN
C
      ELSEIF(STRING(K:K) .EQ. ' ') THEN
       STRING(K:NS-1) = STRING(K+1:NS)
       NS = NS - 1
C
      ELSE
       K = K + 1
C
      ENDIF
C
      GO TO 10
C
      END




      SUBROUTINE GETARG0(IARG,ARG)
C------------------------------------------------
C     Same as GETARG, but...
C
C     ...in the case of Intel Fortran, this one
C     doesn't barf if there's no Unix argument 
C      (just returns blank string instead)
C------------------------------------------------
      CHARACTER*(*) ARG
C
      NARG = IARGC()
      IF(NARG.GE.IARG) THEN
       CALL GETARG(IARG,ARG)
      ELSE
       ARG = ' '
      ENDIF
C
      RETURN
      END ! GETARG0


      
