
      subroutine gppre(X,a,xs,N,D,P,sf,fs)
c==============================================================
c     Calculates GP prediction using squared exponential covariance
c     kernal with Automatic Relevance Determination (ARD)
c     
c   Inputs:
c     N     number of training points
c     D     number of design variables
c     X     training points (N x D)
c     xs    test point (D x 1)
c     P     vector of length scales for covariance kernel (D x 1)
c     sf    signal variance
c     a     coefficients for GP computed offline
c
c   Output:
c     fs    prediction output of test point
c
c==============================================================

      implicit real (a-h,m,o-z)
      integer D
      real X(N,D), P(D), xs(D), a(N)
      real fs

c---- Local Arrays
      real alpha(N), kstar(N)
      real temp


c---- Calculate kstar by looping over X and xs
      do i=1,N
            temp = 0.
            do k=1,D
                temp = temp + 1/P(k)*(X(i,k) - xs(k))**2
            enddo
            kstar(i) = sf**2*exp(-0.5*temp)
      enddo

c---- Do kstar'*a to get fs
      fs = 0
      do i=1,N
        fs = fs + kstar(i)*a(i)
      enddo

      return
      end ! gppre
