
      subroutine gradop(ndim,n, vopt,dvopt,fopt,
     &                  ftol,fdel,nstepmax)
c----------------------------------------------------------------------
C   Multidimensional minimization routine of the function and gradient
C   supplied by user in GOBJ(n,x,f,g), where x is an n-long vector,
C   f is the functional value at x, and g is its gradient.
C   That is, f(x), and g(x) = df/dx
C
C Input:
C   ndim      Physical first dimension of simp(..) array
C   n         Number of degrees of freedom (size of simp(.*))
C   vopt(.)   Initial variable vector
C   dvopt(.)  Typical variable change vector, used for scaling purposes
C   ftol      convergence tolerance to be achieved in the function value
C   nstepmax  max number of optimization steps gradop is allowed to make
C
C Output:
C   vopt(.)   Modified variable vector
C   fopt      Corresponding objective function value
C
C   This routine calls subroutine gobj, 
C   which returns the objective function and gradient values.
C
C
C   Uses the BFGS method.
C 
c----------------------------------------------------------------------
      implicit real(a-h,l-z)
c----------------------------------------------------------------------
c---- passed variables
      integer ndim, n, nstepmax
      real vopt(ndim), dvopt(ndim)
      real fopt, ftol, fdel

c----------------------------------------------------------------------
c---- local variables
      integer nvmax, ncmax
      parameter (
     &   nvmax = 25,
     &   ncmax = 10 )
      real funv(nvmax), con(ncmax), conv(ncmax,nvmax)

      integer i,j,np1
      integer istep
      integer ncon

      real dfrel
      real temp, fun

      if(n .gt. nvmax) then
       write(*,*) 'GRADOP: Local array overflow. Increase nvmax to', n
       stop
      endif
c
c----------------------------------------------------------------------

      call gobj(nvopt,vopt,dvopt,
     &          fun,funv, ncmax,ncon,con,conv)

      func = fun
      

      do 100 istep = 1, nstepmax

        grms = 0.
        gmax = 0.
        do i = 1, n
          grel = funv(i)*dvopt(i)
          grms = grms + grel**2
        enddo

 100  continue  ! with next step

      dfrel = fdel/ftol
      write(*,'(1x,a,g12.4)')
     &   'GRADOP: Exiting on step limit.  dFrel =', dfrel

 998  continue

c---- return minimmum-function vector in vopt(.)

c---- final minimum function value
      call fobj(n,vopt,
     &          ncon,con, fun,
     &          func,dfrel,'!',istep)
c      write(*,*) 'Nstep =', istep

      return
      end ! gradop

