
      program maptest
c----------------------------------------------------------------------
c     Checks the derivatives returned by the various map* routines
c     by comparing them against finite-differenced values.
c----------------------------------------------------------------------
      implicit real (a-z)

      integer ipass

      character*2 ss

c---- perturbation value for finite-differencing
      data eps / 1.0e-7 /

      include 'tfmap.inc'


      piD = 1.8
      mbD = 3.0
      NbD = 0.95

      pi = 0.80 * piD
      Nb = 0.95 * NbD
      mb = 1.05 * mbD

      effo = 0.9

      piK = 1.5
      effK = -0.1

      call Ncmap(pi,mb, piD,mbD,NbD, Cmapl,
     &           Nb, Nb_pi, Nb_mb )


      pi$ = pi
      mb$ = mb
      Nb$ = Nb

 3100 format(2(/1x,a,g16.8,2x,a))

c--------------------------------------------------------------------


      pi$ = pi + eps
      call Ncmap(pi$,mb$, piD,mbD,NbD, Cmapl,
     &           Nb$, Nb_pi$, Nb_mb$ )
      pi$ = pi
      dd = (Nb$ - Nb)/eps
      aa = (Nb_pi$ + Nb_pi)*0.5
      call compare(ss,aa,dd)
      write(*,3100) 'N_pi',aa,ss,'    ',dd,ss


      mb$ = mb + eps
      call Ncmap(pi$,mb$, piD,mbD,NbD, Cmapl,
     &           Nb$, Nb_pi$, Nb_mb$ )
      mb$ = mb
      dd = (Nb$ - Nb)/eps
      aa = (Nb_mb$ + Nb_mb)*0.5
      call compare(ss,aa,dd)
      write(*,3100) 'N_m ',aa,ss,'    ',dd,ss


c--------------------------------------------------------------------

      call ecmap(pi,mb, piD,mbD, Cmapl, effo, piK, effK, 
     &            eff, eff_pi, eff_mb )

      pi$ = pi + eps
      call ecmap(pi$,mb$, piD,mbD, Cmapl, effo, piK, effK, 
     &            eff$, eff_pi$, eff_mb$ )
      pi$ = pi
      dd = (eff$ - eff)/eps
      aa = (eff_pi$ + eff_pi)*0.5
      call compare(ss,aa,dd)
      write(*,3100) 'e_pi',aa,ss,'    ',dd,ss


      mb$ = mb + eps
      call ecmap(pi$,mb$, piD,mbD, Cmapl, effo, piK, effK, 
     &            eff$, eff_pi$, eff_mb$ )
      mb$ = mb
      dd = (eff$ - eff)/eps
      aa = (eff_mb$ + eff_mb)*0.5
      call compare(ss,aa,dd)
      write(*,3100) 'e_m ',aa,ss,'    ',dd,ss

      write(*,*)
      write(*,*)


c--------------------------------------------------------------------

      piD = 1.8
      mbD = 3.0
      NbD = 0.95
      ept0 = 0.9

      pref = 1.0e5
      Tref = 288.0

      Tt = 2000.0
      cpt = 1230.0
      Rt  = 287.0
      dh = cpt*Tt*(1.0 - 0.65)
      Nb = 0.95 * NbD
      mb = 1.05 * mbD

      Tt$  = Tt 
      cpt$ = cpt
      Rt$  = Rt 
      dh$  = dh 
      Nb$  = Nb  
      mb$  = mb  

      dTt  = Tt * eps
      dcpt = cpt* eps
      dRt  = Rt * eps
      ddh  = dh * eps
      dNb  = Nb * eps
      dmb  = mb * eps

      call etmap(dh,mb,Nb, piD,mbD,NbD,ept0,Tmaph,
     &           Tt,cpt,Rt,
     &           ept,
     &           ept_dh,ept_mb,ept_Nb,
     &           ept_Tt,ept_cpt,ept_Rt )

      dh$ = dh + ddh
      call etmap(dh$,mb$,Nb$, piD,mbD,NbD,ept0,Tmaph,
     &           Tt$,cpt$,Rt$,
     &           ept$,
     &           ept_dh$,ept_mb$,ept_Nb$,
     &           ept_Tt$,ept_cpt$,ept_Rt$ )
      dh$ = dh
      dd = (ept$ - ept)/ddh
      aa = (ept_dh$ + ept_dh)*0.5
      call compare(ss,aa,dd)
      write(*,3100) 'et_dh',aa,ss,'     ',dd,ss


      mb$ = mb + dmb
      call etmap(dh$,mb$,Nb$, piD,mbD,NbD,ept0,Tmaph,
     &           Tt$,cpt$,Rt$,
     &           ept$,
     &           ept_dh$,ept_mb$,ept_Nb$,
     &           ept_Tt$,ept_cpt$,ept_Rt$ )
      mb$ = mb
      dd = (ept$ - ept)/dmb
      aa = (ept_mb$ + ept_mb)*0.5
      call compare(ss,aa,dd)
      write(*,3100) 'et_mb',aa,ss,'     ',dd,ss

 
      Nb$ = Nb + dNb
      call etmap(dh$,mb$,Nb$, piD,mbD,NbD,ept0,Tmaph,
     &           Tt$,cpt$,Rt$,
     &           ept$,
     &           ept_dh$,ept_mb$,ept_Nb$,
     &           ept_Tt$,ept_cpt$,ept_Rt$ )
      Nb$ = Nb
      dd = (ept$ - ept)/dNb
      aa = (ept_Nb$ + ept_Nb)*0.5
      call compare(ss,aa,dd)
      write(*,3100) 'et_Nb',aa,ss,'     ',dd,ss
 
      Tt$ = Tt + dTt
      call etmap(dh$,mb$,Nb$, piD,mbD,NbD,ept0,Tmaph,
     &           Tt$,cpt$,Rt$,
     &           ept$,
     &           ept_dh$,ept_mb$,ept_Nb$,
     &           ept_Tt$,ept_cpt$,ept_Rt$ )
      Tt$ = Tt
      dd = (ept$ - ept)/dTt
      aa = (ept_Tt$ + ept_Tt)*0.5
      call compare(ss,aa,dd)
      write(*,3100) 'et_Tt',aa,ss,'     ',dd,ss


      cpt$ = cpt + dcpt
      call etmap(dh$,mb$,Nb$, piD,mbD,NbD,ept0,Tmaph,
     &           Tt$,cpt$,Rt$,
     &           ept$,
     &           ept_dh$,ept_mb$,ept_Nb$,
     &           ept_Tt$,ept_cpt$,ept_Rt$ )
      cpt$ = cpt
      dd = (ept$ - ept)/dcpt
      aa = (ept_cpt$ + ept_cpt)*0.5
      call compare(ss,aa,dd)
      write(*,3100) 'et_cpt',aa,ss,'      ',dd,ss

 
      Rt$ = Rt + dRt
      call etmap(dh$,mb$,Nb$, piD,mbD,NbD,ept0,Tmaph,
     &           Tt$,cpt$,Rt$,
     &           ept$,
     &           ept_dh$,ept_mb$,ept_Nb$,
     &           ept_Tt$,ept_cpt$,ept_Rt$ )
      Rt$ = Rt
      dd = (ept$ - ept)/dRt
      aa = (ept_Rt$ + ept_Rt)*0.5
      call compare(ss,aa,dd)
      write(*,3100) 'et_Rt',aa,ss,'     ',dd,ss


      write(*,*)
      write(*,*)


c--------------------------------------------------------------------

      call Pimap(mb,Nb, piD,mbD,NbD, Cmapl,
     &           pi, pi_mb, pi_Nb )


      mb$ = mb + dmb
      call Pimap(mb$,Nb$, piD,mbD,NbD, Cmapl,
     &           pi$, pi_mb$, pi_Nb$ )
      mb$ = mb
      dd = (pi$ - pi)/dmb
      aa = (pi_mb$ + pi_mb)*0.5

      write(*,*) '-------------'
      write(*,*) pi, pi_mb
      write(*,*) pi$, pi_mb$
      write(*,*) '-------------'

      call compare(ss,aa,dd)
      write(*,3100) 'pi_mb',aa,ss,'     ',dd,ss

 
      Nb$ = Nb + dNb
      call Pimap(mb$,Nb$, piD,mbD,NbD, Cmapl,
     &           pi$, pi_mb$, pi_Nb$ )
      Nb$ = Nb
      dd = (pi$ - pi)/dNb
      aa = (pi_Nb$ + pi_Nb)*0.5
      call compare(ss,aa,dd)
      write(*,3100) 'pi_Nb',aa,ss,'     ',dd,ss


      stop
      end
