
      subroutine noise(pari,parg,parm,para,pare,
     &                 initeng,iairf,ichoke5,ichoke7)
c===========================================================
c     Calculates sideline and overhead (cutback) noise
c
c     Note:  This routine is for noise estimates only,
c            and is not intended to replace ANOPP etc.
c
c===========================================================
      implicit real (a-h,l-z)

      include 'index.inc'
      integer pari(iitotal)
      real parg(igtotal),
     &     parm(imtotal),
     &     para(iatotal,iptotal),
     &     pare(ietotal,iptotal)
      integer ifuel, initeng
      integer ichoke5(iptotal),
     &        ichoke7(iptotal)


      include 'airf.inc'
      include 'constants.inc'

      integer type, method, inite


      inprint = 0
c      inprint = 1

      type = 0    ! unmixed exhaust
c     type = 1    ! mixed   exhaust

c     method = 1  ! ESDU method
      method = 2  ! Heidmann's method (part of ANOPP)
c     method = 3  ! Heidmann's method with Allied Signal modifications 


      iengloc = pari(iiengloc)
      ifclose = pari(iifclose)
      ifuel   = pari(iifuel)
      iBLIc   = pari(iiBLIc)  

      W    = parm(imWTO)

      neng = parg(igneng)
      htr  = parg(igHTRf)
      dfan = parg(igdfan)

      rss  = 0.2*dfan * 100.0   ! Rotor Stator Spacing  [mm]
      B = 20.0    ! Number rotor blades
      V = 44.0    ! Number stator blades

c-------------------------------------------------------------------------------

c---- set takeoff operation at full power, V2 (initial climbout)
      ip = iptakeoff
      icdfun = 0
      call cdsum(pari,parg,para(1,ip),pare(1,ip), icdfun,iairf)

cc      write(*,*) 'takeoff...'

c---- initialize with rotation engine state
      ipc = iprotate
      pare(iembf ,ip) = pare(iembf ,ipc)
      pare(iemblc,ip) = pare(iemblc,ipc)
      pare(iembhc,ip) = pare(iembhc,ipc)
      pare(iepif ,ip) = pare(iepif ,ipc)
      pare(iepilc,ip) = pare(iepilc,ipc)
      pare(iepihc,ip) = pare(iepihc,ipc)
      pare(iept5 ,ip) = pare(iept5 ,ipc)

      icall = 1
      icool = 1
      call tfcalc(pari,parg,para(1,ip),pare(1,ip), ip,
     &            icall,icool,initeng,
     &            ichoke5(ip),ichoke7(ip))

      F   = pare(ieFe,ip)*neng
      DoL = para(iaCD,ip)/para(iaCL,ip)
      phi = F/W
      sing = (phi - DoL*sqrt(1.0 - phi**2 + DoL**2)) / (1.0 + DoL**2)
      cosg = sqrt(1.0 - sing**2)
      para(iagamV,ip) = atan2(sing,cosg)

c-----------------------------------------------------------------
c---- set cutback point operation
      ip = ipcutback
      icdfun = 0
      call cdsum(pari,parg,para(1,ip),pare(1,ip), icdfun,iairf)

c---- set thrust required for prescribed cutback climb angle
      DoL = para(iaCD,ip)/para(iaCL,ip)
      gamVCB = parm(imgamVCB)
      singCB = sin(gamVCB)
      cosgCB = cos(gamVCB)
      F = W*(singCB + DoL*cosgCB)
      Fe = F/neng
      pare(ieFe,ip) = Fe

ccc      write(*,*) 'cutbackT...'
cc---- calculate required cutback-segment engine power to get specified climb angle
c      icall = 1
c      icool = 1
c      call tfcalc(pari,parg,para(1,ip),pare(1,ip), ip,
c     &            icall,icool,initeng,
c     &            ichoke5(ip),ichoke7(ip))  ! initialize with reasonable Tt4
c      pare(ieFe,ip) = Fe                    ! now prescribe the required thrust


c---- initialize with climb engine state
      ipc = ipclimb1
      pare(iembf ,ip) = pare(iembf ,ipc)
      pare(iemblc,ip) = pare(iemblc,ipc)
      pare(iembhc,ip) = pare(iembhc,ipc)
      pare(iepif ,ip) = pare(iepif ,ipc)
      pare(iepilc,ip) = pare(iepilc,ipc)
      pare(iepihc,ip) = pare(iepihc,ipc)
      pare(iept5 ,ip) = pare(iept5 ,ipc)

      dTburn = pare(ieTt4,ipc) - pare(ieTt3,ipc)
      pare(ieTt4 ,ip) = pare(ieTt3,ipc) + dTburn


c---- calculate required cutback-segment engine power to get specified climb angle
ccc      write(*,*) 'cutbackF...'
      icall = 2
      icool = 1

      inite = 1
      call tfcalc(pari,parg,para(1,ip),pare(1,ip), ip,
     &            icall,icool,inite,
     &            ichoke5(ip),ichoke7(ip)) 

c---- set actual climb angle (should be same as asin(singCB))
      F   = pare(ieFe,ip)*neng
      DoL = para(iaCD,ip)/para(iaCL,ip)
      phi = F/W
      sing = (phi - DoL*sqrt(1.0 - phi**2 + DoL**2)) / (1.0 + DoL**2)
      cosg = sqrt(1.0 - sing**2)
      para(iagamV,ip) = atan2(sing,cosg)

c-------------------------------------------------------------------------------
c---- sideline noise
      ip = iptakeoff

      A2 = pare(ieA2,ip)
      M2 = pare(ieM2,ip)

      A6 = pare(ieA6,ip)
      A8 = pare(ieA8,ip)
      u6 = pare(ieu6,ip)
      u8 = pare(ieu8,ip)
      T6 = pare(ieT6,ip)
      T8 = pare(ieT8,ip)
      M0 = pare(ieM0,ip)
      etaf = pare(ieetaf,ip)
      FPR  = pare(iepif ,ip)
      BPR  = pare(ieBPR ,ip)
      mdot = pare(iemcore,ip)

      h = para(iaalt,ip)

      rho0 = pare(ierho0,ip)
      p0   = pare(iep0  ,ip)
      T0   = pare(ieT0  ,ip)
      mu0  = pare(iemu0 ,ip)
      c0   = pare(iea0  ,ip)

      Mt   = 1.2    ! tangential tip Mach number
      Mtr  = 1.38   ! relative   tip Mach number
      Mtrd = 1.30   ! relative   tip Mach number at design conditions
      c2 = c0 * sqrt( (1.0+0.5*(gamSL-1.0)*M0**2)
     &               /(1.0+0.5*(gamSL-1.0)*M2**2) )
      omega = Mt*c2 / (0.5*dfan)
      RPM  = omega * 30.0/pi

      c2 = c0 * sqrt( (1.0+0.5*(gamSL-1.0)*M0**2)
     &               /(1.0+0.5*(gamSL-1.0)*M2**2) )
      omega = Mt*c2 / (0.5*dfan)
      RPM  = omega * 30.0/pi


      climb = parm(imgamVTO)
      alpha = 5.0
      vector = 0.0

      x = 0.0
      y = 450.0
      z = 0.0

      call tfnoise(
     &       x,y,z,
     &       climb,alpha,vector, 
     &       rho0, p0, T0, mu0, c0,
     &       A6,A8,u6,u8,T6,T8,M0,
     &       etaf,FPR,mdot,
     &       BPR,Mtrd,Mtr,Mt,RPM,
     &       rss,B,V,htr,
     &       neng,type,method,
     &       spl_jet_dba,
     &       spl_f_t_dba,
     &       spl_f_b_dba,
     &       spl_f_z_dba,
     &       spl_r_t_dba,
     &       spl_r_b_dba,
     &       spl_tot_dba )
      if(inprint.ne.0) then
      write(*,*)
      write(*,*) 'Sideline...'
      write(*,*) 'Jet                    = ', spl_jet_dba
      write(*,*) 'Foward Fan Tonal       = ', spl_f_t_dba
      write(*,*) 'Foward Fan Broadband   = ', spl_f_b_dba
      write(*,*) 'Foward Fan Buzzsaw     = ', spl_f_z_dba
      write(*,*) 'Rearward Fan Tonal     = ', spl_r_t_dba
      write(*,*) 'Rearward Fan Broadband = ', spl_r_b_dba
      write(*,*) 'Total                  = ', spl_tot_dba
      endif

      parm(imdBSL) = spl_tot_dba

c-------------------------------------------------------------------------------
c---- cutback point noise  (still at takeoff power and climb angle)

      gamVTO = parm(imgamVTO)
      singTO = sin(gamVTO)
      cosgTO = cos(gamVTO)
      tangTO = singTO/cosgTO

      gamVCB = parm(imgamVCB)
      singCB = sin(gamVCB)
      cosgCB = cos(gamVCB)
      tangCB = singCB/cosgCB

      cott = tan(0.5*pi-parm(imthCB))

      lTO  = parm(imlTO)
      lobs = 6500.0

      dhTO = (lobs-lTO)*tangTO/(1.0 + tangTO*cott)
      lCB  = (lobs-lTO)       /(1.0 + tangTO*cott)  +  lTO

c---- aircraft's absolute altitude at cutback point is dhTO above runway
      h = para(iaalt,ip) + dhTO

c---- slightly modified atmospheric quantities, based on pressure gradient and dhTO
      p0   = pare(iep0,ip) - pare(ierho0,ip)*gee*dhTO 
      T0   = pare(ieT0,ip)

      prat = p0/pare(iep0,ip)
      Trat = T0/pare(ieT0,ip)

c---- new density from state equation
      rho0 = pare(ierho0,ip) * prat / Trat

c---- new viscosity from mu ~ T assumption
      mu0  = pare(iemu0,ip) * Trat
      c0   = pare(iea0 ,ip) * sqrt(Trat)


      Mt   = 1.2    ! tangential tip Mach number
      Mtr  = 1.38   ! relative   tip Mach number
      Mtrd = 1.30   ! relative   tip Mach number at design conditions
      c2 = c0 * sqrt( (1.0+0.5*(gamSL-1.0)*M0**2)
     &               /(1.0+0.5*(gamSL-1.0)*M2**2) )
      omega = Mt*c2 / (0.5*dfan)
      RPM  = omega * 30.0/pi

      climb = asin(singTO)
      alpha = 5.0 * pi/180.0
      vector = 0.0

c---- observer position relative to aicraft (+x forward, +y right, +z down)
      x = lobs - lCB
      y = 0.0
      z = dhTO


c      write(*,*) 'Cutback...'
c      write(*,*) x,y,z
c      write(*,*) climb
      call tfnoise(
     &       x,y,z,
     &       climb,alpha,vector, 
     &       rho0, p0, T0, mu0, c0,
     &       A6,A8,u6,u8,T6,T8,M0,
     &       etaf,FPR,mdot,
     &       BPR,Mtrd,Mtr,Mt,RPM,
     &       rss,B,V,htr,
     &       neng,type,method,
     &       spl_jet_dba,
     &       spl_f_t_dba,
     &       spl_f_b_dba,
     &       spl_f_z_dba,
     &       spl_r_t_dba,
     &       spl_r_b_dba,
     &       spl_tot_dba )
      if(inprint.ne.0) then
      write(*,*)
      write(*,*) 'Cutback point...'
      write(*,*) 'Jet                    = ', spl_jet_dba
      write(*,*) 'Foward Fan Tonal       = ', spl_f_t_dba
      write(*,*) 'Foward Fan Broadband   = ', spl_f_b_dba
      write(*,*) 'Foward Fan Buzzsaw     = ', spl_f_z_dba
      write(*,*) 'Rearward Fan Tonal     = ', spl_r_t_dba
      write(*,*) 'Rearward Fan Broadband = ', spl_r_b_dba
      write(*,*) 'Total                  = ', spl_tot_dba
      endif

      parm(imdBCB) = spl_tot_dba
      parm(imxCB) = x
      parm(imzCB) = z
      parm(imlCB) = lCB

c-------------------------------------------------------------------------------
c---- overhead point noise  (at cutback power, same V2 takeoff speed)
      ip = ipcutback

cc---- calculate cutback-segment engine power
c      icall = 1
c      icool = 1
c      call tfcalc(pari,parg,para(1,ip),pare(1,ip), ip,
c     &            icall,icool,initeng,
c     &            ichoke5(ip),ichoke7(ip)) 
c
cc---- calculate sin,cos,tan of climb angle
c      W = parg(igWMTO)
c      DoL = para(iaCD,ip)/para(iaCL,ip)
c      F   = pare(ieFe,ip)*neng
c      phi = F/W
c      singCB = (phi - DoL*sqrt(1.0 - phi**2 + DoL**2)) / (1.0 + DoL**2)
c      cosgCB = sqrt(1.0 - singCB**2)
c      tangCB = singCB/cosgCB
c      para(iagamV,ip) = atan2(singCB,cosgCB)
c      parg(igsingCB) = singCB

c---- initialize engine corrected quantities from cruise1 point
c      pif  = pare(iepif ,ipcruise1)
c      pilc = pare(iepilc,ipcruise1)
c      pihc = pare(iepihc,ipcruise1)
c      mbf  = pare(iembf ,ipcruise1)
c      mblc = pare(iemblc,ipcruise1)
c      mbhc = pare(iembhc,ipcruise1)
c      Tt4  = pare(ieTt4 ,ipcruise1)

c---- calculate required cutback-segment engine power to get specified climb angle
c      pare(iepif ,ipcutback) = 1.0 + (pif -1.0)*0.50
c      pare(iepilc,ipcutback) = 1.0 + (pilc-1.0)*0.55
c      pare(iepihc,ipcutback) = 1.0 + (pihc-1.0)*0.60
c      pare(iembf ,ipcutback) = mbf *0.70
c      pare(iemblc,ipcutback) = mblc*0.70
c      pare(iembhc,ipcutback) = mbhc*0.80
c      pare(ieTt4 ,ipcutback) = Tt4
c      W = parg(igWMTO)
c      DoL = para(iaCD,ip)/para(iaCL,ip)
c      singCB = parg(igsingCB)
c      cosgCB = sqrt(1.0 - singCB**2)
c      F = W*(singCB + DoL*cosgCB)
c      Fe = F/neng
c      pare(ieFe,ip) = Fe
cc      icall = 2
c      icall = 1
c      icool = 1
c      call tfcalc(pari,parg,para(1,ip),pare(1,ip), ip,
c     &            icall,icool,initeng,
c     &            ichoke5(ip),ichoke7(ip)) 

c---- set quantities for tfnoise call
      Mt   = 1.2    ! tangential tip Mach number
      Mtr  = 1.38   ! relative   tip Mach number
      Mtrd = 1.30   ! relative   tip Mach number at design conditions
      c2 = c0 * sqrt( (1.0+0.5*(gamSL-1.0)*M0**2)
     &               /(1.0+0.5*(gamSL-1.0)*M2**2) )
      omega = Mt*c2 / (0.5*dfan)
      RPM  = omega * 30.0/pi

c---- set quantities for tfnoise call
      climb = asin(singCB)
      alpha = 5.0 * pi/180.0
      vector = 0.0

      A2 = pare(ieA2,ip)
      M2 = pare(ieM2,ip)

      A6 = pare(ieA6,ip)
      A8 = pare(ieA8,ip)
      u6 = pare(ieu6,ip)
      u8 = pare(ieu8,ip)
      T6 = pare(ieT6,ip)
      T8 = pare(ieT8,ip)
      M0 = pare(ieM0,ip)
      etaf = pare(ieetaf,ip)
      FPR  = pare(iepif ,ip)
      BPR  = pare(ieBPR ,ip)
      mdot = pare(iemcore,ip)

c---- aircraft position at cutback point, relative to observer (+x to right, +z up)
      rxCB = -parm(imxCB)
      rzCB =  parm(imzCB)

c---- note: (cosgCB,singCB) is unit vector along aircraft path
      sxr = cosgCB*rzCB - singCB*rxCB
      rnx = -sxr*singCB
      rnz =  sxr*cosgCB

c---- sighting elevation angle to airplane
      thetad = 90.0
      theta = thetad*pi/180.0

c------ unit vector from oberver to airplane
        sint = sin(pi-theta)
        cost = cos(pi-theta)

        sxt = cosgCB*cost + singCB*sint
        tanp = sxt/sqrt(1.0-sxt**2)

c------ aircraft location at intersection of cutback climb line and (cost,sint)
        xa = rnx + rnz*tanp
        za = rnz - rnx*tanp

c------ observer position relative to aicraft (+x forward, +y right, +z down)
        x = -xa
        y = 0.0
        z =  za
        call tfnoise(
     &       x,y,z,
     &       climb,alpha,vector, 
     &       rho0, p0, T0, mu0, c0,
     &       A6,A8,u6,u8,T6,T8,M0,
     &       etaf,FPR,mdot,
     &       BPR,Mtrd,Mtr,Mt,RPM,
     &       rss,B,V,htr,
     &       neng,type,method,
     &       spl_jet_dba,
     &       spl_f_t_dba,
     &       spl_f_b_dba,
     &       spl_f_z_dba,
     &       spl_r_t_dba,
     &       spl_r_b_dba,
     &       spl_tot_dba )

      parm(imdBFO) = spl_tot_dba
      parm(imxFO) = x
      parm(imzFO) = z
      if(inprint.ne.0) then
      write(*,*)
      write(*,*) 'Flyover...'
      write(*,*) 'Jet                    = ', spl_jet_dba
      write(*,*) 'Foward Fan Tonal       = ', spl_f_t_dba
      write(*,*) 'Foward Fan Broadband   = ', spl_f_b_dba
      write(*,*) 'Foward Fan Buzzsaw     = ', spl_f_z_dba
      write(*,*) 'Rearward Fan Tonal     = ', spl_r_t_dba
      write(*,*) 'Rearward Fan Broadband = ', spl_r_b_dba
      write(*,*) 'Total                  = ', spl_tot_dba
      endif

      return
      end ! noise
