c
      subroutine surfcd2(
     &  S,
     &  b,bs,bo,
     &  lambdat,lambdas,gammat,gammas,
     &  toco,tocs,toct,
     &  Mach,sweep,co, 
     &  CL,CLhtail, fLo,fLt,
     &  Reco,aRexp, kSuns,fexcd,
     &  iAdim,jAdim,kAdim,lAdim,
     &  nAMa,nAcl,nAtau,nAfun,
     &  AMa,Acl,Atau,ARe,
     &  A,
     &  A_M,
     &  A_cl,
     &  A_tau,
     &  A_M_cl,
     &  A_M_tau,
     &  A_cl_tau,
     &  A_M_cl_tau,
     &  fduo,fdus,fdut,
     &  clpo,clps,clpt,
     &  CDfwing,CDpwing,CDwing,CDover)
C=======================================================
c     Calculates wing or tail surface profile CD
c
c     Inputs
c      S    reference area
c      b    span
c      bs   outer panel break span
c      bo   wing-root (fuselage) span
c      lambdat  outer-panel chord taper ratio  ct/co
c      lambdas  inner-panel chord taper ratio  cs/co
c      gammat   outer-panel load  taper ratio  pt/po
c      gammas   inner-panel load  taper ratio  ps/po
c      toco   root  airfoil t/c
c      tocs   break airfoil t/c
c      toct   tip   airfoil t/c
c      sweep  wing sweep, degrees
c      co     wing root chord
c      Reco   Reynolds number for co
c      aRexp  Re-scaling exponent
c      kSuns  shock-unsweep area constant
c      fexcd  excrescence cd scaling factor
c      fduo   fractional overspeeds at wing stations
c      fdus
c      fdut
c
c     Outputs
c      clpo     cl at wing stations
c      clps
c      clpt
c      CDfwing  friction profile cd in perp. plane
c      CDpwing  pressure profile cd in perp. plane
c      CDwing   overall profile CD
c      CDover   fuselage added CD due to lift carryover
c
c=======================================================
      implicit real (a-z)

      integer iAdim,jAdim,kAdim, lAdim
      integer nAMa, nAcl, nAtau, nAfun
      real AMa(iAdim), Acl(jAdim), Atau(kAdim), ARe
      real A(iAdim,jAdim,kAdim,lAdim),
     &   A_M(iAdim,jAdim,kAdim,lAdim),
     &   A_cl(iAdim,jAdim,kAdim,lAdim),
     &   A_tau(iAdim,jAdim,kAdim,lAdim),
     &   A_M_cl(iAdim,jAdim,kAdim,lAdim),
     &   A_M_tau(iAdim,jAdim,kAdim,lAdim),
     &   A_cl_tau(iAdim,jAdim,kAdim,lAdim),
     &   A_M_cl_tau(iAdim,jAdim,kAdim,lAdim)

c---- local arrays
      integer i, j, k, n
c     parameter (n=4)      ! n must be even
c     parameter (n=0)      ! n must be even
c      real eta(n), deta(4)

      include 'time.inc'

      data pi / 3.1415926535897932384626 /

c     n = 4     !  ~0.30% error
c     n = 6     !  ~0.13% error
      n = 8     !  ~0.07% error
c     n = 12    !  ~0.07% error

c     call tset(time0)

      cosL = cos(sweep*pi/180.0)

      AR = b**2/S

      etao = bo/b
      etas = bs/b

c---- set center clpo
      Kc = etao
     &   + 0.5*(1.0    +lambdas)*(etas-etao)
     &   + 0.5*(lambdas+lambdat)*(1.0 -etas)

      Kp0 = etao
     &    + 0.5*(1.0    +gammas )*(etas-etao)
     &    + 0.5*(gammas +gammat )*(1.0 -etas)

      Ko = 1.0/(Kc*AR)
      Kp = Kp0 + fLo*etao + 2.0*fLt*Ko*gammat*lambdat

      clp1 = (CL-CLhtail)/cosL**2 * S/(Kp*b*co)

c---- set break and tip clp for passing back
      clpo = clp1                  / (1.0+fduo)**2
      clps = clp1 * gammas/lambdas / (1.0+fdus)**2
      clpt = clp1 * gammat/lambdat / (1.0+fdut)**2

cc---- area of exposed wing
c      Swing = co*b * ( 0.5*(1.0    +lambdas)*(etas-etao)
c     &               + 0.5*(lambdas+lambdat)*(1.0 -etas) )

c      write(*,*)
c      write(*,*) CL, CLhtail, S/(Kp*b*co)

c---- fraction of exposed wing subject to wing-root shock unsweep
      CDfwing = 0.
      CDpwing = 0.
      CDwing = 0.
      Snorm = 0.
      do i = 1, n/2
        frac = (float(i)-0.5)/float(n/2)
        eta = etao*(1.0-frac) + etas*frac
        deta = (etas-etao)/float(n/2)
        P = 1.0-frac + gammas *frac
        C = 1.0-frac + lambdas*frac
        toc = toco*(1.0-frac) + tocs*frac
        fdu = fduo*(1.0-frac) + fdus*frac

        fSuns = exp( -(eta-etao) * b / (kSuns*C * 2.0*co) )

        clp   = clp1 * (P/C) / (1.0+fdu)**2
        Rec   = Reco *  C    * (1.0+fdu)
        Mperp = Mach * cosL  * (1.0+fdu)
        call airfun(clp,toc,Mperp,
     &              iAdim,jAdim,kAdim,lAdim,
     &              nAMa,nAcl,nAtau,nAfun,
     &              AMa,Acl,Atau,ARe,
     &              A,
     &              A_M,
     &              A_cl,
     &              A_tau,
     &              A_M_cl,
     &              A_M_tau,
     &              A_cl_tau,
     &              A_M_cl_tau,
     &              cdf1, cdp1, cdwbar, cm)
        Refac = (Rec/ARe)**ARexp
        cdf = cdf1*Refac * fexcd * (1.0+fdu)**3
        cdp = cdp1*Refac * fexcd * (1.0+fdu)**3
     &      * (fSuns + (1.0-fSuns)*cosL**2)*cosL

        cd = cdf + cdp

c        write(*,'(1x,8f12.6)') eta, fSuns, clp, cdf+cdp*cosL**3, cd

        Snorm   = Snorm   + C*deta
        CDwing  = CDwing  + C*deta*cd

        CDfwing = CDfwing + C*deta*cdf
        CDpwing = CDpwing + C*deta*cdp
      enddo

      do i = n/2+1, n
        frac = (float(i-n/2)-0.5)/float(n/2)
        eta = etas*(1.0-frac) + 1.0*frac
        deta = (1.0-etas)/float(n/2)
        P = gammas *(1.0-frac) + gammat *frac
        C = lambdas*(1.0-frac) + lambdat*frac
        toc = tocs*(1.0-frac) + toct*frac
        fdu = fdus*(1.0-frac) + fdut*frac

        fSuns = exp( -(eta-etao) * b / (kSuns*C * 2.0*co) )

        clp   = clp1 * (P/C) / (1.0+fdu)**2
        Rec   = Reco *  C    * (1.0+fdu)
        Mperp = Mach * cosL  * (1.0+fdu)
        call airfun(clp,toc,Mperp,
     &              iAdim,jAdim,kAdim,lAdim,
     &              nAMa,nAcl,nAtau,nAfun,
     &              AMa,Acl,Atau,ARe,
     &              A,
     &              A_M,
     &              A_cl,
     &              A_tau,
     &              A_M_cl,
     &              A_M_tau,
     &              A_cl_tau,
     &              A_M_cl_tau,
     &              cdf1, cdp1, cdwbar, cm)
        Refac = (Rec/ARe)**ARexp
        cdf = cdf1*Refac * fexcd * (1.0+fdu)**3
        cdp = cdp1*Refac * fexcd * (1.0+fdu)**3
     &      * (fSuns + (1.0-fSuns)*cosL**2)*cosL

        cd = cdf + cdp

c        write(*,'(1x,8f12.6)') eta, fSuns, clp, cdf+cdp*cosL**3, cd

        Snorm   = Snorm   + C*deta
        CDwing  = CDwing  + C*deta*cd

        CDfwing = CDfwing + C*deta*cdf
        CDpwing = CDpwing + C*deta*cdp
      enddo

      CDwing  = CDwing/Snorm
      CDfwing = CDfwing/Snorm
      CDpwing = CDpwing/Snorm

cc---- fuselage carryover CD
c      Cdiss = 0.001
c      dCp = (Kc/Kp)*(CL-CLhtail)*(1.0+fLo)
c      CDover = 2.0*Cdiss*(etao/Kc)*0.5*dCp*(3.0 + 0.0625*dCp**2)
      CDover = 0.

c     call tadd(time0,t_surfcd2)

      return
      end ! surfcd2



      subroutine surfcd(S,
     &  b,bs,bo,lambdat,lambdas,sweep,co, 
     &  cdf,cdp,Reco,Reref,aRexp, kSuns,
     &  fCDcen,
     &  CDsurf,CDover)
c=======================================================
c     Calculates wing or tail surface profile CD
c
c     Inputs
c      S   reference area
c      b   span
c      bs  outer panel break span
c      bo  wing-root (fuselage) span
c      lambdat  outer-panel taper ratio  ct/co
c      lambdas  inner-panel taper ratio  cs/co
c      sweep    wing sweep, degrees
c      co    wing root chord
c      cdf   friction profile cd

c      cdp   pressure profile cd
c      Reco  Reynolds number for co
c      Reref reference Reynolds number for cd scaling
c      aRexp Re-scaling exponent
c      kSuns shock-unsweep area constant
c
c     Outputs
c      CDsurf  overall profile CD
c      CDover  fuselage added CD due to lift carryover
c
c=======================================================
      implicit real (a-z)

      include 'time.inc'

      data pi / 3.1415926535897932384626 /

c     call tset(time0)

      cosL = cos(sweep*pi/180.0)

      etao = bo/b
      etas = bs/b

c---- area of exposed surf
      Ssurf = co*0.5*b * ( (1.0    +lambdas)*(etas-etao)
     &                   + (lambdas+lambdat)*(1.0 -etas))

c---- fraction of exposed surf subject to root shock unsweep
      fSuns = kSuns*co**2 / (0.5*Ssurf)

c---- overall profile cd with Re and sweep effects, referenced to root chord co
      Refac = (Reco/Reref)**ARexp
      cdo = (cdf + cdp*(fSuns + (1.0-fSuns)*cosL**2)*cosL) * Refac
c
c---- integrate cd over tapered surf, assuming  cd ~ Re^aRexp  dependence
      lsxa = lambdas**aRexp
      lxa  = lambdat**aRexp
c
      if(abs(lambdas-1.0) .lt. 0.02) then
c----- asymptotic form for lambdas ~ 1
       lsfac = 1.0 - aRexp*(1.0-lambdas)/(1.0+lambdas)
      else
c----- exact form, but singular if lambdas = 1
       lsfac = 2.0/(2.0+aRexp)
     &       * (1.0 - lsxa*lambdas**2)
     &       / (1.0 -      lambdas**2)
      endif
c
      if(abs(lambdat-lambdas) .lt. 0.02) then
c----- asymptotic form for lambdat ~ lambdas
       lfac = 1.0 - aRexp*(lambdas-lambdat)/(lambdas+lambdat)
      else
c----- exact form, but singular if lambdat = lambdas
       lfac = 2.0/(2.0+aRexp)
     &       * (lsxa*lambdas**2 - lxa*lambdat**2)
     &       / (     lambdas**2 -     lambdat**2)
      endif

      CDsurf = (co*0.5*b/S) * cdo
     &    *(   2.0             * etao * fCDcen
     &      + (1.0    +lambdas)*(etas-etao)*lsfac
     &      + (lambdas+lambdat)*(1.0 -etas)*lfac  )

c      if(lambdat.eq.0.25) then
c      write(*,*)
c      write(*,*) fCDcen, lsfac, lfac
c      write(*,*) etao, etas-etao, 1.0-etas
c      write(*,*) etao, etas-etao, 1.0-etas
c      write(*,*) 2.0 * etao,
c     &          (1.0    +lambdas)*(etas-etao),
c     &          (lambdas+lambdat)*(1.0 -etas), 
c     &          2.0 * etao
c     &         +(1.0    +lambdas)*(etas-etao)
c     &         +(lambdas+lambdat)*(1.0 -etas)
c        write(*,*) 2.0 * etao * fCDcen,
c     &          (1.0    +lambdas)*(etas-etao)*lsfac,
c     &          (lambdas+lambdat)*(1.0 -etas)*lfac,
c     &           2.0 * etao * fCDcen
c     &         +(1.0    +lambdas)*(etas-etao)*lsfac
c     &         +(lambdas+lambdat)*(1.0 -etas)*lfac
c      write(*,*) Sh/0.3048**2 , CDsurf
c      endif

      CDover = 0.

c     call tadd(time0,t_surfcd)

      return
      end ! surfcd
