
      subroutine takeoff(pari,parg,parm,para,pare,
     &                   initeng,
     &                   ichoke5,ichoke7)
c==============================================================
c     Calculates takeoff parameters 
c     and balanced field length.
c
c     The aircraft must be defined in parg array.
c     The ipstatic and iprotate points are assumed to exist
c==============================================================
      implicit real (a-h,k-z)

      include 'index.inc'
      integer pari(iitotal)
      real parg(igtotal), 
     &     parm(imtotal),
     &     para(iatotal,iptotal),
     &     pare(ietotal,iptotal)
      integer ifuel,initeng
      integer ichoke5(iptotal),
     &        ichoke7(iptotal)


c---- airfoil database arrays
      include 'airf.inc'

c---- constants.inc  contains pi,gee,cp,gamma
      include 'constants.inc'

c---- Newton convergence tolerance
      data toler / 1.0e-7 /

c---- zero-fuel weight for this mission
      Wzero = parg(igWMTO)
     &      - parg(igWfuel)
     &      - parg(igWpay)
     &      + parm(imWpay)

c---- mission TO fuel weight
      WfTO = parm(imWTO) - Wzero

c---- unpack parameters passed in via global data arrays parg,pare
      W     = parm(imWTO)    ! total takeoff weight
      S     = parg(igS)      ! reference (wing) area
      sweep = parg(igsweep)  ! sweep angle, degrees
      dfan  = parg(igdfan)   ! fan diameter , for engine-out CD_eng estimate
      HTRf  = parg(igHTRf)   ! hub/tip ratio, for engine-out CD_eng estimate
      neng  = parg(igneng)   ! number of engines
      muroll  = parg(igmuroll)   ! rolling resistance coefficient
      mubrake = parg(igmubrake)  ! max braking resistance coefficient
      hobst   = parg(ighobst)    ! obstacle height (FAA spec, ~10m I think)

      Vstall = pare(ieu0,iprotate )
      V2     = pare(ieu0,iptakeoff)

      cosL = cos(sweep*pi/180.0)
      Afan = 0.25*pi*dfan**2 * (1.0 - HTRf**2)
      CDgear = parg(igCDgear)
      CDeng = parg(igcdefan) * (0.25*pi*dfan**2) / S
      CDspoiler = parg(igCDspoil)
      CDivert = 0.002

cc---- set static thrust value Fmax
c      ip = ipstatic
c      icall = 1
c      icool = 1
c      call tfcalc(pari,parg,para(1,ip),pare(1,ip), ip,
c     &            icall,icool,initeng,
c     &            ichoke5(ip),ichoke7(ip)) 

cc---- set reference thrust value Fref
c      ip = iprotate
c      icall = 1
c      icool = 1
c      call tfcalc(pari,parg,para(1,ip),pare(1,ip), ip,
c     &            icall,icool,initeng,
c     &            ichoke5(ip),ichoke7(ip)) 

      Fmax = pare(ieFe,ipstatic)
      Fref = pare(ieFe,iprotate)

c---- single-engine thrust-curve constants for takeoff roll calculations
      F01 = (Fmax+Fref)/2.0
      KV1 = (Fmax-Fref)/Vstall**2

c---- set max takeoff thrust for output
      FTO = Fmax * neng

c================================================================
c---- normal takeoff
      ip = iprotate
      rho0 = pare(ierho0,ip)
      CLroll = 0.0
      para(iaCL,ip) = CLroll
      para(iaCLh,ip) = 0.

ccc      write(*,*) '** 3a', Fmax, Fref

c---- total CD during roll
      icdfun = 0
      iairf = 1
      call cdsum(pari,parg,para(1,ip),pare(1,ip), icdfun,iairf)
      CDroll = para(iaCD,ip) + parg(igCDgear)

c---- thrust constants for all engines operating
      KV = KV1 * neng
      F0 = F01 * neng

c---- calculate takeoff length and takeoff time
      kA = (KV + rho0*S*CDroll)/(W/gee)
      VAlimsq = 2.0*(F0 - W*muroll)/(KV + rho0*S*CDroll)

c      write(*,*) 'F0 W muroll KV', F0, W, muroll, KV
c      write(*,*) 'VAlimsq', VAlimsq

      VAlim = sqrt(VAlimsq)
      if(VAlim .le. V2) then
       write(*,*) 'Normal takeoff impossible. VAlim < V2:', VAlim, V2
       lTO = 10.0/kA
       tTO = 2.0*lTO/V2
      else
       Vrat = V2/VAlim
       lTO = -log(1.0-Vrat**2)/kA
       tTO = log((1.0+Vrat)/(1.0-Vrat))/(kA*VAlim)
      endif

c---- initial normal climbout sin(angle), gear up
      CDclimb = para(iaCD,ipclimb1)
      F2 = F0 - KV*0.5*V2**2
      singTO = (F2 - 0.5*V2**2 * rho0*S*CDclimb) / W
      singTO = max( 0.01 , min( 0.99 , singTO ) )

c================================================================
c---- balanced field length

c---- total CD during ground roll with one engine out
      CDb = CDroll + CDeng + CDivert

c---- total CD during braking
      CDc = CDroll + CDeng*neng + CDspoiler

c---- thrust constants with one engine out
      KV = KV1 * (neng-1.0)
      F0 = F01 * (neng-1.0)

c---- velocity segment constants
      kB = (KV + rho0*S*CDb)/(W/gee)
      kC = (     rho0*S*CDc)/(W/gee)

      VBlimsq = 2.0*(F0 - W*muroll )/(KV + rho0*S*CDb)
      VClimsq = 2.0*(   - W*mubrake)/(     rho0*S*CDc)

      VBlim = sqrt(VBlimsq)

      if(VBlim .le. V2) then
       write(*,*) 'Engine-out takeoff impossible. VBlim < V2:', VBlim,V2
       l1  =  0.7*lTO
       lBF = 10.0*lTO
       V1 = Vstall
       V2 = Vstall
       singBF = 0.
       go to 50
      endif

c---- initial guesses for Newton iteration
      l1  = 0.8*lTO
      lBF = 1.3*lTO
      V2sq = V2**2

c---- Newton iteration loop
      do iter = 1, 15
        exA = exp(kA*(   -l1))
        exB = exp(kB*(lBF-l1))
        exC = exp(kC*(lBF-l1))
        exA_l1  = -kA*exA
        exB_l1  = -kB*exB
        exC_l1  = -kC*exC
        exA_lBF =  0.
        exB_lBF =  kB*exB
        exC_lBF =  kC*exC

        r1  = VAlimsq*(1.0-exA    ) + (VBlimsq-V2sq)*exB    - Vblimsq
        a11 = VAlimsq*(   -exA_l1 ) + (VBlimsq-V2sq)*exB_l1
        a12 = VAlimsq*(   -exA_lBF) + (VBlimsq-V2sq)*exB_lBF

        r2  = VAlimsq*(1.0-exA    ) - VClimsq*(1.0-exC    )
        a21 = VAlimsq*(   -exA_l1 ) - VClimsq*(   -exC_l1 )
        a22 = VAlimsq*(   -exA_lBF) - VClimsq*(   -exC_lBF)

        det  =   a11*a22 - a12*a21
        dl1  = -(r1 *a22 - a12*r2 )/det
        dlBF = -(a11*r2  - r1 *a21)/det

        dmax = max( abs(dl1) , abs(dlBF) )

cc------ print convergence history for debugging
c        write(*,'(1x, i3, 3g13.5, e12.4)') 
c     &   iter, lTO*3.28, l1*3.28, lBF*3.28, dmax*3.28

        l1  = l1  + dl1
        lBF = lBF + dlBF

        if(dmax .lt. toler*lTO) go to 20
      enddo
      write(*,*) 'TAKEOFF: BFL convergence failed.  dl1,dlBF=',dl1,dlBF
      write(*,*) '         Continuing anyway...'

 20   continue
      V1 = VAlim * sqrt(1.0 - exp(-kA*l1))

c---- engine-out climbout, gear down
      F2 = F0 - KV*0.5*V2**2
      CD = CDclimb + CDgear + CDeng
ccc   singBF = (F2 - 0.5*V2**2 * rho0*S*CDb) / W
      singBF = (F2 - 0.5*V2**2 * rho0*S*CD ) / W
      singBF = max( 0.01 , min( 0.99 , singBF ) )

 50   continue

c---- calculate fuel burn during takeoff run
      ip1 = ipstatic
      ip2 = iprotate
      mdotf1 = pare(iemcore,ip1)*pare(ieff,ip1)*neng
      mdotf2 = pare(iemcore,ip2)*pare(ieff,ip2)*neng
      WfTO = 0.5*(mdotf1+mdotf2)*tTO * gee
      para(iafracW,ipstatic) = para(iafracW,iptakeoff)
     &                       + WfTO/parg(igWMTO)
      para(iatime ,ipstatic) = para(iatime ,iptakeoff) - tTO
      para(iaRange,ipstatic) = para(iaRange,iptakeoff) - lTO

c---- store output quantities for returning
      parm(imV1    ) = V1    
      parm(imV2    ) = V2    
      parm(imlTO   ) = lTO   
      parm(iml1    ) = l1    
      parm(imlBF   ) = lBF   
      parm(imtTO   ) = tTO   
      parm(imFTO   ) = FTO   
      parm(imgamVTO) = asin(singTO)
      parm(imgamVBF) = asin(singBF)

      return
      end ! takeoff


