
      subroutine Ncmap(pi,mb, piD,mbD,NbD, Cmap,
     &                  Nb, Nb_pi, Nb_mb )
c==============================================================
c     Calculates compressor or fan corrected speed as a function 
c     of pressure ratio and corrected mass flow
c
c     Inputs
c       pi      pressure ratio
c       mb      corrected mass flow
c       piD     design pressure ratio
c       mbD     design corrected mass flow
c       NbD     design corrected speed
c       Cmap(.) map constants
c
c     Outputs
c       Nb    wheel speed
c       Nb_?  derivatives
c==============================================================
      implicit real (a-z)
      real Cmap(9)

      integer iter, ipass

cc      character*2 ss

cc      data eps / 1.0e-8 /
cc      data eps / 1.0e-9 /
      data eps / 1.0e-11 /


c----              a     b     k     mo     da    c    d     C    D
c     data Cmapf / 3.50, 0.80, 0.03, 0.75, -0.50, 3.0, 6.0,  2.5, 15.0 /
      a  = Cmap(1)
      b  = Cmap(2)
      k  = Cmap(3)
      mrato = Cmap(4)
      da = Cmap(5)
      c  = Cmap(6)
      d  = Cmap(7)
      CK = Cmap(8)
      DK = Cmap(9)

c---- corrected mass flow / design corrected mass flow
      m    =  mb / mbD
      m_mb = 1.0 / mbD

c---- scaled pressure ratio
      p = (pi-1.0)/(piD-1.0)
      p_pi =   1.0/(piD-1.0)

c---- value of p on spine at specified m
      psm = m**a

c---- corrected speed ratio initial guess from spine
      N = m**(1.0/b)

c---- converge to actual corrected speed by inverting compressor map function
      do iter = 1, 20

c        do ipass = 1, 2

        ms = N**b
        ms_N = b*ms/N

        ps = N**(a*b)
        ps_N = a*b*ps/N

        if(p .ge. psm) then
c------- above spine... specify p
c           write(*,*) 'above'
         res   = ps   + 2.0*N*k*log(1.0-(m-ms)/k) - p
         res_N = ps_N + 2.0  *k*log(1.0-(m-ms)/k)
     &                + 2.0*N*k /  (1.0-(m-ms)/k) * ms_N/k
         res_m =      - 2.0*N*k /  (1.0-(m-ms)/k) *  1.0/k
         res_p = -1.0

        else
c------- below spine... specify m
c           write(*,*) 'below'
         res   = ms   + k*(1.0 - exp((p-ps)/(2.0*N*k))) - m
         res_N = ms_N + k*(    - exp((p-ps)/(2.0*N*k)))
     &                   *(-ps_N/(2.0*N*k) - (p-ps)/(2.0*N*k) / N)
         res_m = -1.0
         res_p =        k*(    - exp((p-ps)/(2.0*N*k))) / (2.0*N*k)

        endif

c         if(ipass.eq.1) then
c           res$ = res
c           res_N$ = res_N
c           N$ = N
c           N = N + eps
c         endif
c         enddo
c 3100    format(2(/1x,a,g16.8,2x,a))
c         dd = (res   - res$  )/eps
c         aa = (res_N + res_N$)*0.5
c         call compare(ss,aa,dd)
c         write(*,3100) 'aa',aa,ss,'dd',dd,ss
c         stop

        dN = -res/res_N
cc      if(abs(dN) .lt. eps) go to 20

        rlx = 1.0
        dNlim = 0.05
        if(rlx*dN .gt.  dNlim) rlx =  dNlim/dN
        if(rlx*dN .lt. -dNlim) rlx = -dNlim/dN

c        write(*,'(1x,i3,3f8.5, e12.4, g12.4, e12.4, f8.4)')
c     &   iter, p, m, N, res, res_N, dN, rlx

        if(abs(dN) .lt. eps) go to 20

        N = N + rlx*dN
      enddo
      write(*,*) 'Ncmap: conv.failed.  N, dN =', N, dN
 20   continue

c---- N( m(mb) , p(pi) )
      N_m = -res_m/res_N
      N_p = -res_p/res_N

c---- N( mb, pi )
      N_mb = N_m*m_mb
      N_pi = N_p*p_pi

c---- Nb( N(mb,pi) )
      Nb    = NbD*N
      Nb_mb = NbD*N_mb
      Nb_pi = NbD*N_pi

      return
      end ! Ncmap


      subroutine ecmap(pi,mb, piD,mbD, Cmap, effo, piK, effK,
     &                  eff, eff_pi, eff_mb )
c==============================================================
c     Calculates compressor or fan efficiency as a function 
c     of pressure ratio and corrected mass flow
c
c     Inputs
c       pi       pressure ratio
c       mb       corrected mass flow
c       piD      design pressure ratio
c       mbD      design corrected mass flow
c       Cmap(.)  map constants
c       effo     maximum efficiency
c       piK      pi-dependence offset  eff = effo + effK*(pi-piK)
c       effK     pi-dependence slope
c
c     Outputs
c       eff       efficiency
c       eff_?     derivatives
c==============================================================
      implicit real (a-z)
      real Cmap(9)

      data eps / 1.0e-8 /

c----              a     b     k     mo     da    c    d     C    D
c     data Cmapf / 3.50, 0.80, 0.03, 0.75, -0.50, 3.0, 6.0,  2.5, 15.0 /
      a  = Cmap(1)
      b  = Cmap(2)
      k  = Cmap(3)
      mo = Cmap(4)
      da = Cmap(5)
      c  = Cmap(6)
      d  = Cmap(7)
      CK = Cmap(8)
      DK = Cmap(9)

      adm = a + da - 1.0

c---- scaled pressure ratio
      p = (pi-1.0)/(piD-1.0)
      p_pi =   1.0/(piD-1.0)

c---- corrected mass flow / design correct mass flow
      m    =  mb / mbD
      m_mb = 1.0 / mbD

      psgn = sign( 1.0 , p/m**adm - m   )
      msgn = sign( 1.0 , m/mo     - 1.0 )

cc    eo  = 1.0 - DK*(  abs(1.0 /mo   - 1.0))**d
      eo  = 1.0

c---- eff( m(mdot,Tr,pr) , p(pi) , pi )
      e1   = 1.0 - CK*(psgn*(p/m**adm - m))**c
     &           - DK*(msgn*(m/mo   - 1.0))**d

      e1_p =     - CK*(psgn*(p/m**adm - m))**(c-1.0)
     &            *c * psgn   /m**adm       

      e1_m =     - CK*(psgn*(p/m**adm - m))**(c-1.0)
     &            *c * psgn*(-adm*p/m**adm / m - 1.0)
     &           - DK*(msgn*(m/mo   - 1.0))**(d-1.0)
     &            *d * msgn   /mo 
      eff    = effo*e1  /eo + effK*(pi-piK)
      eff_p  = effo*e1_p/eo
      eff_m  = effo*e1_m/eo
      eff_pi =                effK

c---- eff( pi, nb )
      eff_pi = eff_p*p_pi  +  eff_pi 
      eff_mb = eff_m*m_mb

c      write(*,*) prat, mrat, mo
c      write(*,*) - CK*(psgn*(prat/mrat**2 - 1.0))**c ,
c     &           - DK*(msgn*(mrat/mo   - 1.0))**d
c      write(*,*) eff, effo
c      stop

      return
      end ! ecmap




      subroutine Ncmap1(pi,m, piD,mbD,NbD, ABCDm,iabcd, Tr,pr,
     &                  N, N_pi, N_m, N_Tr, N_pr)
c==============================================================
c     Calculates compressor or fan speed as a function 
c     of pressure ratio and mass flow
c
c     Inputs
c       pi     pressure ratio
c       m      mass flow
c       piD    design pressure ratio
c       mbD    design corrected mass flow
c       NbD     design corrected speed
c       ABCDm  map constants
c       iabcd  map exponents
c       Tr     T/Tref
c       pr     p/pref
c
c     Outputs
c       N       wheel speed
c       N_?     derivatives
c==============================================================
      implicit real (a-z)

      real ABCDm(5)
      integer iabcd(4)

      integer ia, ib

      A = ABCDm(1)
      B = ABCDm(2)

      ia = iabcd(1)
      ib = iabcd(2)

      binv = 1.0/float(ib)

c---- corrected mass flow
      mb    = m*sqrt(Tr)/pr
      mb_m  =   sqrt(Tr)/pr
      mb_Tr = 0.5*mb/Tr
      mb_pr =    -mb/pr

c---- calculate inverse map
      prat = (pi-1.0)/(piD-1.0)
      mrat = mb/mbD

      fac      = (prat-1.0 + A*(mrat**ia-1.0))/B + 1.0
      fac_prat = 1.0/B
      fac_mrat =             A* mrat**(ia-1)*float(ia)/B

      Nb      = NbD * fac**binv
      Nb_prat = binv*Nb/fac * fac_prat
      Nb_mrat = binv*Nb/fac * fac_mrat

      Nb_pi = Nb_prat / (piD-1.0)
      Nb_mb = Nb_mrat /  mbD

c---- N( mb(m,Tr,pr) , pi, Tr )
      N    = Nb    *sqrt(Tr)
      N_mb = Nb_mb *sqrt(Tr)
      N_pi = Nb_pi *sqrt(Tr)
      N_Tr = Nb*0.5/sqrt(Tr)

c---- N( m, pi, Tr, pr )
      N_m  = N_mb*mb_m
ccc   N_pi =              N_pi
      N_Tr = N_mb*mb_Tr + N_Tr
      N_pr = N_mb*mb_pr

      return
      end ! Ncmap1




      subroutine ecmap1(pi,m, piD,mbD, ABCDm,iabcd, effo, Tr,pr, 
     &                  eff, eff_pi, eff_m, eff_Tr, eff_pr )
c==============================================================
c     Calculates compressor or fan efficiency as a function 
c     of pressure ratio and mass flow
c
c     Inputs
c       pi       pressure ratio
c       m        mass flow
c       piD      design pressure ratio
c       mbD      design corrected mass flow
c       ABCDm(.) map constants
c       iabcd(.) map exponents
c       effo     maximum efficiency
c       Tr       T/Tref
c       pr       p/pref
c
c     Outputs
c       eff       efficiency
c       eff_?     derivatives
c==============================================================
      implicit real (a-z)

      real ABCDm(5)
      integer iabcd(4)

      integer ic, id

      data eps / 1.0e-8 /


      C = ABCDm(3)
      D = ABCDm(4)
      mrato = ABCDm(5)

      ic = iabcd(3)
      id = iabcd(4)

c---- corrected mass flow
      mb    = m*sqrt(Tr)/pr
      mb_m  =   sqrt(Tr)/pr
      mb_Tr = 0.5*mb/Tr
      mb_pr =    -mb/pr

      prat = (pi-1.0)/(piD-1.0)
      mrat = mb/mbD

      psgn = sign( 1.0 , prat/m**2  - 1.0 )
      msgn = sign( 1.0 , mrat/mrato - 1.0 )

cc    eo  = 1.0 - D*(  abs(1.0 /mrato   - 1.0))**id
      eo  = 1.0

      e1  = 1.0 - C*(psgn*(prat/mrat**2 - 1.0))**ic
     &          - D*(msgn*(mrat/mrato   - 1.0))**id
      e1_prat = - C*(psgn*(prat/mrat**2 - 1.0))**(ic-1)
     &             * psgn      /mrat**2   * float(ic)
      e1_mrat = - D*(msgn*(mrat/mrato   - 1.0))**(id-1)
     &             * msgn      /mrato     * float(id)
     &          + C*(psgn*(prat/mrat**2 - 1.0))**(ic-1)
     &       * 2.0 * psgn* prat/mrat**3   * float(ic)

      eff      = effo*e1     /eo
      eff_prat = effo*e1_prat/eo
      eff_mrat = effo*e1_mrat/eo

c---- eff( mb(m,Tr,pr) , pi )
      eff_pi = eff_prat/(piD-1.0)
      eff_mb = eff_mrat/ mbD

c---- eff( m, pi, Tr, pr )
ccc   eff_pi = eff_pi
      eff_m  = eff_mb*mb_m
      eff_Tr = eff_mb*mb_Tr
      eff_pr = eff_mb*mb_pr

c      write(*,*) prat, mrat, mrato
c      write(*,*) - C*(psgn*(prat/mrat**2 - 1.0))**ic ,
c     &           - D*(msgn*(mrat/mrato   - 1.0))**id
c      write(*,*) eff, effo
c      stop

      return
      end ! ecmap1



      subroutine etmap(dh,mb,Nb, piD,mbD,NbD,ept0,Tmap,
     &           Tt,cpt,Rt,
     &           ept,
     &           ept_dh,ept_mb,ept_Nb,
     &           ept_Tt,ept_cpt,ept_Rt )
c==============================================================
c     Calculates turbine efficiency as a function 
c     of work and corrected mass flow
c
c     Inputs
c       dh       enthalpy change
c       mb       corrected mass flow
c       Nb       corrected speed
c       piD      design pressure ratio
c       mbD      design corrected mass flow
c       NbD      design corrected speed
c       ept0     turbine polytropic efficiency estimate
c       Tmap(.)  map constants
c
c     Outputs
c       eff       efficiency
c       eff_?     derivatives
c==============================================================
      implicit real (a-z)
      real Tmap(*)

      data eps / 1.0e-8 /

      pcon = Tmap(1)
      Ncon = Tmap(2)

      Trat     =  Tt       /(Tt + dh/cpt)
      Trat_Tt  = (1.0-Trat)/(Tt + dh/cpt)
      Trat_dh  =     -Trat /(Tt + dh/cpt) / cpt
      Trat_cpt =      Trat /(Tt + dh/cpt) * dh/cpt**2

      gex     = cpt/(Rt*ept0)
      gex_cpt = 1.0/(Rt*ept0)
      gex_Rt  = -gex/Rt

      prat = Trat**gex
      prat_Trat = gex*prat/Trat
      prat_gex  = prat*log(Trat)

      prat_Tt  = prat_Trat*Trat_Tt
      prat_dh  = prat_Trat*Trat_dh
      prat_cpt = prat_Trat*Trat_cpt + prat_gex*gex_cpt
      prat_Rt  =                      prat_gex*gex_Rt

      Nmb    = Nb*mb
      Nmb_Nb =    mb
      Nmb_mb = Nb

      NmbD = NbD*mbD

      ept      = ept0*(1.0 - pcon*(1.0-prat/piD)**2
     &                     - Ncon*(1.0-Nmb/NmbD)**2      )
      ept_prat = ept0*(      pcon*(1.0-prat/piD)*2.0/piD )
      ept_Nmb  = ept0*(      Ncon*(1.0-Nmb/NmbD)*2.0/NmbD)

      ept_dh  = ept_prat*prat_dh
      ept_mb  =                    ept_Nmb*Nmb_mb  
      ept_Nb  =                    ept_Nmb*Nmb_Nb   
      ept_Tt  = ept_prat*prat_Tt
      ept_cpt = ept_prat*prat_cpt
      ept_Rt  = ept_prat*prat_Rt 

      return
      end ! etmap



      subroutine Pimap(mb,Nb, piD,mbD,NbD, Cmap,
     &                 pi, pi_mb, pi_Nb )
c==============================================================
c     Calculates compressor or fan pressure ratio as a function 
c     of pressure ratio and corrected mass flow
c
c     Inputs
c       mb      corrected mass flow
c       Nb      corrected wheel speed
c       piD     design pressure ratio
c       mbD     design corrected mass flow
c       NbD     design corrected speed
c       Cmap(.) map constants
c
c     Outputs
c       pi      pressure ratio
c       pi_?    derivatives
c==============================================================
      implicit real (a-z)
      real Cmap(9)

c----              a     b     k     mo     da    c    d     C    D
c     data Cmapf / 3.50, 0.80, 0.03, 0.75, -0.50, 3.0, 6.0,  2.5, 15.0 /
      a  = Cmap(1)
      b  = Cmap(2)
      k  = Cmap(3)
c      mrato = Cmap(4)
c      da = Cmap(5)
c      c  = Cmap(6)
c      d  = Cmap(7)
c      CK = Cmap(8)
c      DK = Cmap(9)

c---- corrected mass flow / design corrected mass flow
      m    =  mb / mbD
      m_mb = 1.0 / mbD

c---- corrected speed / design corrected speed
      N    =  Nb / NbD
      N_Nb = 1.0 / NbD

      Nxb   = N**b
      Nxb_N = Nxb * b/N

      Nxab   = N**(a*b)
      Nxab_N = Nxab * a*b/N

      write(*,*) m, Nxb

      write(*,*) (m-Nxb)/k

      g   =  log(1.0-(m-Nxb)/k)
      g_N = 1.0/(1.0-(m-Nxb)/k) * (Nxb_N/k)
      g_m = 1.0/(1.0-(m-Nxb)/k) * ( -1.0/k)


      p   = Nxab   + 2.0*N*k*g
      p_N = Nxab_N + 2.0  *k*g + 2.0*N*k*g_N
      p_m =                      2.0*N*k*g_m

      pi   = 1.0 + (piD-1.0)*p
      pi_N =       (piD-1.0)*p_N
      pi_m =       (piD-1.0)*p_m

c      pi   = Nxb
c      pi_N = Nxb_N
c      pi_m = 0.


      pi_Nb = pi_N*N_Nb
      pi_mb = pi_m*m_mb

      return
      end ! Pimap
