
      subroutine tfsize(gee,M0,T0,p0,a0, M2,M25,      ! inputs
     &                Feng, Phiinl,Kinl,iBLIc,        !
     &                BPR,pif,pilc,pihc,
     &                pid,pib,pifn,pitn,              !
     &                Ttf,ifuel,etab,
     &                epf0,eplc0,ephc0,epht0,eplt0,
     &                pifK,epfK,
     &                mofft,Pofft,
     &                Tt9,pt9,
     &                epsl,epsh,
     &                icool,
     &                Mtexit,dTstrk,StA,efilm,tfilm,
     &                M4a,ruc,
     &                ncrowx, ncrow,
     &                epsrow,Tmrow,                   ! input or output
     &                TSFC,Fsp,hfuel,ff,mcore,        ! outputs
     &                Tt0 ,ht0 ,pt0 ,cpt0 ,Rt0 ,      !
     &                Tt18,ht18,pt18,cpt18,Rt18,
     &                Tt19,ht19,pt19,cpt19,Rt19,
     &                Tt2 ,ht2 ,pt2 ,cpt2 ,Rt2 ,
     &                Tt21,ht21,pt21,cpt21,Rt21,
     &                Tt25,ht25,pt25,cpt25,Rt25,
     &                Tt3 ,ht3 ,pt3 ,cpt3 ,Rt3 ,
     &                Tt4, ht4 ,pt4 ,cpt4 ,Rt4 ,      ! Tt4 is input
     &                Tt41,ht41,pt41,cpt41,Rt41,
     &                Tt45,ht45,pt45,cpt45,Rt45,
     &                Tt49,ht49,pt49,cpt49,Rt49,
     &                Tt5 ,ht5 ,pt5 ,cpt5 ,Rt5 ,
     &                Tt7 ,ht7 ,pt7 ,cpt7 ,Rt7 ,
     &                     u0 ,
     &                 T2 ,u2 ,p2 ,cp2 ,R2 ,A2 ,
     &                 T25,u25,p25,cp25,R25,A25,
     &                 T5 ,u5 ,p5 ,cp5 ,R5 ,A5 ,
     &                 T6 ,u6 ,p6 ,cp6 ,R6 ,A6 ,
     &                 T7 ,u7 ,p7 ,cp7 ,R7 ,A7 ,
     &                 T8 ,u8 ,p8 ,cp8 ,R8 ,A8 ,
     &                     u9 ,             A9 ,
     &                 epf ,eplc ,ephc ,epht ,eplt ,
     &                 etaf,etalc,etahc,etaht,etalt,
     &                 Lconv )

c----------------------------------------------------------------
c     Turbofan performance and sizing routine.
c
c     Calculation procedure follows that of Kerrebrock,
c     but the usual gas property formulas are replaced
c     by subroutine calls, which can therefore implement
c     more general gas models.  
c     In addition, a turbine cooling model is added.
c
c     The gas routines reside in the following source files:
c      gascalc.f  Routines for various processes 
c                 (compressor, turbine, combustor, etc)
c      gasfun.f   Routines for computing cp(T), h(t), sigma(T), R,
c                 called by the routines in gascalc.f
c
c     Input
c     -----
c     gee    gravity acceleration
c     M0     freestream Mach
c     T0     freestream temperature  [K]
c     p0     freestream pressure  [Pa]
c     M2     fan-face Mach number
c     M25    HPC-face Mach number
c     Feng   required net thrust  (PK_inl+PK_out-Phi_jet)/u0  =  sum( mdot u)
c     Phiinl inlet ingested dissipation
c     iBLIc  0=core in clear flow, 1=core sees Phiinl
c     BPR    bypass ratio  = mdot_fan/mdot_core
c     pif    fan      pressure ratio  ( = pt7 /pt2)
c     pilc   LP comp  pressure ratio  ( = pt25/pt2)
c     pihc   HP comp  pressure ratio  ( = pt3 /pt25)
c     pid    diffuser pressure ratio  ( = pt2 /pt0)
c     pib    burner   pressure ratio  ( = pt4 /pt3)
c     pifn   fan     nozzle pressure ratio  ( = pt7/pt2.1)
c     pitn   turbine nozzle pressure ratio  ( = pt5/pt4.9)
c     Ttf    fuel temperature entering combustor
c     ifuel  fuel index, see subroutine gasfun (in gasfun.f)
c     etab   combustor efficiency (fraction of fuel burned)
c     epf0   fan max polytropic efficiency
c     eplc0  LPC max polytropic efficiency
c     ephc0  HPC max polytropic efficiency
c     epht0  HPT max polytropic efficiency
c     eplt0  LPT max polytropic efficiency
c     pifK   fan efficiency FPR offset:    epolf = epf0 + epfK*(pif-pifK)
c     epfK   fan efficiency pif derivative
c
c     mofft   mass flow offtake at LPC discharge station 2.5
c     Pofft   low spool power offtake
c     Tt9    offtake air discharge total temperature
c     pt9    offtake air discharge total pressure
c     epsl   low  spool power loss fraction
c     epsh   high spool power loss fraction
c
c     icool   turbine cooling flag
c              0 = no cooling, ignore all cooling parameters below
c              1 = usual cooling, using passed-in fcool
c              2 = usual cooling, but set (and return) fcool from Tmetal
c     Mtexit  turbine blade-row exit Mach, for setting temperature drops
c     dTstrk  hot-streak temperature delta {K}, used only if icool=2
c     StA     area-weighted Stanton number    , used only if icool=2
c     M4a     effective Mach at cooling-flow outlet (start of mixing)
c     ruc     cooling-flow outlet velocity ratio, u/ue
c     ncrowx     dimension of epsrow array
c     ncrow      number of blade rows requiring cooling
c     epsrow(.)  input specified  cooling-flow bypass ratio if icool=1
c                output resulting cooling-flow bypass ratio if icool=2
c     Tmrow(.)   input specified  metal temperature  [K]    if icool=2
c                output resulting metal temperature  [K]    if icool=1
c
c     Output
c     ------
c     epsrow(.)  see above
c     Tmrow(.)   see above
c     TSFC   thrust specific fuel consumption = mdot_fuel g / F   [1/s]
c     Fsp    specific thrust  = F / (mdot u0) = F / ((1+BPR) mdot_core u0)
c     hfuel  fuel heating value   [J / kg K]
c     ff     fuel mass flow fraction  =  mdot_fuel / mdot_core
c     mcore  core mass flow = mdot_core  [kg/s]
c     A2     fan-face area [m^2]
c     A25    HPC-face area [m^2]
c     A5     core nozzle area [m^2]
c     A7     fan  nozzle area [m^2]
c     A6     core plume  area [m^2]
c     A8     fan  plume  area [m^2]
c     Tt?    total temperature
c     ht?    total complete enthalpy (includes heat of formation)
c     pt?    total pressure
c     cpt?   specific heat at stagnation temperature  (= dh/dT)
c     Rt?    gas constant  at stagnation conditions
c     T?     static temperature
c     u?     velocity
c     epf    fan polytropic efficiency
c     eplc   LPC polytropic efficiency
c     ephc   HPC polytropic efficiency
c     epht   HPT polytropic efficiency
c     eplt   LPT polytropic efficiency
c     etaf   fan overall efficiency
c     etalc  LPC overall efficiency
c     etahc  HPC overall efficiency
c     etaht  HPT overall efficiency
c     etalt  LPT overall efficiency
c     Lconv  T if convergence was successful, F otherwise
c
c     The "?" symbol denotes the station index:
c       0  freestream
c       18 fan face outside of casing BLs
c       19 fan face over LPC portion
c       2  fan face over fan portion
c       21 fan exit
c       25 LPC exit, HPC inlet
c       3  compressor exit
c       4  combustor exit before cooling air addition
c       41 turbine  inlet after  cooling air addition
c       45 HPT exit, LPT inlet
c       49 LPT exit
c       5  core nozzle
c       6  core flow downstream
c       7  fan nozzle
c       8  fan flow downstream
c----------------------------------------------------------------
c
      implicit real (a-z)
      integer iBLIc
      integer ifuel, icool, ncrowx, ncrow, ncrowy, ipass, npass
      real epsrow(*), Tmrow(*)
      logical Lconv

      parameter (ncrowy=8)
      real epsrow_Tt3(ncrowy),
     &     epsrow_Tt4(ncrowy),
     &     epsrow_Trr(ncrowy)

      integer icrow, i

c---- number of gas constituents
      integer n, nair
      parameter (n=6)
c
c---- mass fractions
      real alpha(n),   ! air
     &     beta(n),    ! fuel
     &     gamma(n),   ! combustion-caused change in air
     &     lambda(n),  ! combustion product gas
     &     lambdap(n)  ! combustion product gas with cooling flow added
c
      include 'tfmap.inc'

      include 'airfrac.inc'

      data pi / 3.1415926535897932384626 /

c---- fractional core mass flow convergence tolerance
      data toler / 1.0e-12 /

c---- mass offtake fraction update under-relaxation factor
      data rlxfo / 0.8 /

      mcore = 0.
      fo = 0.
      Pom = 0.

c---- overall pressure ratio
      pic = pilc*pihc

c---- number of air constitutents (all but fuel)
      nair = n-1

c===============================================================
c---- set combustion-change mass fractions gamma(i) for specified fuel
      call gasfuel(ifuel,gamma,n)

c---- apply combustor efficiency
      do i = 1, nair
        gamma(i) = etab*gamma(i)
      enddo
      gamma(n) = 1.0-etab
c
c===============================================================
c---- freestream static quantities
      call gassum(alpha,nair, T0, s0,dsdt,h0,dhdt,cp0,R0)
      gam0 = cp0/(cp0-R0)
c      a0 = sqrt(gam0*R0*T0)
      u0 = M0*a0

c===============================================================
c---- freestream total quantities
      hspec = h0 + 0.5*u0**2
      Tguess = T0*(1.0 + 0.5*(gam0-1.0)*M0**2)
      call gas_tset(alpha,nair, hspec,Tguess, Tt0)

      call gassum(alpha,nair, Tt0, st0,dsdt,ht0,dhdt,cpt0,Rt0)
      pt0 = p0 * exp((st0-s0)/Rt0)
      at0 = sqrt( Tt0*Rt0*cpt0/(cpt0-Rt0) )

c     pt0 = p0 *     (1.0 + 0.5*(gam0-1.0)*M0**2)**(gam0/(gam0-1.0))
c     at0 = a0 * sqrt(1.0 + 0.5*(gam0-1.0)*M0**2)
c
c===============================================================
c---- offtake plume flow 9
      Trat = (p0/pt9)**(Rt0/cpt0)
      if(Trat .lt. 1.0) then
       u9 = sqrt( 2.0*cpt0*Tt9 * (1.0-Trat) )
       rho9 = p0/(Rt0*Tt0*Trat)
      else
       u9 = 0.
       rho9 = p0/(Rt0*Tt0)
      endif

c===============================================================
c---- diffuser flow 0-2
      Tt18 = Tt0
      st18 = st0
      ht18 = ht0
      cpt18 = cpt0
      Rt18 = Rt0
      pt18 = pt0 * pid
c

cc    epf  = epf0
      epht = epht0
      eplt = eplt0


c---- initial guesses for station 2 and 1.9
      pt2 = pt18
      Tt2 = Tt18
      pt19 = pt18
      Tt19 = Tt18


      if(Kinl  .eq. 0.0 .and. 
     &   mofft .eq. 0.0 .and.
     &   Pofft .eq. 0.0       ) then
c----- single design pass will be sufficient
       npass = 1
      else
c----- must use multiple passes to converge pt2,pt19 from inlet defect Kinl,
c-     and to converge on offtake fractions fo, Pom
       npass = 60
      endif

      do 100 ipass = 1, npass

c===============================================================
c---- set fan inlet conditions corrected for BLI
      if(ipass.eq.1) then
cc      if(mcore .eq. 0.0) then
c----- don't know engine mass flow yet, so ignore any BLI mixing
       if(IBLIc .eq. 0) then
        sbfan = 0.
        sbcore = 0.
       else
        sbfan = 0.
        sbcore = 0.
       endif

      else
c----- account for inlet BLI defect via mass-averaged entropy
       a2sq = at0**2
     &      / (1.0 + 0.5*(gam0-1.0)*M2**2)

       if(IBLIc .eq. 0) then
c------ BL mixes with fan flow only
        mmix = BPR*mcore * sqrt(Tt2/Tt0) * pt0/pt2
        sbfan2  = Kinl*gam0/(mmix*a2sq)
        sbcore2 = 0.
       else
c------ BL mixes with fan + core flow
        mmix = BPR*mcore * sqrt(Tt2 /Tt0) * pt0/pt2
     &       +     mcore * sqrt(Tt19/Tt0) * pt0/pt19
        sbfan2  = Kinl*gam0/(mmix*a2sq)
        sbcore2 = sbfan
       endif
c----- update mixed-out entropies, with some underrelaxation       
       rlxs = 0.85
       sbfan  = sbfan  + rlxs*(sbfan2  - sbfan )
       sbcore = sbcore + rlxs*(sbcore2 - sbcore)

      endif

cc      write(*,*) 'tfsize', ipass, p0, sbfan, sbcore

c---- note: BL is assumed adiabatic, 
c-     so Tt2,ht2,st2,cpt2,Rt2  will not change due to BL ingestion
      Tt2 = Tt18
      ht2 = ht18
      st2 = st18
      cpt2 = cpt18
      Rt2 = Rt18
      pt2 = pt18 * exp(-sbfan)

      Tt19 = Tt18
      ht19 = ht18
      st19 = st18
      cpt19 = cpt18
      Rt19 = Rt18
      pt19 = pt18 * exp(-sbcore)

c===============================================================
c---- fan flow 2-7
      pifD = pif
      mbfD = 1.0
      mf = 1.0

      call ecmap(pif, mf, pifD,mbfD, Cmapf, epf0, pifK, epfK,
     &           epf, epf_pf, epf_mf )

      call gas_prat(alpha,nair,
     &             pt2 ,Tt2 ,ht2 ,st2 ,cpt2 ,Rt2 , pif,epf,
     &             pt21,Tt21,ht21,st21,cpt21,Rt21)

c---- fan duct nozzle total quantities
      pt7 = pt21 * pifn
      Tt7 = Tt21
      ht7 = ht21
      st7 = st21
      cpt7 = cpt21
      Rt7 = Rt21

cc      write(*,*) 'epf0 epf ', epf0, epf
c
c===============================================================
c---- LP compressor flow 2-25
      pilcD = pilc
      mblcD = 1.0
      ml = 1.0
      call ecmap(pilc,ml, pilcD,mblcD, Cmapl, eplc0, 1.0, 0.0,
     &           eplc, eplc_pl, eplc_ml )

      call gas_prat(alpha,nair,
     &               pt19,Tt19,ht19,st19,cpt19,Rt19, pilc,eplc,
     &               pt25,Tt25,ht25,st25,cpt25,Rt25 )

cc      write(*,*) 'eplc0 eplc', eplc0, eplc
    
c===============================================================
c---- HP compressor flow 25-3
      pihcD = pihc
      mbhcD = 1.0
      mh = 1.0
      call ecmap(pihc,mh, pihcD,mbhcD, Cmaph, ephc0, 1.0, 0.0,
     &           ephc, ephc_ph, ephc_mh )
      call gas_prat(alpha,nair,
     &              pt25,Tt25,ht25,st25,cpt25,Rt25, pihc,ephc,
     &              pt3 ,Tt3 ,ht3 ,st3 ,cpt3 ,Rt3  )

cc      write(*,*) 'ephc Tt3', ephc, Tt3

c===============================================================
c---- combustor flow 3-4   (ffb = mdot_fuel/mdot_burner)
      call gas_burn(alpha,beta,gamma,n,ifuel, Tt3,Ttf,Tt4, ffb,lambda)
      call gassum(lambda,nair, Tt4, st4,dsdt,ht4,dhdt,cpt4,Rt4)
      pt4 = pt3*pib
      gam4 = cpt4/(cpt4-Rt4)

cc       write(*,*) 'ffb    Tt4', ffb, Tt4
cc       write(*,12) 'pt0 pt2 pt3 pt4', p0, pt0, pt2, pt3, pt4


c     write(*,'(1x,8f9.5)') alpha,
c    & alpha(1)+alpha(2)+alpha(3)+alpha(4)+alpha(5)+alpha(6)
c     write(*,'(1x,8f9.5)') gamma,
c    & gamma(1)+gamma(2)+gamma(3)+gamma(4)+gamma(5)+gamma(6)
c     write(*,'(1x,8f9.5)') lambda,
c    & lambda(1)+lambda(2)+lambda(3)+lambda(4)+lambda(5)+lambda(6)

c===============================================================

      if(icool.eq.0) then
c----- no cooling air present... station 41 is same as 4
       ff = ffb*(1.0-fo)

       pt41 = pt4
       Tt41 = Tt4
       ht41 = ht4
       st41 = st4
       cpt41= cpt4
       Rt41 = Rt4
       do i = 1, nair
         lambdap(i) = lambda(i)
       enddo
c
c----------------------------------------------------------------
      else
c----- cooling air is present... calculate station 41

c----- hot-section temperature ratio for each blade row (for cooling model)
       gmi4 = Rt4/(cpt4-Rt4)
       Trrat = 1.0 / (1.0 + 0.5*gmi4*Mtexit**2)

       if(icool.eq.1) then
c------ epsrow(.) is assumed to be passed in.. calculate Tmrow(.)
        call Tmcalc(ncrowx, ncrow,
     &              Tmrow,Tt3,Tt4,dTstrk, Trrat,
     &              efilm,tfilm,StA, epsrow)
       else
c------ calculate cooling mass flow ratios epsrow(.) to get specified Tmrow(.)
        call mcool(ncrowx, ncrow,
     &             Tmrow,Tt3,Tt4,dTstrk, Trrat,
     &             efilm,tfilm,StA, 
     &             epsrow,epsrow_Tt3,epsrow_Tt4,epsrow_Trr)
       endif

c----- total cooling-flow fraction
       fc = 0.
       do icrow = 1, ncrow
         fc = fc + (1.0-fo)*epsrow(icrow)
       enddo

       if(fc .ge. 0.99) then
        write(*,*) 
        write(*,*) 'TFSIZE: Excessive cooling flow'
        write(*,*) 'mcool/mcore =', fc
        write(*,*) 'Tt3 Tt4 Tmetal', Tt3, Tt4, Tmrow(1)
        stop
       endif

c----- set ff = mdot_fuel/mdot_core = ffb * mdot_burner/mdot_core
       ff = (1.0-fo-fc)*ffb

       pt4a = pt4  
       Tt4a = Tt4  
       ht4a = ht4  
       st4a = st4  
       cpt4a= cpt4 
       Rt4a = Rt4  

c----- speed at start-of-mixing station 4a
       call gas_mach(lambda,nair,
     &               pt4a,Tt4a,ht4a,st4a,cpt4a,Rt4a, 0.0,M4a, 1.0,
     &               p4a ,T4a ,h4a ,s4a ,cp4a ,R4a)
       u4sq = max( 2.0*(ht4a-h4a) , 0.0 )
       u4a = sqrt(u4sq)
c       write(*,'(1x,8g14.6)') u4a, p4a, T4a, cp4a, h4a

cc----- use fixed-gamma form to get speed from Mach number
cc-      (good enough for cooling estimates)
c       gam4a = cpt4a/(cpt4a-rt4a)
c       trat = 1.0 + 0.5*(gam4a-1.0)*M4a**2
c       at4sq = gam4a*rt4a*Tt4a
c       u4sq = at4sq * M4a**2 / trat
c       u4a = sqrt(u4sq)
c       T4a = Tt4a / trat
c       p4a = pt4a / trat**(gam4a/(gam4a-1.0))
c       h4a = ht4a - 0.5*u4sq
c       cp4a = cpt4a
c       write(*,'(1x,8g14.6)') u4a, p4a, T4a, cp4a, h4a

c----- exit speed of cooling air at station 4a
       uc = ruc*u4a

c----- IGV exit mixing
       frac4 = (1.0-fo-fc+ff)/(1.0-fo+ff)
       fracm =         fc    /(1.0-fo+ff)

c----- mixed constituent fraction vector from mass equation
       do i = 1, nair
         lambdap(i) = frac4*lambda(i) + fracm*alpha(i)
       enddo

c----- mixed total enthalpy from enthalpy equation
       ht41 = frac4*ht4 + fracm*ht3

c----- total temperature from total enthalpy
       Tguess = frac4*Tt4 + fracm*Tt3
       call gas_tset(lambdap,nair, ht41,Tguess, Tt41)

c----- all total quantities (except for total pressure), from total temperature
       call gassum(lambdap,nair, Tt41, st41,dsdt,ht41,dhdt,cpt41,Rt41)

c----- mixed velocity from momentum equation, assuming constant static pressure
       p41 = p4a
       u41 = frac4*u4a + fracm*uc

c----- static temperature from static enthalpy
       h41 = ht41 - 0.5*u41**2
       Tguess = t4a + (h41-h4a)/cp4a
       call gas_tset(lambdap,nair, h41,Tguess, T41)

c----- all static quantities, from static temperature
       call gassum(lambdap,nair, T41, s41,dsdt,h41,dhdt,cp41,R41)

c----- all stagnation quantities, from total-static enthalpy difference
       dhb = ht41 - h41
       epi = 1.0
       call gas_delh(lambdap,nair,
     &                p41 ,T41 ,h41 ,s41 ,cp41 ,R41 ,dhb,epi,
     &                pt41,Tt41,ht41,st41,cpt41,Rt41)
c        write(*,*)
c        write(*,12) 'uc u4a u41 ', uc, u4a, u41
c        write(*,12) 'fc', fc, (epsrow(icrow), icrow=1, ncrow)
c        write(*,*)
c        write(*,12) 'ht3 ht4 ht4a ht41', ht3, ht4, ht4a, ht41
c        write(*,12) 'Tt3 Tt4 Tt4a Tt41', Tt3, Tt4, Tt4a, Tt41
c        write(*,12) 'pt3 pt4 pt4a pt41', pt3, pt4, pt4a, pt41
c 12    format(1x,a,8g12.4)
      endif

c===============================================================
c---- LPT and HPT work, per unit mass flow
      dhfac    = -(1.0-fo)/(1.0-fo+ff) / (1.0-epsh)
      dlfac    =     -1.0 /(1.0-fo+ff) / (1.0-epsl)

      dhht = (ht3 -ht25                       )*dhfac
      dhlt = (ht25-ht19 + BPR*(ht21-ht2) + Pom)*dlfac

c---- HPT flow
c     Trh =  Tt41/(Tt41 + dhht/cpt41)
c     gexh = cpt41/(Rt41*epht0)
c     pihtD = Trh**gexh

      epi = 1.0/epht
      call gas_delh(lambdap,nair,
     &              pt41,Tt41,ht41,st41,cpt41,Rt41, dhht,epi,
     &              pt45,Tt45,ht45,st45,cpt45,Rt45 )

c      write(*,*) 'pt45 Tt45', pt45, Tt45

c---- LPT flow
c     Trl =  Tt45/(Tt45 + dhlt/cpt45)
c     gexl = cpt45/(Rt45*eplt0)
c     piltD = Trl**gexl

      epi = 1.0/eplt
      call gas_delh(lambdap,nair,
     &              pt45,Tt45,ht45,st45,cpt45,Rt45, dhlt,epi,
     &              pt49,Tt49,ht49,st49,cpt49,Rt49 )
      pt5 = pt49 * pitn
      Tt5 = Tt49
      ht5 = ht49
      st5 = st49
      cpt5= cpt49
      Rt5 = Rt49

cc      write(*,*) 'pt5  Tt5 ', pt5, Tt5

c
c===============================================================
c---- fan plume flow 7-8, use alpha mass fraction (air)
      pt8 = pt7
      ht8 = ht7
      Tt8 = Tt7
      st8 = st7
      cpt8 = cpt7
      Rt8 = Rt7
      pratfn = p0/pt8
      call gas_prat(alpha,nair, pt8,Tt8,ht8,st8,cpt8,Rt8, pratfn,1.0,
     &                          p8 ,T8 ,h8, s8, cp8 ,R8 )
      if(h8 .ge. ht8) then
 1005  format(1x,a,2g13.5)
       write(*,*) 'TFSIZE: Negative fan plume velocity'
       write(*,1005) 'pt2  Tt2  =', pt2, Tt2 
       write(*,1005) 'pt8  Tt8  =', pt8, Tt8
       write(*,1005) 'p8   T8   =', p8 , T8
       write(*,1005) 'pif  BPR  =', pif, BPR
       Lconv = .false.

       u8 = 0.001*sqrt(R8*T8)
ccc    return
       stop
      else
       u8 = sqrt(2.0*(ht8-h8))
      endif
      rho8 = p8/(R8*T8)

c===============================================================
c---- core plume flow 5-6, use lambdap mass fraction (combustion products)
      pt6 = pt5
      ht6 = ht5
      Tt6 = Tt5
      st6 = st5
      cpt6 = cpt5
      Rt6 = Rt5
      prattn = p0/pt6
      call gas_prat(lambdap,nair, pt6,Tt6,ht6,st6,cpt6,Rt6, prattn,1.0,
     &                            p6 ,T6 ,h6, s6 ,cp6 ,R6 )
      if(h6 .ge. ht6) then
       write(*,*) 'TFSIZE: Negative core plume velocity'
       write(*,1005) 'pt2  Tt2  =', pt2 , Tt2 
       write(*,1005) 'pt3  Tt3  =', pt3 , Tt3 
       write(*,1005) 'pt4  Tt4  =', pt4 , Tt4 
       write(*,1005) 'pt41 Tt41 =', pt41, Tt41
       write(*,1005) 'pt6  Tt6  =', pt6 , Tt6
       write(*,1005) 'p6   T6   =', p6  , T6
       write(*,1005) 'pif  BPR  =', pif , BPR
       Lconv = .false.
ccc    return
       stop
 
       u6 = 0.001*sqrt(R6*T6)
      else
       u6 = sqrt(2.0*(ht6-h6))
      endif
      rho6 = p6/(R6*T6)

c      write(*,*) 'Pt6   u6 ', pt6, u6
c
c===============================================================
c---- effective fuel heating value, over states 3, 4  (just for info)
      cpa = 0.5*(cpt3+cpt4)
      hfuel = cpa*(Tt4-Tt3 + ffb*(Tt4-Ttf)) / (etab*ffb)

c---- effective fuel heating value, over states 3, 4.1  (just for info)
c      cpa = 0.5*(cpt3+cpt41)
c      hfuel = cpa*((1.0-fo)*(Tt41-Tt3) + ff*(Tt41-Ttf)) / (etab*ff)

c===============================================================
c---- size core mass flow

c---- store current values for better update, convergence checks
      mcold = mcore
      foold = fo

c---- added effective net thrust from dissipation in ingested streamtube
      if(u0.eq.0.0) then
       Finl = 0.
      else
       Finl = Phiinl/u0
      endif

c---- set core mass flow from specified effective net thrust
      mcore = (Feng - Finl)
     &      / ( (1.0-fo+ff)*u6 - u0 + BPR*(u8-u0) + fo*u9)

c---- corresponding new offtake mass flow fraction
      fonew = mofft/mcore
      dfo = fonew - foold

      fo = fo + rlxfo*dfo

c---- estimate better new mass flow, compensating for change in mass offtake
cc    mfac = min( 2.0 , 1.0 / (1.0 - dfo - dfo**2) )
      mfac = min( 2.0 , 1.0 / (1.0 - dfo) )
      mcore = mcore * mfac

c---- power offtake per mass flow
      Pom = Pofft / mcore

c      write(*,'(1x,i4, 9g12.4)')  
c     &  ipass, fo,ff,u6,u8,BPR, mcold, 1.0-mcold/mcore, mfac

c---- overall Fsp and TSFC
      Fsp = Feng / (u0*mcore*(1.0+BPR))
      if(Feng .le. 0.0) then
       TSFC = 0.0
      else
       TSFC = (gee*ff*mcore) / Feng
      endif

c===============================================================

      M8 = u8 / sqrt( cp8*R8/(cp8-R8) * T8 )
      if(M8 .lt. 1.0) then
c----- subsonic fan plume... fan nozzle flow is same as plume
       p7 = p8
       T7 = T8
       h7 = h8
       s7 = s8
       cp7= cp8
       R7 = R8
       u7 = u8
      else
c----- supersonic fan plume... fan nozzle is choked
       M7 = 1.0
       call gas_mach(alpha,nair,
     &               pt7,Tt7,ht7,st7,cpt7,Rt7,0.0,M7,1.0,
     &               p7 ,T7 ,h7 ,s7 ,cp7 ,R7 )
       u7 = sqrt(2.0*(ht7-h7))
      endif
      rho7 = p7/(R7*T7)

c---- size fan  nozzle and plume areas
      A7 = BPR*mcore/(rho7*u7)
      A8 = BPR*mcore/(rho8*u8)

c===============================================================
      M6 = u6 / sqrt( cp6*R6/(cp6-R6) * T6 )
c      write(*,*) 'u6,M6', u6,M6
      if(M6 .lt. 1.0) then
c----- subsonic core plume... core nozzle flow is same as plume
       p5 = p6
       T5 = T6
       h5 = h6
       s5 = s6
       cp5= cp6
       R5 = R6
       u5 = u6
      else
c----- supersonic core plume... core nozzle is choked
       M5 = 1.0
       call gas_mach(lambdap,nair,
     &               pt5,Tt5,ht5,st5,cpt5,Rt5,0.0,M5,1.0,
     &               p5 ,T5 ,h5 ,s5 ,cp5 ,R5 )
       u5 = sqrt(2.0*(ht5-h5))
      endif
      rho5 = p5/(R5*T5)

c---- size core nozzle and plume areas
      A5 = (1.0-fo+ff)*mcore/(rho5*u5)
      A6 = (1.0-fo+ff)*mcore/(rho6*u6)

      if(u9 .eq. 0.0) then
       A9 = 0.
      else
       A9 = fo*mcore/(rho9*u9)
      endif

c      write(*,*) '5', rho5, u5, A5
c      write(*,*) '6', rho6, u6, A6
c      write(*,*) '7', rho7, u7, A7
c      write(*,*) '8', rho8, u8, A8

c===============================================================
c---- size fan and compressor areas
      call gas_mach(alpha,nair,
     &             pt2,Tt2,ht2,st2,cpt2,Rt2, 0.0, M2 , 1.0,
     &             p2 ,T2 ,h2 ,s2 ,cp2 ,R2  )
      u2 = sqrt(2.0*(ht2-h2))
      rho2 = p2/(R2*T2)

      call gas_mach(alpha,nair,
     &             pt19,Tt19,ht19,st19,cpt19,Rt19, 0.0, M2 , 1.0,
     &             p19 ,T19 ,h19 ,s19 ,cp19 ,R19  )
      u19 = sqrt(2.0*(ht19-h19))
      rho19 = p19/(R19*T19)
      A2 = BPR*mcore/(rho2*u2) + mcore/(rho19*u19)

c      write(*,*) BPR*mcore, rho2, u2
c      write(*,*)     mcore, rho19, u19
c      write(*,*) A2

      call gas_mach(alpha,nair,
     &              pt25,Tt25,ht25,st25,cpt25,Rt25,0.0,M25,1.0,
     &              p25 ,T25 ,h25 ,s25 ,cp25 ,R25)
      u25 = sqrt(2.0*(ht25-h25))
      rho25 = p25/(R25*T25)
      A25   = (1.0-fo)*mcore / (rho25*u25)


      if(ipass .ge. 2) then
       dmfrac = 1.0 - mcold/mcore
c       write(*,*) ipass, dmfrac
       if(abs(dmfrac) .lt. toler) go to 110
      endif

 100  continue  ! with next core mass flow iteration

      if(npass .gt. 1) then
       write(*,*) 'TFSIZE: Convergence failed.  dm/m = ', dmfrac
      endif

 110  continue

c===============================================================
c---- calculate component efficiencies  (informative only -- not needed here)
      etaf  = 0.
      etalc = 0.
      etahc = 0.
      etaht = 0.
      etalt = 0.

c---- fan
      call gas_prat(alpha,nair,
     &             pt2  ,Tt2  ,ht2  ,st2  ,cpt2  ,Rt2  , pif,1.0,
     &             pt21i,Tt21i,ht21i,st21i,cpt21i,Rt21i )
      etaf = (ht21i-ht2)/(ht21-ht2)

c---- LP compressor
      call gas_prat(alpha,nair,
     &              pt19 ,Tt19 ,ht19 ,st19 ,cpt19 ,Rt19 , pilc,1.0,
     &              pt25i,Tt25i,ht25i,st25i,cpt25i,Rt25i)
      etalc = (ht25i-ht19)/(ht25-ht19)

c---- HP compressor
      call gas_prat(alpha,nair,
     &              pt25,Tt25,ht25,st25,cpt25,Rt25, pihc,1.0,
     &              pt3i,Tt3i,ht3i,st3i,cpt3i,Rt3i)
      etahc = (ht3i-ht25)/(ht3-ht25)

c---- HP turbine
      piht = pt45/pt41
      call gas_prat(lambdap,nair,
     &              pt41 ,Tt41 ,ht41 ,st41 ,cpt41 ,Rt41, piht,1.0,
     &              pt45i,Tt45i,ht45i,st45i,cpt45i,Rt45i)
      etaht = (ht45-ht41)/(ht45i-ht41)

c---- LP turbine
      pilt = pt49/pt45
      call gas_prat(lambdap,nair,
     &              pt45 ,Tt45 ,ht45 ,st45 ,cpt45 ,Rt45, pilt,1.0,
     &              pt49i,Tt49i,ht49i,st49i,cpt49i,Rt49i)
      etalt = (ht49-ht45)/(ht49i-ht45)

c      write(*,*) 'ef     ', etaf
c      write(*,*) 'elc ehc', etalc, etahc
c      write(*,*) 'eht elt', etaht, etalt

      Lconv = .true.
      return
      end ! tfsize
